@@ -5,9 +5,12 @@
 
 """A convenience wrapper for endpointscfg.py for appengine for python."""
 
+from __future__ import absolute_import
+from __future__ import unicode_literals
 import os
 
-import bootstrapping.bootstrapping as bootstrapping
+from bootstrapping import bootstrapping
+from googlecloudsdk.calliope import exceptions
 from googlecloudsdk.core.updater import update_manager
 
 
@@ -22,6 +25,10 @@ def main():
 
 
 if __name__ == '__main__':
-  bootstrapping.CommandStart('endpointscfg', component_id='core')
-  bootstrapping.CheckUpdates('endpointscfg')
-  main()
+  bootstrapping.DisallowPython3()
+  try:
+    bootstrapping.CommandStart('endpointscfg', component_id='core')
+    bootstrapping.CheckUpdates('endpointscfg')
+    main()
+  except Exception as e:  # pylint: disable=broad-except
+    exceptions.HandleError(e, 'endpointscfg')
