@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2014 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,6 +15,8 @@
 
 """Encapsulation of a docker image."""
 
+from __future__ import unicode_literals
+
 
 class Image(object):
   """Docker image that requires building and should be removed afterwards."""
