@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2014 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -16,6 +17,7 @@
 
 # The version of the docker API the docker-py client uses.
 # Warning: other versions might have different return values for some functions.
+from __future__ import unicode_literals
 DOCKER_PY_VERSION = 'auto'
 
 # Timeout of HTTP request from docker-py client to docker daemon, in seconds.
