@@ -24,10 +24,11 @@ import os
 import os.path
 import tarfile
 
-from googlecloudsdk.api_lib.storage import storage_util
+from googlecloudsdk.api_lib.cloudbuild import metric_names
 from googlecloudsdk.api_lib.util import apis as core_apis
 from googlecloudsdk.command_lib.util import gcloudignore
 from googlecloudsdk.core import log
+from googlecloudsdk.core import metrics
 from googlecloudsdk.core.util import files
 
 _IGNORED_FILE_MESSAGE = """\
@@ -89,6 +90,13 @@ class Snapshot(object):
         self.any_files_ignored = True
         continue
       for fname in filenames:
+        path = os.path.join(relpath, fname)
+        if os.path.islink(path) and not os.path.exists(path):
+          # The file is a broken symlink; ignore it.
+          log.info(
+              'Ignoring [{}] which is a symlink to non-existent path'.format(
+                  path))
+          continue
         # Join file paths with Linux path separators, avoiding ./ prefix.
         # GCB workers are Linux VMs so os.path.join produces incorrect output.
         fpath = '/'.join([relpath, fname]) if relpath != '.' else fname
@@ -142,28 +150,26 @@ class Snapshot(object):
     Returns:
       storage_v1_messages.Object, The written GCS object.
     """
-    with files.ChDir(self.src_dir):
-      with files.TemporaryDirectory() as tmp:
-        archive_path = os.path.join(tmp, 'file.tgz')
-        tf = self._MakeTarball(archive_path)
-        tf.close()
-        ignore_file_path = os.path.join(self.src_dir,
-                                        gcloudignore.IGNORE_FILE_NAME)
-        if self.any_files_ignored:
-          if os.path.exists(ignore_file_path):
-            log.info('Using gcloudignore file [{}]'.format(ignore_file_path))
-          else:
-            log.status.Print(_IGNORED_FILE_MESSAGE.format(
-                log_file=log.GetLogFilePath()))
-        log.status.write(
-            'Uploading tarball of [{src_dir}] to '
-            '[gs://{bucket}/{object}]\n'.format(
-                src_dir=self.src_dir,
-                bucket=gcs_object.bucket,
-                object=gcs_object.object,
-            ),
-        )
-        return storage_client.CopyFileToGCS(
-            storage_util.BucketReference.FromBucketUrl(gcs_object.bucket),
-            archive_path,
-            gcs_object.object)
+    with metrics.RecordDuration(metric_names.UPLOAD_SOURCE):
+      with files.ChDir(self.src_dir):
+        with files.TemporaryDirectory() as tmp:
+          archive_path = os.path.join(tmp, 'file.tgz')
+          tf = self._MakeTarball(archive_path)
+          tf.close()
+          ignore_file_path = os.path.join(self.src_dir,
+                                          gcloudignore.IGNORE_FILE_NAME)
+          if self.any_files_ignored:
+            if os.path.exists(ignore_file_path):
+              log.info('Using gcloudignore file [{}]'.format(ignore_file_path))
+            else:
+              log.status.Print(_IGNORED_FILE_MESSAGE.format(
+                  log_file=log.GetLogFilePath()))
+          log.status.write(
+              'Uploading tarball of [{src_dir}] to '
+              '[gs://{bucket}/{object}]\n'.format(
+                  src_dir=self.src_dir,
+                  bucket=gcs_object.bucket,
+                  object=gcs_object.object,
+              ),
+          )
+          return storage_client.CopyFileToGCS(archive_path, gcs_object)
