@@ -19,11 +19,13 @@ from __future__ import division
 from __future__ import unicode_literals
 
 from googlecloudsdk.calliope import exceptions as calliope_exceptions
+from googlecloudsdk.command_lib.compute.networks.subnets import flags
 import six
 
 
 def MakeSubnetworkUpdateRequest(client,
                                 subnet_ref,
+                                release_track='GA',
                                 enable_private_ip_google_access=None,
                                 add_secondary_ranges=None,
                                 remove_secondary_ranges=None,
@@ -32,12 +34,15 @@ def MakeSubnetworkUpdateRequest(client,
                                 flow_sampling=None,
                                 metadata=None,
                                 set_role_active=None,
-                                drain_timeout_seconds=None):
+                                drain_timeout_seconds=None,
+                                enable_private_ipv6_access=None,
+                                private_ipv6_google_access_type=None):
   """Make the appropriate update request for the args.
 
   Args:
     client: GCE API client
     subnet_ref: Reference to a subnetwork
+    release_track: The release track (ALPHA, BETA, GA)
     enable_private_ip_google_access: Enable/disable access to Google Cloud APIs
       from this subnet for instances without a public ip address.
     add_secondary_ranges: List of secondary IP ranges to add to the subnetwork
@@ -51,6 +56,10 @@ def MakeSubnetworkUpdateRequest(client,
     set_role_active: Updates the role of a BACKUP subnet to ACTIVE.
     drain_timeout_seconds: The maximum amount of time to drain connections from
       the active subnet to the backup subnet with set_role_active=True.
+    enable_private_ipv6_access: Enable/disable private IPv6 access for the
+      subnet.
+    private_ipv6_google_access_type: The private IPv6 google access type for the
+      VMs in this subnet.
 
   Returns:
     response, result of sending the update request for the subnetwork
@@ -107,43 +116,65 @@ def MakeSubnetworkUpdateRequest(client,
     with client.apitools_client.IncludeFields(cleared_fields):
       return client.MakeRequests(
           [CreateSubnetworkPatchRequest(client, subnet_ref, subnetwork)])
-  elif enable_flow_logs is not None:
-    subnetwork = client.MakeRequests(
-        [(client.apitools_client.subnetworks,
-          'Get', client.messages.ComputeSubnetworksGetRequest(
-              **subnet_ref.AsDict()))])[0]
+  elif (enable_flow_logs is not None or aggregation_interval is not None or
+        flow_sampling is not None or metadata is not None):
+    subnetwork = client.messages.Subnetwork()
+    original_subnetwork = client.MakeRequests([
+        (client.apitools_client.subnetworks, 'Get',
+         client.messages.ComputeSubnetworksGetRequest(**subnet_ref.AsDict()))
+    ])[0]
+    subnetwork.fingerprint = original_subnetwork.fingerprint
+
+    if release_track == 'GA':
+      subnetwork.enableFlowLogs = enable_flow_logs
+    elif release_track == 'BETA':
+      log_config = client.messages.SubnetworkLogConfig(enable=enable_flow_logs)
+      if aggregation_interval is not None:
+        log_config.aggregationInterval = flags.GetLoggingAggregationIntervalArg(
+            client.messages).GetEnumForChoice(aggregation_interval)
+      if flow_sampling is not None:
+        log_config.flowSampling = flow_sampling
+      if metadata is not None:
+        log_config.metadata = flags.GetLoggingMetadataArg(
+            client.messages).GetEnumForChoice(metadata)
+
+      subnetwork.logConfig = log_config
+    else:
+      log_config = client.messages.SubnetworkLogConfig(enable=enable_flow_logs)
+      if aggregation_interval is not None:
+        log_config.aggregationInterval = (
+            flags.GetLoggingAggregationIntervalArgAlpha(
+                client.messages).GetEnumForChoice(aggregation_interval))
+      if flow_sampling is not None:
+        log_config.flowSampling = flow_sampling
+      if metadata is not None:
+        log_config.metadata = flags.GetLoggingMetadataArgAlpha(
+            client.messages).GetEnumForChoice(metadata)
+
+      subnetwork.logConfig = log_config
 
-    subnetwork.enableFlowLogs = enable_flow_logs
     return client.MakeRequests(
         [CreateSubnetworkPatchRequest(client, subnet_ref, subnetwork)])
-  elif aggregation_interval is not None:
+  elif private_ipv6_google_access_type is not None:
     subnetwork = client.MakeRequests([
         (client.apitools_client.subnetworks, 'Get',
          client.messages.ComputeSubnetworksGetRequest(**subnet_ref.AsDict()))
     ])[0]
 
-    subnetwork.aggregationInterval = (
-        client.messages.Subnetwork.AggregationIntervalValueValuesEnum(
-            convert_to_enum(aggregation_interval)))
+    subnetwork.privateIpv6GoogleAccess = (
+        client.messages.Subnetwork.PrivateIpv6GoogleAccessValueValuesEnum(
+            ConvertPrivateIpv6GoogleAccess(
+                convert_to_enum(private_ipv6_google_access_type))))
     return client.MakeRequests(
         [CreateSubnetworkPatchRequest(client, subnet_ref, subnetwork)])
-  elif flow_sampling is not None:
+  elif enable_private_ipv6_access is not None:
     subnetwork = client.MakeRequests([
         (client.apitools_client.subnetworks, 'Get',
          client.messages.ComputeSubnetworksGetRequest(**subnet_ref.AsDict()))
     ])[0]
 
-    subnetwork.flowSampling = flow_sampling
-    return client.MakeRequests(
-        [CreateSubnetworkPatchRequest(client, subnet_ref, subnetwork)])
-  elif metadata is not None:
-    subnetwork = client.MakeRequests([
-        (client.apitools_client.subnetworks, 'Get',
-         client.messages.ComputeSubnetworksGetRequest(**subnet_ref.AsDict()))
-    ])[0]
-
-    subnetwork.metadata = client.messages.Subnetwork.MetadataValueValuesEnum(
-        convert_to_enum(metadata))
+    subnetwork.enablePrivateV6Access = enable_private_ipv6_access
+    subnetwork.privateIpv6GoogleAccess = None
     return client.MakeRequests(
         [CreateSubnetworkPatchRequest(client, subnet_ref, subnetwork)])
   elif set_role_active is not None:
@@ -172,3 +203,12 @@ def CreateSubnetworkPatchRequest(client, subnet_ref, subnetwork_resource):
       region=subnet_ref.region,
       subnetworkResource=subnetwork_resource)
   return (client.apitools_client.subnetworks, 'Patch', patch_request)
+
+
+def ConvertPrivateIpv6GoogleAccess(choice):
+  choices_to_enum = {
+      'DISABLE': 'DISABLE_GOOGLE_ACCESS',
+      'ENABLE_BIDIRECTIONAL_ACCESS': 'ENABLE_BIDIRECTIONAL_ACCESS_TO_GOOGLE',
+      'ENABLE_OUTBOUND_VM_ACCESS': 'ENABLE_OUTBOUND_VM_ACCESS_TO_GOOGLE'
+  }
+  return choices_to_enum.get(choice)
