@@ -47,3 +47,11 @@ class OperationTimeoutError(OperationError):
 
 class ParseError(Error):
   """File parsing error."""
+
+
+class FileUploadError(Error):
+  """File upload error."""
+
+
+class ValidationError(Error):
+  """Error while validating YAML against schema."""
