@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2014 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,13 +15,38 @@
 
 """Helper methods for importing record-sets."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 import re
 
 from dns import rdatatype
 from dns import zone
 from googlecloudsdk.api_lib.util import apis as core_apis
-from googlecloudsdk.calliope import exceptions
-import yaml
+from googlecloudsdk.core import exceptions
+from googlecloudsdk.core import yaml
+from googlecloudsdk.core.util import encoding
+
+
+class Error(exceptions.Error):
+  """Base exception for all import errors."""
+
+
+class RecordsFileNotFound(Error):
+  """The specified records file was not found."""
+
+
+class RecordsFileIsADirectory(Error):
+  """The specified records file is a directory."""
+
+
+class UnableToReadRecordsFile(Error):
+  """Unable to read record sets from the specified records file."""
+
+
+class ConflictingRecordsFound(Error):
+  """Conflicts found between records being imported and current records."""
 
 
 def _AddressTranslation(rdata, unused_origin):
@@ -46,7 +72,8 @@ def _CAATranslation(rdata, unused_origin):
   Returns:
     str, The translation of the given CAA rdata. See RFC 6844.
   """
-  return '{0} {1} {2}'.format(rdata.flags, rdata.tag, QuotedText(rdata.value))
+  return '{0} {1} {2}'.format(rdata.flags, encoding.Decode(rdata.tag),
+                              QuotedText(rdata.value))
 
 
 def _MXTranslation(rdata, origin):
@@ -129,6 +156,7 @@ def QuotedText(text):
     necessary, please look at the TXT section at:
     https://cloud.google.com/dns/what-is-cloud-dns#supported_record_types.
   """
+  text = encoding.Decode(text)
   if text.startswith('"') and text.endswith('"'):
     # Nothing to do if already escaped.
     return text
@@ -296,7 +324,7 @@ def RecordSetsFromYamlFile(yaml_file, api_version='v1'):
   record_sets = {}
   messages = core_apis.GetMessagesModule('dns', api_version)
 
-  yaml_record_sets = yaml.safe_load_all(yaml_file)
+  yaml_record_sets = yaml.load_all(yaml_file)
   for yaml_record_set in yaml_record_sets:
     rdata_type = rdatatype.from_text(yaml_record_set['type'])
     if GetRdataTranslation(rdata_type) is None:
@@ -417,8 +445,8 @@ def NextSOARecordSet(soa_record_set, api_version='v1'):
   next_soa_record_set = _RecordSetCopy(soa_record_set, api_version=api_version)
   rdata_parts = soa_record_set.rrdatas[0].split()
   # Increment the 32 bit serial number by one and wrap around if needed.
-  rdata_parts[2] = str((long(rdata_parts[2]) + 1) % (1 << 32))
-  next_soa_record_set.rrdatas[0] = u' '.join(rdata_parts)
+  rdata_parts[2] = str((int(rdata_parts[2]) + 1) % (1 << 32))
+  next_soa_record_set.rrdatas[0] = ' '.join(rdata_parts)
   return next_soa_record_set
 
 
@@ -439,7 +467,7 @@ def IsOnlySOAIncrement(change, api_version='v1'):
 
 
 def _NameAndType(record):
-  return u'{0} {1}'.format(record.name, record.type)
+  return '{0} {1}'.format(record.name, record.type)
 
 
 def ComputeChange(current, to_be_imported, replace_all=False,
@@ -456,7 +484,7 @@ def ComputeChange(current, to_be_imported, replace_all=False,
     api_version: [str], the api version to use for creating the records.
 
   Raises:
-    ToolException: If conflicting CNAME records are found.
+    ConflictingRecordsFound: If conflicting records are found.
 
   Returns:
     A Change that describes the actions required to import the given
@@ -472,8 +500,8 @@ def ComputeChange(current, to_be_imported, replace_all=False,
 
   intersecting_keys = current_keys.intersection(keys_to_be_imported)
   if not replace_all and intersecting_keys:
-    raise exceptions.ToolException(
-        'Conflicting records for the following (name type): {0}'.format(
+    raise ConflictingRecordsFound(
+        'The following records (name type) already exist: {0}'.format(
             [_NameAndType(current[key]) for key in sorted(intersecting_keys)]))
 
   for key in intersecting_keys:
