@@ -20,7 +20,6 @@ from __future__ import division
 from __future__ import unicode_literals
 
 import datetime
-import os
 import posixpath
 import random
 import re
@@ -32,6 +31,7 @@ from googlecloudsdk.api_lib.firebase.test import util as util
 from googlecloudsdk.api_lib.storage import storage_util
 from googlecloudsdk.calliope import arg_parsers
 from googlecloudsdk.calliope import exceptions
+from googlecloudsdk.core.util import files
 import six
 
 
@@ -75,6 +75,8 @@ def ValidateArgFromFile(arg_internal_name, arg_value):
 POSITIVE_INT_PARSER = arg_parsers.BoundedInt(1, sys.maxsize)
 NONNEGATIVE_INT_PARSER = arg_parsers.BoundedInt(0, sys.maxsize)
 TIMEOUT_PARSER = arg_parsers.Duration(lower_bound='1m', upper_bound='6h')
+TIMEOUT_PARSER_US = arg_parsers.Duration(
+    lower_bound='1m', upper_bound='6h', parsed_unit='us')
 ORIENTATION_LIST = ['portrait', 'landscape', 'default']
 
 
@@ -132,10 +134,16 @@ def _ValidateDuration(arg_internal_name, arg_value):
   raise test_exceptions.InvalidArgException(arg_internal_name, arg_value)
 
 
-def _ValidateInteger(arg_internal_name, arg_value):
-  """Validates an argument which should have any integer value."""
-  if isinstance(arg_value, int):
-    return arg_value
+def _ValidateDurationUs(arg_internal_name, arg_value):
+  """Validates an argument which should have Duration value in microseconds."""
+  try:
+    if isinstance(arg_value, six.string_types):
+      return TIMEOUT_PARSER_US(arg_value)
+    elif isinstance(arg_value, int):
+      return TIMEOUT_PARSER_US(str(arg_value))
+  except arg_parsers.ArgumentTypeError as e:
+    raise test_exceptions.InvalidArgException(arg_internal_name,
+                                              six.text_type(e))
   raise test_exceptions.InvalidArgException(arg_internal_name, arg_value)
 
 
@@ -259,14 +267,10 @@ _FILE_ARG_VALIDATORS = {
     'device_ids': ValidateStringList,
     'directories_to_pull': ValidateStringList,
     'environment_variables': _ValidateKeyValueStringPairs,
-    'event_count': _ValidatePositiveInteger,
-    'event_delay': _ValidateNonNegativeInteger,
     'locales': ValidateStringList,
     'orientations': _ValidateOrientationList,
     'obb_files': _ValidateObbFileList,
-    'random_seed': _ValidateInteger,
-    'max_steps': _ValidateNonNegativeInteger,
-    'max_depth': _ValidatePositiveInteger,
+    'num_flaky_test_attempts': _ValidateNonNegativeInteger,
     'os_version_ids': ValidateStringList,
     'other_files': _ValidateKeyValueStringPairs,
     'performance_metrics': _ValidateBool,
@@ -276,6 +280,7 @@ _FILE_ARG_VALIDATORS = {
     'scenario_numbers': _ValidatePositiveIntList,
     'test_targets': ValidateStringList,
     'timeout': _ValidateDuration,
+    'timeout_us': _ValidateDurationUs,
     'use_orchestrator': _ValidateBool,
 }
 
@@ -389,7 +394,7 @@ def _GenerateUniqueGcsObjectName():
 
 
 def ValidateOsVersions(args, catalog_mgr):
-  """Validate os-version-ids strings against the TestingEnvironmentCatalog.
+  """Validate os-version-ids strings against the TestEnvironmentCatalog.
 
   Also allow users to alternatively specify OS version strings (e.g. '5.1.x')
   but translate them here to their corresponding version IDs (e.g. '22').
@@ -399,7 +404,7 @@ def ValidateOsVersions(args, catalog_mgr):
     args: an argparse namespace. All the arguments that were provided to the
       command invocation (i.e. group and command arguments combined).
     catalog_mgr: an AndroidCatalogManager object for working with the Android
-      TestingEnvironmentCatalog.
+      TestEnvironmentCatalog.
   """
   if not args.os_version_ids:
     return
@@ -410,6 +415,12 @@ def ValidateOsVersions(args, catalog_mgr):
   args.os_version_ids = sorted(validated_versions)
 
 
+def ValidateXcodeVersion(args, catalog_mgr):
+  """Validates an Xcode version string against the TestEnvironmentCatalog."""
+  if args.xcode_version:
+    catalog_mgr.ValidateXcodeVersion(args.xcode_version)
+
+
 _OBB_FILE_REGEX = re.compile(
     r'(.*[\\/:])?(main|patch)\.\d+(\.[a-zA-Z]\w*)+\.obb$')
 
@@ -427,7 +438,7 @@ def NormalizeAndValidateObbFileNames(obb_files):
     obb_files[:] = [
         obb_file if not obb_file or
         obb_file.startswith(storage_util.GSUTIL_BUCKET_PREFIX) else
-        os.path.expanduser(obb_file) for obb_file in obb_files
+        files.ExpandHomeDir(obb_file) for obb_file in obb_files
     ]
   for obb_file in (obb_files or []):
     if not _OBB_FILE_REGEX.match(obb_file):
@@ -443,11 +454,11 @@ def ValidateRoboDirectivesList(args):
   duplicates = set()
   for key, value in six.iteritems((args.robo_directives or {})):
     (action_type, resource_name) = util.ParseRoboDirectiveKey(key)
-    if action_type == 'click' and value:
+    if action_type in ['click', 'ignore'] and value:
       raise test_exceptions.InvalidArgException(
           'robo_directives',
-          'Input value not allowed for click action: [{0}={1}]'.format(
-              key, value))
+          'Input value not allowed for click or ignore actions: [{0}={1}]'
+          .format(key, value))
 
     # Validate resource_name validity
     if not resource_name:
