@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2017 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -41,6 +42,9 @@ Pythonicness of the Transform*() methods:
 """
 
 
+from __future__ import unicode_literals
+
+
 def TransformTrigger(r, undefined=''):
   """Returns textual information about functions trigger.
 
