@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2017 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,6 +14,10 @@
 # limitations under the License.
 """Wrapper for user-visible error exceptions to raise in the CLI."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.core import exceptions
 
 
@@ -22,3 +27,7 @@ class ResourceManagerError(exceptions.Error):
 
 class ResourceManagerInputFileError(ResourceManagerError):
   """More specific errors for failure to read input files."""
+
+
+class ArgumentError(ResourceManagerError):
+  """Command argument error."""
