@@ -30,7 +30,7 @@ from functools import wraps
 import os
 import sys
 
-from googlecloudsdk.api_lib.util import exceptions as api_exceptions  # pytype: disable=import-error
+from googlecloudsdk.api_lib.util import exceptions as api_exceptions
 from googlecloudsdk.core import exceptions as core_exceptions
 from googlecloudsdk.core import log
 from googlecloudsdk.core import properties
@@ -65,12 +65,12 @@ def NewErrorFromCurrentException(error, *args):
     file_logger = log.file_only_logger
     file_logger.error('Handling the source of a tool exception, '
                       'original details follow.')
-    file_logger.exception(current_exception)  # pytype: disable=wrong-arg-types
+    file_logger.exception(current_exception)
 
   if args:
     return error(*args)
   elif current_exception:
-    return error(*current_exception.args)  # pytype: disable=attribute-error
+    return error(*current_exception.args)
   return error('An unknown error has occurred')
 
 
