@@ -62,6 +62,8 @@ from __future__ import unicode_literals
 
 import abc
 import argparse
+import collections
+import io
 import itertools
 import os
 import re
@@ -71,15 +73,23 @@ from googlecloudsdk.calliope import arg_parsers
 from googlecloudsdk.calliope import base  # pylint: disable=unused-import
 from googlecloudsdk.calliope import parser_arguments
 from googlecloudsdk.calliope import parser_errors
+from googlecloudsdk.calliope import suggest_commands
 from googlecloudsdk.calliope import usage_text
 from googlecloudsdk.core import config
 from googlecloudsdk.core import log
 from googlecloudsdk.core import metrics
 from googlecloudsdk.core.console import console_attr
+from googlecloudsdk.core.console import console_io
+from googlecloudsdk.core.document_renderers import render_document
 from googlecloudsdk.core.updater import update_manager
 import six
 
 
+_HELP_SEARCH_HINT = """\
+To search the help text of gcloud commands, run:
+  gcloud help -- SEARCH_TERMS"""
+
+
 class Namespace(argparse.Namespace):
   """A custom subclass for parsed args.
 
@@ -93,7 +103,7 @@ class Namespace(argparse.Namespace):
   """
 
   def __init__(self, **kwargs):
-    self._deepest_parser = None  # type: ArgumentParser
+    self._deepest_parser = None
     self._parsers = []
     self._specified_args = {}
     super(Namespace, self).__init__(**kwargs)
@@ -306,6 +316,14 @@ class _ErrorContext(object):
     self.message = re.sub(r"\bu'", "'", message)
     self.parser = parser
     self.error = error
+    self.flags_locations = parser.flags_locations
+
+  def AddLocations(self, arg):
+    """Adds locaton info from context for arg if specified."""
+    locations = self.flags_locations.get(arg)
+    if locations:
+      arg = '{} ({})'.format(arg, ','.join(sorted(locations)))
+    return arg
 
 
 class ArgumentParser(argparse.ArgumentParser):
@@ -314,6 +332,7 @@ class ArgumentParser(argparse.ArgumentParser):
   This overrides the default argparse parser.
 
   Attributes:
+    _args: Original argv passed to argparse.
     _calliope_command: base._Command, The Calliope command or group for this
       parser.
     _error_context: The most recent self.error() method _ErrorContext.
@@ -329,6 +348,8 @@ class ArgumentParser(argparse.ArgumentParser):
       _get_values().
   """
 
+  _args = None
+
   def __init__(self, *args, **kwargs):
     self._calliope_command = kwargs.pop('calliope_command')
     # Would rather isinstance(self._calliope_command, CommandGroup) here but
@@ -336,8 +357,9 @@ class ArgumentParser(argparse.ArgumentParser):
     self._is_group = hasattr(self._calliope_command, 'commands')
     self._remainder_action = None
     self._specified_args = {}
-    self._error_context = None  # type: _ErrorContext
+    self._error_context = None
     self._probe_error = False
+    self.flags_locations = collections.defaultdict(set)
     super(ArgumentParser, self).__init__(*args, **kwargs)
 
   def _Error(self, error):
@@ -373,6 +395,17 @@ class ArgumentParser(argparse.ArgumentParser):
     """Returns the scrubbed names for args specified on the command line."""
     return sorted(self._specified_args.values())
 
+  def _AddLocations(self, arg, value=None):
+    """Adds file and line info from context for arg if specified."""
+    if value and '=' not in arg:
+      argval = '{}={}'.format(arg, value)
+    else:
+      argval = arg
+    locations = self.flags_locations.get(argval)
+    if locations:
+      arg = '{} ({})'.format(argval, ','.join(sorted(locations)))
+    return arg
+
   def _Suggest(self, unknown_args):
     """Error out with a suggestion based on text distance for each unknown."""
     messages = []
@@ -391,12 +424,17 @@ class ArgumentParser(argparse.ArgumentParser):
     suggestions = {}
     for arg in unknown_args:
       # Only do this for flag names.
+      if not isinstance(arg, six.string_types):
+        continue
       if arg.startswith('--'):
         # Strip the flag value if any from the suggestion.
         flag = arg.split('=')[0]
         suggestion = suggester.GetSuggestion(flag)
+        arg = self._AddLocations(arg)
       else:
         suggestion = None
+      if arg in messages:
+        continue
       if suggestion:
         suggestions[arg] = suggestion
         messages.append(arg + " (did you mean '{0}'?)".format(suggestion))
@@ -404,8 +442,14 @@ class ArgumentParser(argparse.ArgumentParser):
         messages.append(arg)
 
     # If there is a single arg, put it on the same line.  If there are multiple
-    # add each on it's own line for better clarity.
-    separator = '\n  ' if len(messages) > 1 else ' '
+    # add each on its own line for better clarity.
+    if len(messages) > 1:
+      separator, prefix = '\n  ', ''
+    else:
+      separator, prefix = ' ', '\n\n'
+    # Always add a final message suggesting gcloud help. Set off with new line
+    # if this will be the only new line.
+    messages.append('{}{}'.format(prefix, _HELP_SEARCH_HINT))
     self._Error(parser_errors.UnrecognizedArgumentsError(
         'unrecognized arguments:{0}{1}'.format(
             separator, separator.join(messages)),
@@ -423,13 +467,13 @@ class ArgumentParser(argparse.ArgumentParser):
 
     Args:
       args: The list of command line args.
-      namespace: The argparse namespace.
+      namespace: The parsed args namespace.
       wrapper: Calls the parse_known_args() wrapper if True, otherwise the
         wrapped argparse parse_known_args().
 
     Returns:
-      namespace: The parse arg namespace.
-      unknown_args: True if there were unknown args.
+      namespace: The parsed arg namespace.
+      unknown_args: The list of unknown args.
       error_context: The _ErrorContext if there was an error, None otherwise.
     """
     self._error_context = None
@@ -438,6 +482,8 @@ class ArgumentParser(argparse.ArgumentParser):
         parser.parse_known_args(args, namespace) or (namespace, []))
     error_context = self._error_context
     self._error_context = None
+    if not unknown_args and hasattr(parser, 'flags_locations'):
+      parser.flags_locations = collections.defaultdict(set)
     return namespace, unknown_args, error_context
 
   def _DeduceBetterError(self, context, args, namespace):
@@ -463,7 +509,10 @@ class ArgumentParser(argparse.ArgumentParser):
         skip = False
         required.append(arg)
         continue
-      if not arg.startswith('-'):
+      try:
+        if not arg.startswith('-'):
+          break
+      except AttributeError:
         break
       _, _, error_context = self._ParseKnownArgs(required + [arg], namespace)
       if not error_context:
@@ -476,6 +525,7 @@ class ArgumentParser(argparse.ArgumentParser):
         context = error_context
         break
     self._probe_error = False
+    context.error.argument = context.AddLocations(context.error.argument)
     context.parser.error(context=context, reproduce=True)
 
   @staticmethod
@@ -484,8 +534,8 @@ class ArgumentParser(argparse.ArgumentParser):
     return set([getattr(a, 'dest', a) for a in args])
 
   # pylint: disable=invalid-name, argparse style
-  def validate_specified_args(self, ai, specified_args, is_required=True,
-                              top=True):
+  def validate_specified_args(self, ai, specified_args, namespace,
+                              is_required=True, top=True):
     """Validate specified args against the arg group constraints.
 
     Each group may be mutually exclusive and/or required. Each argument may be
@@ -495,6 +545,7 @@ class ArgumentParser(argparse.ArgumentParser):
       ai: ArgumentInterceptor, The argument interceptor containing the
         ai.arguments argument group.
       specified_args: set, The dests of the specified args.
+      namespace: object, The parsed args namespace.
       is_required: bool, True if all containing groups are required.
       top: bool, True if ai.arguments is the top level group.
 
@@ -507,6 +558,7 @@ class ArgumentParser(argparse.ArgumentParser):
     Returns:
       True if the subgroup was specified.
     """
+    # TODO(b/120132521) Replace and eliminate argparse extensions
     also_optional = []  # The optional args in group that were not specified.
     have_optional = []  # The specified optional (not required) args.
     have_required = []  # The specified required args.
@@ -516,6 +568,7 @@ class ArgumentParser(argparse.ArgumentParser):
         arg_was_specified = self.validate_specified_args(
             arg,
             specified_args,
+            namespace,
             is_required=is_required and arg.is_required,
             top=False)
       else:
@@ -563,6 +616,14 @@ class ArgumentParser(argparse.ArgumentParser):
                 sorted(have_required + have_optional,
                        key=usage_text.GetArgSortKey)[0],
                 value=False, hidden=True, top=top)
+            try:
+              flag = namespace.GetFlagArgument(argument)
+            except parser_errors.UnknownDestinationException:
+              flag = None
+            if flag:
+              value = namespace.GetValue(flag.dest)
+              if not isinstance(value, (bool, dict, list)):
+                argument = self._AddLocations(argument, value)
           else:
             argument = None
           self._Error(parser_errors.RequiredMutexError(
@@ -613,8 +674,24 @@ class ArgumentParser(argparse.ArgumentParser):
         self._actions.append(self._remainder_action)
     return (namespace, unknown_args)
 
+  @classmethod
+  def _SaveOriginalArgs(cls, original_args):
+    if original_args:
+      cls._args = original_args[:]
+    else:
+      cls._args = None
+
+  @classmethod
+  def _ClearOriginalArgs(cls):
+    cls._args = None
+
+  @classmethod
+  def _GetOriginalArgs(cls):
+    return cls._args
+
   def parse_args(self, args=None, namespace=None):
     """Overrides argparse.ArgumentParser's .parse_args method."""
+    self._SaveOriginalArgs(args)
     namespace, unknown_args, _ = self._ParseKnownArgs(args, namespace)
 
     # pylint:disable=protected-access
@@ -628,7 +705,8 @@ class ArgumentParser(argparse.ArgumentParser):
       for parser in namespace._parsers:
         try:
           # pylint: disable=protected-access
-          parser.validate_specified_args(parser.ai, namespace._specified_args)
+          parser.validate_specified_args(
+              parser.ai, namespace._specified_args, namespace)
         except argparse.ArgumentError as e:
           deepest_parser._Error(e)
       if namespace._GetCommand().is_group:
@@ -732,23 +810,19 @@ class ArgumentParser(argparse.ArgumentParser):
           extra_path_arg=arg,
           suggestions=existing_alternatives))
 
-    # See if the spelling was close to something else that exists here.
+    # If we are dealing with flags, see if the spelling was close to something
+    # else that exists here.
+    suggestion = None
     choices = sorted(action.choices)
-    suggester = usage_text.TextChoiceSuggester(choices)
-    suggester.AddSynonyms()
-    if is_subparser:
-      # Add command suggestions if the group registered any.
-      cmd_suggestions = self._calliope_command._common_type.CommandSuggestions()
-      cli_name = self._calliope_command.GetPath()[0]
-      for cmd, suggestion in six.iteritems(cmd_suggestions):
-        suggester.AddAliases([cmd], cli_name + ' ' + suggestion)
-    suggestion = suggester.GetSuggestion(arg)
-    if suggestion:
-      message += " Did you mean '{0}'?".format(suggestion)
-    elif not is_subparser:
-      # Command group choices will be displayed in the usage message.
-      message += '\n\nValid choices are [{0}].'.format(
-          ', '.join([six.text_type(c) for c in choices]))
+    if not is_subparser:
+      suggester = usage_text.TextChoiceSuggester(choices)
+      suggestion = suggester.GetSuggestion(arg)
+      if suggestion:
+        message += " Did you mean '{0}'?".format(suggestion)
+      else:
+        # Command group choices will be displayed in the usage message.
+        message += '\n\nValid choices are [{0}].'.format(
+            ', '.join([six.text_type(c) for c in choices]))
 
     # Log to analytics the attempt to execute a command.
     # We don't know if the user entered 'value' is a mistyped command or
@@ -894,6 +968,14 @@ class ArgumentParser(argparse.ArgumentParser):
         self._SetErrorContext(context or _ErrorContext(message, parser, error))
         return
 
+    # Add file/line info if specified.
+
+    prefix = 'argument '
+    if context and message.startswith(prefix):
+      parts = message.split(':', 1)
+      arg = context.AddLocations(parts[0][len(prefix):])
+      message = '{}{}:{}'.format(prefix, arg, parts[1])
+
     # Ignore errors better handled by validate_specified_args().
     if '_ARGCOMPLETE' not in os.environ:
       if re.search('too few arguments', message):
@@ -902,8 +984,6 @@ class ArgumentParser(argparse.ArgumentParser):
           not re.search('in dict arg but not provided', message)):
         return
 
-    parser.ReportErrorMetrics(error, message)
-
     # No need to output help/usage text if we are in completion mode. However,
     # we do need to populate group/command level choices. These choices are not
     # loaded when there is a parser error since we do lazy loading.
@@ -915,10 +995,53 @@ class ArgumentParser(argparse.ArgumentParser):
       message = console_attr.SafeText(message)
       log.error('({prog}) {message}'.format(prog=self.prog, message=message))
       # multi-line message means hints already added, no need for usage.
-      # pylint:disable=protected-access
+      # pylint: disable=protected-access
       if '\n' not in message:
-        argparse._sys.stderr.write(self._calliope_command.GetUsage())
+        # Provide "Maybe you meant" suggestions if we are dealing with an
+        # invalid command.
+        suggestions = None
+        if 'Invalid choice' in message:
+          suggestions = suggest_commands.GetCommandSuggestions(
+              self._GetOriginalArgs())
+          self._ClearOriginalArgs()
+        if suggestions:
+          argparse._sys.stderr.write(
+              '\n  '.join(['Maybe you meant:'] + suggestions) + '\n')
+          argparse._sys.stderr.write('\n' + _HELP_SEARCH_HINT + '\n')
+          error.error_extra_info = {
+              'suggestions': suggestions,
+              'total_suggestions': len(suggestions),
+              'total_unrecognized': 1,
+          }
+        # Otherwise print out usage string.
+        elif 'Command name argument expected.' == message:
+          usage_string = self._calliope_command.GetCategoricalUsage()
+          # The next if clause is executed if there were no categories to
+          # display.
+          uncategorized_usage = False
+          if not usage_string:
+            uncategorized_usage = True
+            usage_string = self._calliope_command.GetUncategorizedUsage()
+          interactive = False
+          if not uncategorized_usage:
+            interactive = console_io.IsInteractive(error=True)
+          if interactive:
+            out = io.StringIO()
+            out.write('{message}\n'.format(message=message))
+          else:
+            out = argparse._sys.stderr
+          out.write('\n')
+          render_document.RenderDocument(
+              fin=io.StringIO(usage_string), out=out)
+          if uncategorized_usage:
+            out.write(self._calliope_command.GetHelpHint())
+          if interactive:
+            console_io.More(out.getvalue(), out=argparse._sys.stderr)
+        else:
+          usage_string = self._calliope_command.GetUsage()
+          argparse._sys.stderr.write(usage_string)
 
+    parser.ReportErrorMetrics(error, message)
     self.exit(2, exception=error)
 
   def exit(self, status=0, message=None, exception=None):
@@ -945,6 +1068,9 @@ class ArgumentParser(argparse.ArgumentParser):
     Returns:
       The normal return value of argparse.ArgumentParser._parse_optional.
     """
+    if not isinstance(arg_string, six.string_types):
+      # Flag value injected by --flags-file.
+      return None
     positional_actions = self._get_positional_actions()
     option_tuple = super(ArgumentParser, self)._parse_optional(arg_string)
     # If parse_optional finds an action for this arg_string, use that option.
@@ -1129,7 +1255,7 @@ class DynamicPositionalAction(six.with_metaclass(abc.ABCMeta,
 
   def __call__(self, parser, namespace, values, option_string=None):
     choice = values[0]
-    args = self.GenerateArgs(namespace, choice)  # type: list[base.Argument]
+    args = self.GenerateArgs(namespace, choice)
     sub_parser = self._name_parser_map[choice]
 
     # This is tricky. When we create a new parser above, that parser does not
@@ -1201,3 +1327,4 @@ class DynamicPositionalAction(six.with_metaclass(abc.ABCMeta,
       # argument lookup down the road.
       # for _, arg in args.iteritems():
       #   arg.RemoveFromParser(ai)
+
