@@ -20,7 +20,7 @@ from __future__ import division
 from __future__ import unicode_literals
 
 import argparse
-from collections import OrderedDict
+import collections
 import copy
 import difflib
 import io
@@ -39,6 +39,8 @@ import six
 
 LINE_WIDTH = 80
 HELP_INDENT = 25
+# Used to offset hanging indentation of arg choices in markdown.
+_CHOICE_OFFSET = 3
 
 
 class HelpInfo(object):
@@ -62,12 +64,6 @@ class TextChoiceSuggester(object):
   """Utility to suggest mistyped commands.
 
   """
-  _SYNONYM_SETS = [
-      set(['create', 'add']),
-      set(['delete', 'remove']),
-      set(['describe', 'get']),
-      set(['patch', 'update']),
-  ]
 
   def __init__(self, choices=None):
     # A mapping of 'thing typed' to the suggestion that should be offered.
@@ -104,17 +100,6 @@ class TextChoiceSuggester(object):
       if alias not in self._choices:
         self._choices[alias] = suggestion
 
-  def AddSynonyms(self):
-    """Activate the set of synonyms for this suggester."""
-    for s_set in TextChoiceSuggester._SYNONYM_SETS:
-      valid_choices = set(self._choices.keys()) & s_set
-      for choice in valid_choices:
-        # Add all synonyms in the set as aliases for each real choice that is
-        # valid.  This will never clobber the original choice that is there.
-        # If none of the synonyms are valid choices, this will not add any
-        # aliases for this synonym set.
-        self.AddAliases(s_set, choice)
-
   def GetSuggestion(self, arg):
     """Find the item that is closest to what was attempted.
 
@@ -276,7 +261,7 @@ def _GetInvertedFlagName(flag):
   return flag.option_strings[0].replace('--', '--no-', 1)
 
 
-def GetArgDetails(arg):
+def GetArgDetails(arg, depth=0):
   """Returns the help message with autogenerated details for arg."""
   help_text = getattr(arg, 'hidden_help', arg.help)
   if callable(help_text):
@@ -316,15 +301,19 @@ def GetArgDetails(arg):
       one_of = '(currently only one value is supported)'
     if isinstance(choices, dict):
       choices_iteritems = six.iteritems(choices)
-      if not isinstance(choices, OrderedDict):
+      if not isinstance(choices, collections.OrderedDict):
         choices_iteritems = sorted(choices_iteritems)
+      choices = [
+          '*{name}*{depth} {desc}'.format(
+              name=name, desc=desc, depth=':' * (depth + _CHOICE_OFFSET))
+          for name, desc in choices_iteritems]
+      # Append marker to indicate end of list.
+      choices.append(':' * (depth + _CHOICE_OFFSET))
       extra_help.append(
           '_{metavar}_ must be {one_of}:\n\n{choices}\n\n'.format(
               metavar=metavar,
               one_of=one_of,
-              choices='\n'.join(
-                  ['*{name}*::: {desc}'.format(name=name, desc=desc)
-                   for name, desc in choices_iteritems])))
+              choices='\n'.join(choices)))
     else:
       extra_help.append('_{metavar}_ must be {one_of}: {choices}.'.format(
           metavar=metavar,
@@ -675,7 +664,7 @@ def GetArgSections(arguments, is_root):
       dests.add(arg.dest)
     if category not in categories:
       categories[category] = set()
-    categories[category].add(arg)  # pytype: disable=attribute-error
+    categories[category].add(arg)
 
   # Collect the priority sections first in order:
   #   POSITIONAL ARGUMENTS, REQUIRED, COMMON, OTHER, and categorized.
@@ -783,10 +772,10 @@ def GetUsage(command, argument_interceptor):
   group_helps = command.GetSubGroupHelps()
   command_helps = command.GetSubCommandHelps()
 
-  groups = sorted([name for (name, help_info) in six.iteritems(group_helps)
-                   if command.IsHidden() or not help_info.is_hidden])
-  commands = sorted([name for (name, help_info) in six.iteritems(command_helps)
-                     if command.IsHidden() or not help_info.is_hidden])
+  groups = sorted(name for (name, help_info) in six.iteritems(group_helps)
+                  if command.IsHidden() or not help_info.is_hidden)
+  commands = sorted(name for (name, help_info) in six.iteritems(command_helps)
+                    if command.IsHidden() or not help_info.is_hidden)
 
   all_subtypes = []
   if groups:
@@ -815,14 +804,105 @@ def GetUsage(command, argument_interceptor):
     WrapWithPrefix('optional flags may be', ' | '.join(optional_flags),
                    HELP_INDENT, LINE_WIDTH, spacing='  ', writer=buf)
 
-  buf.write("""
-For detailed information on this command and its flags, run:
-  {command_path} --help
-""".format(command_path=' '.join(command.GetPath())))
+  buf.write('\n' + GetHelpHint(command))
 
   return buf.getvalue()
 
 
+def GetCategoricalUsage(command, categories):
+  """Constructs an alternative Usage markdown string organized into categories.
+
+  The string is formatted as a series of tables, one for each category. Each
+  subcommand and subgroup of the parent command is printed in its corresponding
+  table together with a short summary describing its functionality. If there are
+  no categories to display, then an empty string is returned.
+
+  Args:
+    command: calliope._CommandCommon, The command object that we're helping.
+    categories: A dictionary mapping category name to the set of elements
+      belonging to that category.
+
+  Returns:
+    str, The command usage markdown string organized into categories.
+  """
+  if not categories:
+    return ''
+  buf = io.StringIO()
+  buf.write('# Available commands for {group}:\n'.format(
+      group=' '.join(command.GetPath())))
+  for category, elements in sorted(six.iteritems(categories)):
+    buf.write('\n### {category}\n\n'.format(category=category))
+    buf.write('---------------------- | ---\n')
+    for element in sorted(elements, key=lambda e: e.name):
+      short_help = None
+      if element.name == 'alpha':
+        short_help = element.short_help[10:]
+      elif element.name == 'beta':
+        short_help = element.short_help[9:]
+      else:
+        short_help = element.short_help
+      buf.write('{name} | {description}\n'.format(
+          name=element.name.replace('_', '-'), description=short_help))
+  return buf.getvalue()
+
+
+def _WriteUncategorizedTable(command, elements, element_type, writer):
+  """Helper method to GetUncategorizedUsage().
+
+  The elements are written to a markdown table with a special heading. Element
+  names are printed in the first column, and help snippet text is printed in the
+  second. No categorization is performed.
+
+  Args:
+    command: calliope._CommandCommon, The command object that we're helping.
+    elements: an iterable over backend.CommandCommon, The sub-elements that
+      we're printing to the table.
+    element_type: str, The type of elements we are dealing with. Usually
+      'groups' or 'commands'.
+    writer: file-like, Receiver of the written output.
+  """
+  writer.write('# Available {element_type} for {group}:\n'.format(
+      element_type=element_type, group=' '.join(command.GetPath())))
+  writer.write('---------------------- | ---\n')
+  for element in sorted(elements, key=lambda e: e.name):
+    if element.IsHidden():
+      continue
+    writer.write('{name} | {description}\n'.format(
+        name=element.name.replace('_', '-'), description=element.short_help))
+
+
+def GetUncategorizedUsage(command):
+  """Constructs a Usage markdown string for uncategorized command groups.
+
+  The string is formatted as two tables, one for the subgroups and one for the
+  subcommands. Each sub-element is printed in its corresponding table together
+  with a short summary describing its functionality.
+
+  Args:
+    command: calliope._CommandCommon, the command object that we're helping.
+
+  Returns:
+    str, The command Usage markdown string as described above.
+  """
+  buf = io.StringIO()
+  if command.groups:
+    _WriteUncategorizedTable(command, command.groups.values(), 'groups', buf)
+
+  if command.commands:
+    buf.write('\n')
+    _WriteUncategorizedTable(
+        command, command.commands.values(), 'commands', buf)
+
+  return buf.getvalue()
+
+
+def GetHelpHint(command):
+  return """\
+For detailed information on this command and its flags, run:
+  {command_path} --help
+""".format(command_path=' '.join(command.GetPath()))
+
+
 def ExtractHelpStrings(docstring):
   """Extracts short help and long help from a docstring.
 
