@@ -23,7 +23,6 @@ from googlecloudsdk.calliope import arg_parsers
 from googlecloudsdk.calliope import base
 from googlecloudsdk.calliope import exceptions
 from googlecloudsdk.calliope.concepts import concepts
-from googlecloudsdk.command_lib.projects import resource_args as project_resource_args
 from googlecloudsdk.command_lib.util import completers
 from googlecloudsdk.command_lib.util.concepts import concept_parsers
 from googlecloudsdk.core.util import text
@@ -229,20 +228,18 @@ class ArgAdder(object):
 
 def InstanceAttributeConfig():
   return concepts.ResourceParameterAttributeConfig(
-      name='instance',
-      help_text='The Cloud Bigtable instance for the {resource}.')
+      name='instance', help_text='Cloud Bigtable instance for the {resource}.')
 
 
 def ClusterAttributeConfig():
   return concepts.ResourceParameterAttributeConfig(
-      name='cluster',
-      help_text='The Cloud Bigtable cluster for the {resource}.')
+      name='cluster', help_text='Cloud Bigtable cluster for the {resource}.')
 
 
 def AppProfileAttributeConfig():
   return concepts.ResourceParameterAttributeConfig(
       name='app profile',
-      help_text='The Cloud Bigtable application profile for the {resource}.')
+      help_text='Cloud Bigtable application profile for the {resource}.')
 
 
 def GetInstanceResourceSpec():
@@ -251,7 +248,7 @@ def GetInstanceResourceSpec():
       'bigtableadmin.projects.instances',
       resource_name='instance',
       instancesId=InstanceAttributeConfig(),
-      projectsId=project_resource_args.PROJECT_ATTRIBUTE_CONFIG,
+      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
       disable_auto_completers=False)
 
 
@@ -262,7 +259,7 @@ def GetClusterResourceSpec():
       resource_name='cluster',
       clustersId=ClusterAttributeConfig(),
       instancesId=InstanceAttributeConfig(),
-      projectsId=project_resource_args.PROJECT_ATTRIBUTE_CONFIG,
+      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
       disable_auto_completers=False)
 
 
@@ -270,9 +267,9 @@ def GetAppProfileResourceSpec():
   """Return the resource specification for a Bigtable app profile."""
   return concepts.ResourceSpec(
       'bigtableadmin.projects.instances.appProfiles',
-      resource_name='app-profile',
+      resource_name='app profile',
       instancesId=InstanceAttributeConfig(),
-      projectsId=project_resource_args.PROJECT_ATTRIBUTE_CONFIG,
+      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
       disable_auto_completers=False)
 
 
