@@ -184,11 +184,14 @@ def BackendServiceArgumentForTargetTcpProxy(required=True):
         """)
 
 
-def AddLoadBalancingScheme(parser, include_alpha=False):
+def AddLoadBalancingScheme(parser,
+                           include_l7_ilb=False,
+                           include_traffic_director=False):
   parser.add_argument(
       '--load-balancing-scheme',
-      choices=['INTERNAL', 'EXTERNAL'] + (['INTERNAL_SELF_MANAGED']
-                                          if include_alpha else []),
+      choices=['INTERNAL', 'EXTERNAL'] +
+      (['INTERNAL_MANAGED'] if include_l7_ilb else []) +
+      (['INTERNAL_SELF_MANAGED'] if include_traffic_director else []),
       type=lambda x: x.replace('-', '_').upper(),
       default='EXTERNAL',
       help='Specifies if this is internal or external load balancer.')
@@ -340,8 +343,9 @@ def HealthCheckArgument(required=False, include_alpha=False):
       if include_alpha else None,
       short_help="""\
       Specifies a list of health check objects for checking the health of
-      the backend service. Health checks need not be for the same protocol
-      as that of the backend service.
+      the backend service. Currently at most one health check can be specified.
+      Health checks need not be for the same protocol as that of the backend
+      service.
       """,
       region_explanation=compute_flags.REGION_PROPERTY_EXPLANATION
       if include_alpha else None)
@@ -356,8 +360,14 @@ def HttpHealthCheckArgument(required=False):
       required=required,
       global_collection='compute.httpHealthChecks',
       short_help="""\
-      Specifies a list of HTTP health check objects for checking the health
-      of the backend service.
+      Specifies a list of legacy HTTP health check objects for checking the
+      health of the backend service.
+
+      Legacy health checks are not recommended for backend services. It is
+      possible to use a legacy health check on a backend service for a HTTP(S)
+      load balancer if that backend service uses instance groups. For more
+      information, see this guide:
+      https://cloud.google.com/load-balancing/docs/health-check-concepts#lb_guide
       """)
 
 
@@ -370,8 +380,14 @@ def HttpsHealthCheckArgument(required=False):
       required=required,
       global_collection='compute.httpsHealthChecks',
       short_help="""\
-      Specifies a list of HTTPS health check objects for checking the health
-      of the backend service.
+      Specifies a list of legacy HTTPS health check objects for checking the
+      health of the backend service.
+
+      Legacy health checks are not recommended for backend services. It is
+      possible to use a legacy health check on a backend service for a HTTP(S)
+      load balancer if that backend service uses instance groups. For more
+      information, see this guide:
+      https://cloud.google.com/load-balancing/docs/health-check-concepts#lb_guide
       """)
 
 
@@ -444,7 +460,7 @@ def AddSessionAffinity(parser, target_pools=False, hidden=False):
             'port will go to the same VM in the backend while that VM remains '
             'healthy.'),
     })
-  help_str = 'The type of session affinity to use.'
+  help_str = 'The type of TCP session affinity to use. Not supported for UDP.'
   parser.add_argument(
       '--session-affinity',
       choices=choices,
@@ -484,17 +500,20 @@ def AddTimeout(parser, default='30s'):
       default=default,
       type=arg_parsers.Duration(),
       help="""\
-      The amount of time to wait for a backend to return a full response for the
-      request and for the load balancer to proxy the response to the client
-      before considering the request failed.
+      Only applicable to HTTP(S), SSL Proxy, and TCP Proxy load balancers:
+      The amount of time to wait for a backend to return a full response for
+      the request and for the load balancer to proxy the response to the
+      client before considering the request failed.
 
-      For example, specifying `10s` gives instances 10 seconds to respond to
+      For example, specifying 10s gives instances 10 seconds to respond to
       requests. The load balancer will retry GET requests once if the backend
-      closes the connection or times out before sending response headers to the
-      proxy. If the backend produces any response headers, the load balancer
-      does not retry. If the backend does not reply at all, the load balancer
-      returns a `502 Bad Gateway` error to the client.
-      See $ gcloud topic datetimes for information on duration formats.
+      closes the connection or times out before sending response headers to
+      the proxy. If the backend produces any response headers, the load
+      balancer does not retry. If the backend does not reply at all, the load
+      balancer returns a 502 Bad Gateway error to the client. See $ gcloud
+      topic datetimes for information on duration formats.
+
+      This parameter has no effect if the load-balancing-scheme is INTERNAL.
       """)
 
 
@@ -513,6 +532,8 @@ def AddPortName(parser):
       this flag, your instance groups must have a service named ``http''
       configured. See also
       `gcloud compute instance-groups set-named-ports --help`.
+      The ``port-name'' parameter cannot be set if the
+      load-balancing-scheme is INTERNAL.
       """)
 
 
@@ -528,7 +549,7 @@ def AddProtocol(parser, default='HTTP'):
       `TCP`, `UDP`.
 
       If the load-balancing-scheme is `EXTERNAL`, the protocol must be one of:
-      `HTTP`, `HTTPS`, `HTTP2`, `SSL`, `TCP`, `UDP`.
+      `HTTP`, `HTTPS`, `HTTP2`, `SSL`, `TCP`.
       """
   )
 
@@ -574,6 +595,33 @@ def AddFailoverRatio(parser):
       """)
 
 
+def AddEnableLogging(parser, default):
+  """Adds the enable logging argument to the argparse."""
+  parser.add_argument(
+      '--enable-logging',
+      action='store_true',
+      default=default,
+      help="""\
+      The logging options for the load balancer traffic served by this backend
+      service. If logging is enabled, logs will be exported to Stackdriver.
+      Enabled by default.
+      """)
+
+
+def AddLoggingSampleRate(parser):
+  """Adds the logging sample rate argument to the argparse."""
+  parser.add_argument(
+      '--logging-sample-rate',
+      type=arg_parsers.BoundedFloat(lower_bound=0.0, upper_bound=1.0),
+      help="""\
+      This field can only be specified if logging is enabled for the backend
+      service. The value of the field must be a float in the range [0, 1]. This
+      configures the sampling rate of requests to the load balancer where 1.0
+      means all logged requests are reported and 0.0 means no logged requests
+      are reported. The default value is 1.0.
+      """)
+
+
 def AddInstanceGroupAndNetworkEndpointGroupArgs(parser, verb):
   backend_group = parser.add_group(required=True, mutex=True)
   instance_group = backend_group.add_group('Instance Group')
