@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2017 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,16 +15,19 @@
 
 """Flags and helpers for the compute disk-types commands."""
 
-import argparse
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
 
+from googlecloudsdk.command_lib.compute import completers as compute_completers
 from googlecloudsdk.command_lib.compute import flags as compute_flags
 
 
 def MakeDiskTypeArg(regional):
   return compute_flags.ResourceArgument(
       resource_name='disk type',
-      completion_resource_id='compute.diskTypes',
+      completer=compute_completers.DiskTypesCompleter,
       name='DISK_TYPE',
       zonal_collection='compute.diskTypes',
       regional_collection='compute.regionDiskTypes' if regional else None,
-      region_explanation=argparse.SUPPRESS if regional else None)
+      region_hidden=not regional)
