@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2016 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,6 +15,11 @@
 
 """Flags and helpers for the compute https-health-checks commands."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
+from googlecloudsdk.command_lib.compute import completers as compute_completers
 from googlecloudsdk.command_lib.compute import flags as compute_flags
 
 DEFAULT_LIST_FORMAT = """\
@@ -28,7 +34,7 @@ DEFAULT_LIST_FORMAT = """\
 def HttpsHealthCheckArgument(required=True, plural=False):
   return compute_flags.ResourceArgument(
       resource_name='HTTPS health check',
-      completion_resource_id='compute.httpsHealthChecks',
+      completer=compute_completers.HttpsHealthChecksCompleter,
       plural=plural,
       required=required,
       global_collection='compute.httpsHealthChecks')
