@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2016 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,21 +15,37 @@
 
 """Flags and helpers for the compute backend-buckets commands."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
+from googlecloudsdk.calliope import actions as calliope_actions
 from googlecloudsdk.calliope import arg_parsers
 from googlecloudsdk.calliope import base
 from googlecloudsdk.calliope import exceptions
+from googlecloudsdk.command_lib.compute import completers as compute_completers
 from googlecloudsdk.command_lib.compute import flags as compute_flags
+from googlecloudsdk.command_lib.compute.disks import flags as disks_flags
+from googlecloudsdk.command_lib.util import completers
 
 _SOURCE_DISK_DETAILED_HELP = """\
         A source disk to create the image from. The value for this option can be
         the name of a disk with the zone specified via ``--source-disk-zone''
         flag.
 """
+_SOURCE_SNAPSHOT_DETAILED_HELP = """\
+        A source snapshot to create the image from. The value for this option
+        can be the name of a snapshot within the same project as the destination
+        image.
+"""
 _REPLACEMENT_DISK_DETAILED_HELP = """\
        Specifies a Compute Engine image as a replacement for the image
-       being phased out. Users of the deprecated image will be advised to switch
-       to this replacement. For example, *--replacement example-image* or
-       *--replacement projects/google/global/images/example-image*.
+       being phased out. Users of the deprecated image will be
+       advised to switch to this replacement. For example, *--replacement
+       example-image* or *--replacement
+       projects/google/global/images/example-image*.
+
+       This flag value is purely informational and is not validated in any way.
        """
 
 _SOURCE_DISK_ZONE_EXPLANATION = compute_flags.ZONE_PROPERTY_EXPLANATION
@@ -43,18 +60,53 @@ LIST_FORMAT = """\
     )"""
 
 
-def MakeDiskImageArg(plural=False):
+class ImagesCompleter(compute_completers.ListCommandCompleter):
+
+  def __init__(self, **kwargs):
+    super(ImagesCompleter, self).__init__(
+        collection='compute.images',
+        list_command='compute images list --uri',
+        **kwargs)
+
+
+class SearchImagesCompleter(completers.ResourceSearchCompleter):
+
+  def __init__(self, **kwargs):
+    super(SearchImagesCompleter, self).__init__(
+        collection='compute.images',
+        **kwargs)
+
+
+def MakeDiskImageArg(plural=False, required=True, name='image_name'):
   return compute_flags.ResourceArgument(
       resource_name='disk image',
-      name='image_name',
-      completion_resource_id='compute.images',
+      name=name,
+      completer=ImagesCompleter,
       plural=plural,
+      required=required,
       global_collection='compute.images')
 
 
 def MakeForceCreateArg():
   return base.Argument(
       '--force-create',
+      action=calliope_actions.DeprecationAction(
+          flag_name='force-create',
+          warn='Flag force-create is deprecated. Use --force instead.',
+          error='Flag force-create is removed. Use --force instead.',
+          action='store_true'),
+      default=False,
+      help="""\
+          DEPRECATED, use --force instead.
+          By default, image creation fails when it is created from a disk that
+          is attached to a running instance. When this flag is used, image
+          creation from disk will proceed even if the disk is in use.
+          """)
+
+
+def MakeForceArg():
+  return base.Argument(
+      '--force',
       action='store_true',
       default=False,
       help="""\
@@ -63,10 +115,11 @@ def MakeForceCreateArg():
           creation from disk will proceed even if the disk is in use.
           """)
 
+
 REPLACEMENT_DISK_IMAGE_ARG = compute_flags.ResourceArgument(
     resource_name='disk image',
     name='--replacement',
-    completion_resource_id='compute.images',
+    completer=ImagesCompleter,
     global_collection='compute.images',
     required=False,
     short_help='Specifies a Compute Engine image as a replacement.',
@@ -75,13 +128,31 @@ REPLACEMENT_DISK_IMAGE_ARG = compute_flags.ResourceArgument(
 SOURCE_DISK_ARG = compute_flags.ResourceArgument(
     resource_name='source disk',
     name='--source-disk',
-    completion_resource_id='compute.disks',
+    completer=compute_completers.DisksCompleter,
     zonal_collection='compute.disks',
     short_help='The deprecation state to set on the image.',
     detailed_help=_SOURCE_DISK_DETAILED_HELP,
     zone_explanation=_SOURCE_DISK_ZONE_EXPLANATION,
     required=False)
 
+SOURCE_IMAGE_ARG = compute_flags.ResourceArgument(
+    resource_name='source image',
+    name='--source-image',
+    completer=ImagesCompleter,
+    global_collection='compute.images',
+    short_help='An existing Compute Engine image from which to import.',
+    required=False)
+
+SOURCE_SNAPSHOT_ARG = compute_flags.ResourceArgument(
+    resource_name='snapshot',
+    name='--source-snapshot',
+    completer=disks_flags.SnapshotsCompleter,
+    required=False,
+    global_collection='compute.snapshots',
+    short_help='A source snapshot used to create an image.',
+    detailed_help=_SOURCE_SNAPSHOT_DETAILED_HELP,
+)
+
 
 def AddCommonArgs(parser):
   """Add common image creation args."""
@@ -143,16 +214,9 @@ def AddCloningImagesArgs(parser, sources_group):
       """)
 
 
-def AddGuestOsFeaturesArg(parser, guest_os_features):
-  """Add the guest-os-features arg."""
-  if not guest_os_features:
-    return
-  parser.add_argument(
-      '--guest-os-features',
-      metavar='GUEST_OS_FEATURE',
-      type=arg_parsers.ArgList(element_type=lambda x: x.upper(),
-                               choices=guest_os_features),
-      help=('One or more features supported by the OS in the image.'))
+def AddCreatingImageFromSnapshotArgs(parser, sources_group):
+  """Add args to support creating image from snapshot."""
+  SOURCE_SNAPSHOT_ARG.AddArgument(parser, mutex_group=sources_group)
 
 
 def ValidateSourceArgs(args, sources):
@@ -172,4 +236,3 @@ def ValidateSourceArgs(args, sources):
   if source_arg_count < 1:
     raise exceptions.MinimumArgumentException(source_arg_names,
                                               sources_error_message)
-
