@@ -18,6 +18,7 @@ from __future__ import absolute_import
 from __future__ import division
 from __future__ import unicode_literals
 
+import copy
 import functools
 
 from googlecloudsdk.api_lib.compute import constants
@@ -26,6 +27,7 @@ from googlecloudsdk.api_lib.compute import image_utils
 from googlecloudsdk.api_lib.compute import kms_utils
 from googlecloudsdk.api_lib.compute import utils
 from googlecloudsdk.api_lib.compute.zones import service as zones_service
+from googlecloudsdk.api_lib.util import apis
 from googlecloudsdk.calliope import actions
 from googlecloudsdk.calliope import arg_parsers
 from googlecloudsdk.calliope import exceptions
@@ -114,6 +116,14 @@ INSTANCES_ARG_FOR_CREATE = compute_flags.ResourceArgument(
     zone_explanation=compute_flags.ZONE_PROPERTY_EXPLANATION,
     plural=True)
 
+INSTANCES_ARG_FOR_IMPORT = compute_flags.ResourceArgument(
+    resource_name='instance',
+    name='instance_names',
+    completer=compute_completers.InstancesCompleter,
+    zonal_collection='compute.instances',
+    zone_explanation=compute_flags.ZONE_PROPERTY_EXPLANATION,
+    plural=True)
+
 SSH_INSTANCE_RESOLVER = compute_flags.ResourceResolver.FromMap(
     'instance', {compute_scope.ScopeEnum.ZONE: 'compute.instances'})
 
@@ -291,8 +301,6 @@ def AddLocalSsdArgs(parser):
       help="""\
       Attaches a local SSD to the instances.
 
-      This flag is currently in BETA and may change without notice.
-
       *device-name*::: Optional. A name that indicates the disk name
       the guest operating system will see.  If omitted, a device name
       of the form ``local-ssd-N'' will be used.
@@ -304,6 +312,26 @@ def AddLocalSsdArgs(parser):
       """)
 
 
+def AddLocalNvdimmArgs(parser):
+  """Adds local NVDIMM argument for instances and instance-templates."""
+
+  parser.add_argument(
+      '--local-nvdimm',
+      type=arg_parsers.ArgDict(spec={
+          'size': arg_parsers.BinarySize(),
+      }),
+      action='append',
+      help="""\
+      Attaches a local NVDIMM to the instances.
+
+      *size*::: Optional. Size of the NVDIMM disk. The value must be a whole
+      number followed by a size unit of ``KB'' for kilobyte, ``MB'' for
+      megabyte, ``GB'' for gigabyte, or ``TB'' for terabyte. For example,
+      ``3TB'' will produce a 3 terabyte disk. Allowed values are: 3TB and 6TB
+      and the default is 3 TB.
+      """)
+
+
 def AddLocalSsdArgsWithSize(parser):
   """Adds local SSD argument for instances and instance-templates."""
 
@@ -337,13 +365,35 @@ def AddLocalSsdArgsWithSize(parser):
       """)
 
 
-def AddDiskArgs(parser, enable_regional_disks=False, enable_kms=False):
+def _GetDiskDeviceNameHelp(container_mount_enabled=False):
+  """Helper to get documentation for "device-name" param of disk spec."""
+  if container_mount_enabled:
+    return (
+        'An optional name that indicates the disk name the guest operating '
+        'system will see. Must be the same as `name` if used with '
+        '`--container-mount-disk`. If omitted, a device name of the form '
+        '`persistent-disk-N` will be used. If omitted and used with '
+        '`--container-mount-disk` (where the `name` of the container mount '
+        'disk is the same as in this flag), a device name equal to disk `name` '
+        'will be used.')
+  else:
+    return (
+        'An optional name that indicates the disk name the guest operating '
+        'system will see. If omitted, a device name of the form '
+        '`persistent-disk-N` will be used.')
+
+
+def AddDiskArgs(parser, enable_regional_disks=False, enable_kms=False,
+                container_mount_enabled=False):
   """Adds arguments related to disks for instances and instance-templates."""
 
+  disk_device_name_help = _GetDiskDeviceNameHelp(
+      container_mount_enabled=container_mount_enabled)
+
   parser.add_argument(
       '--boot-disk-device-name',
       help="""\
-      The name the guest operating system will see for the boot disk as.  This
+      The name the guest operating system will see for the boot disk.  This
       option can only be specified if a new boot disk is being created (as
       opposed to mounting an existing persistent disk).
       """)
@@ -409,15 +459,13 @@ def AddDiskArgs(parser, enable_regional_disks=False, enable_kms=False):
       virtual machines will use the first partition of the disk for
       their root file systems. The default value for this is ``no''.
 
-      *device-name*::: An optional name that indicates the disk name
-      the guest operating system will see. If omitted, a device name
-      of the form ``persistent-disk-N'' will be used.
+      *device-name*::: {}
 
       *auto-delete*::: If ``yes'',  this persistent disk will be
       automatically deleted when the instance is deleted. However,
       if the disk is later detached from the instance, this option
       won't apply. The default value for this is ``no''.
-      """
+      """.format(disk_device_name_help)
   if enable_regional_disks:
     disk_help += """
       *scope*::: Can be `zonal` or `regional`. If ``zonal'', the disk is
@@ -433,20 +481,30 @@ def AddDiskArgs(parser, enable_regional_disks=False, enable_kms=False):
       help=disk_help)
 
 
-def AddCreateDiskArgs(parser, enable_kms=False, enable_snapshots=False):
+def AddCreateDiskArgs(parser, enable_kms=False, enable_snapshots=False,
+                      container_mount_enabled=False):
   """Adds create-disk argument for instances and instance-templates."""
 
+  disk_device_name_help = _GetDiskDeviceNameHelp(
+      container_mount_enabled=container_mount_enabled)
+  disk_name_extra_help = '' if not container_mount_enabled else (
+      ' Must specify this option if attaching the disk '
+      'to a container with `--container-mount-disk`.')
+  disk_mode_extra_help = '' if not container_mount_enabled else (
+      ' It is an error to create a disk in `ro` mode '
+      'if attaching it to a container with `--container-mount-disk`.')
+
   disk_help = """\
       Creates and attaches persistent disks to the instances.
 
       *name*::: Specifies the name of the disk. This option cannot be
-      specified if more than one instance is being created.
+      specified if more than one instance is being created.{}
 
       *description*::: Optional textual description for the disk being created.
 
       *mode*::: Specifies the mode of the disk. Supported options
       are ``ro'' for read-only and ``rw'' for read-write. If
-      omitted, ``rw'' is used as a default.
+      omitted, ``rw'' is used as a default.{}
 
       *image*::: Specifies the name of the image that the disk will be
       initialized with. A new disk will be created based on the given
@@ -482,15 +540,14 @@ def AddCreateDiskArgs(parser, enable_kms=False, enable_snapshots=False):
       types, run $ gcloud compute disk-types list. The default disk type
       is ``pd-standard''.
 
-      *device-name*::: An optional name that indicates the disk name
-      the guest operating system will see. If omitted, a device name
-      of the form ``persistent-disk-N'' will be used.
+      *device-name*::: {}
 
       *auto-delete*::: If ``yes'',  this persistent disk will be
       automatically deleted when the instance is deleted. However,
       if the disk is later detached from the instance, this option
       won't apply. The default value for this is ``no''.
-      """
+      """.format(disk_name_extra_help, disk_mode_extra_help,
+                 disk_device_name_help)
   if enable_kms:
     disk_help += """
       *kms-key*::: Fully qualified Cloud KMS cryptokey name that will
@@ -530,7 +587,7 @@ def AddCreateDiskArgs(parser, enable_kms=False, enable_snapshots=False):
       'image': str,
       'image-family': str,
       'image-project': str,
-      'size': arg_parsers.BinarySize(lower_bound='10GB'),
+      'size': arg_parsers.BinarySize(lower_bound='1GB'),
       'type': str,
       'device-name': str,
       'auto-delete': str,
@@ -795,6 +852,7 @@ def ValidateCreateDiskFlags(args, enable_snapshots=False):
     image_value = disk.get('image')
     image_family_value = disk.get('image-family')
     source_snapshot = disk.get('source-snapshot')
+
     disk_source = set()
     if image_value:
       disk_source.add(image_value)
@@ -815,6 +873,14 @@ def ValidateCreateDiskFlags(args, enable_snapshots=False):
       raise exceptions.ToolException(source_error_message)
 
 
+def ValidateImageFlags(args):
+  """Validates the image flags."""
+  if args.image_project and not (args.image or args.image_family):
+    raise exceptions.ToolException(
+        'Must specify either [--image] or [--image-family] when specifying '
+        '[--image-project] flag.')
+
+
 def AddAddressArgs(parser,
                    instances=True,
                    multiple_network_interface_cards=True):
@@ -863,7 +929,8 @@ def AddAddressArgs(parser,
     network_interface_help = """\
         Adds a network interface to the instance. Mutually exclusive with any
         of these flags: *--address*, *--network*, *--network-tier*, *--subnet*,
-        *--private-network-ip*.
+        *--private-network-ip*. This flag can be repeated to specify multiple
+        network interfaces.
 
         The following keys are allowed:
         *address*::: Assigns the given external address to the instance that is
@@ -1039,9 +1106,10 @@ def AddServiceAccountAndScopeArgs(parser, instance_exists,
   service_account_help = """\
   A service account is an identity attached to the instance. Its access tokens
   can be accessed through the instance metadata server and are used to
-  authenticate applications on the instance. The account can be either an email
-  address or an alias corresponding to a service account. You can explicitly
-  specify the Compute Engine default service account using the 'default' alias.
+  authenticate applications on the instance. The account can be set using an
+  email address corresponding to the required service account. You can
+  explicitly specify the Compute Engine default service account using the
+  'default' alias.
 
   If not provided, the instance will {0}.
   """.format(sa_exists if instance_exists else sa_not_exists)
@@ -1056,73 +1124,13 @@ def AddServiceAccountAndScopeArgs(parser, instance_exists,
       else 'Create instance without scopes')
   scopes_exists = 'keep the scopes it currently has'
   scopes_not_exists = 'be assigned the default scopes, described below'
-  scopes_default_list = '- ' + '\n      - '.join(constants.DEFAULT_SCOPES)
   scopes_help = """\
-  If not provided, the instance will {0}. {1}
+If not provided, the instance will {exists}. {extra}
 
-  SCOPE can be either the full URI of the scope or an alias. Available
-  aliases are:
-
-    default
-      Scopes assigned to instances by default:
-      {scopes_default_list}
-
-    bigquery
-      - https://www.googleapis.com/auth/bigquery
-
-    cloud-platform
-      - https://www.googleapis.com/auth/cloud-platform
-
-    compute-ro
-      - https://www.googleapis.com/auth/compute.readonly
-
-    compute-rw
-      - https://www.googleapis.com/auth/compute
-
-    datastore
-      - https://www.googleapis.com/auth/datastore
-
-    logging-write
-      - https://www.googleapis.com/auth/logging.write
-
-    monitoring
-      - https://www.googleapis.com/auth/monitoring
-
-    monitoring-write
-      - https://www.googleapis.com/auth/monitoring.write
-
-    service-control
-      - https://www.googleapis.com/auth/servicecontrol
-
-    service-management
-      - https://www.googleapis.com/auth/service.management.readonly
-
-    sql (DEPRECATED)
-      - https://www.googleapis.com/auth/sqlservice:
-
-    sql-admin
-      - https://www.googleapis.com/auth/sqlservice.admin
-
-    storage-full
-      - https://www.googleapis.com/auth/devstorage.full_control
-
-    storage-ro
-      - https://www.googleapis.com/auth/devstorage.read_only
-
-    storage-rw
-      - https://www.googleapis.com/auth/devstorage.read_write
-
-    taskqueue
-      - https://www.googleapis.com/auth/taskqueue
-
-    userinfo-email
-      - https://www.googleapis.com/auth/userinfo.email
-
-    {scope_deprecation_msg}
-    """.format(scopes_exists if instance_exists else scopes_not_exists,
-               extra_scopes_help,
-               scopes_default_list=scopes_default_list,
-               scope_deprecation_msg=constants.DEPRECATED_SCOPES_MESSAGES)
+{scopes_help}
+""".format(exists=scopes_exists if instance_exists else scopes_not_exists,
+           extra=extra_scopes_help,
+           scopes_help=constants.ScopesHelp())
   scopes_group.add_argument(
       '--scopes',
       type=arg_parsers.ArgList(),
@@ -1173,6 +1181,17 @@ def AddNetworkTierArgs(parser, instance=True, for_update=False):
       help=network_tier_help)
 
 
+def AddDisplayDeviceArg(parser, is_update=False):
+  """Adds public DNS arguments for instance or access configuration."""
+  display_help = 'Enable a display device for instances using a Windows image.'
+  if not is_update:
+    display_help += ' Disabled by default.'
+  parser.add_argument(
+      '--enable-display-device',
+      action=arg_parsers.StoreTrueFalseAction if is_update else 'store_true',
+      help=display_help)
+
+
 def AddPublicDnsArgs(parser, instance=True):
   """Adds public DNS arguments for instance or access configuration."""
 
@@ -1324,10 +1343,12 @@ def AddTagsArgs(parser):
       type=arg_parsers.ArgList(min_length=1),
       metavar='TAG',
       help="""\
-      Specifies a list of tags to apply to the instances for
-      identifying the instances to which network firewall rules will
-      apply. See gcloud_compute_firewall-rules_create(1) for more
-      details.
+      Specifies a list of tags to apply to the instance. These tags allow
+      network firewall rules and routes to be applied to specified VM instances.
+      See gcloud_compute_firewall-rules_create(1) for more details.
+
+      To read more about configuring network tags, read this guide:
+      https://cloud.google.com/vpc/docs/add-remove-network-tags
 
       To list instances with their respective status and tags, run:
 
@@ -1466,44 +1487,8 @@ def AddKonletArgs(parser):
       Default: `--no-container-privileged`.
       """)
 
-  def ParseMountVolumeMode(mode):
-    if not mode or mode == 'rw':
-      return containers_utils.MountVolumeMode.READ_WRITE
-    elif mode == 'ro':
-      return containers_utils.MountVolumeMode.READ_ONLY
-    else:
-      raise exceptions.InvalidArgumentException(
-          '--run-mount-volume', 'Mode can only be "ro" or "rw".')
-
-  parser.add_argument(
-      '--container-mount-host-path',
-      metavar='host-path=HOSTPATH,mount-path=MOUNTPATH[,mode=MODE]',
-      type=arg_parsers.ArgDict(spec={'host-path': str,
-                                     'mount-path': str,
-                                     'mode': ParseMountVolumeMode}),
-      action='append',
-      help="""\
-      Mounts a volume by using host-path.
-
-      *host-path*::: Path on host to mount from.
-
-      *mount-path*::: Path on container to mount to.
-
-      *mode*::: Volume mount mode: rw (read/write) or ro (read-only).
-
-      Default: rw.
-      """)
-
-  parser.add_argument(
-      '--container-mount-tmpfs',
-      metavar='mount-path=MOUNTPATH',
-      type=arg_parsers.ArgDict(spec={'mount-path': str}),
-      action='append',
-      help="""\
-      Mounts empty tmpfs into container at MOUNTPATH.
-
-      *mount-path*::: Path on container to mount to.
-      """)
+  _AddContainerMountHostPathFlag(parser)
+  _AddContainerMountTmpfsFlag(parser)
 
   parser.add_argument(
       '--container-env',
@@ -1590,10 +1575,10 @@ def ValidateLocalSsdFlags(args):
 
 
 def ValidateNicFlags(args):
-  """Valiadates flags specifying network interface cards.
+  """Validates flags specifying network interface cards.
 
   Args:
-    args: parsed comandline arguments.
+    args: parsed command line arguments.
   Raises:
     InvalidArgumentException: when it finds --network-interface that has both
                               address, and no-address keys.
@@ -1690,27 +1675,26 @@ def AddDeletionProtectionFlag(parser, use_default_value=True):
       action=action)
 
 
-def AddShieldedVMConfigArgs(parser, use_default_value=True, for_update=False):
-  """Adds flags for shielded VM configuration.
+def AddShieldedInstanceConfigArgs(parser,
+                                  use_default_value=True,
+                                  for_update=False):
+  """Adds flags for shielded Instance configuration.
 
   Args:
     parser: ArgumentParser, parser to which flags will be added.
     use_default_value: Bool, if True, flag will be given the default value
-        False, else None. Update uses None as an indicator that no update needs
-        to be done for deletion protection.
+      False, else None. Update uses None as an indicator that no update needs to
+      be done for deletion protection.
     for_update: Bool, if True, flags are intended for an update operation.
   """
   if use_default_value:
-    kwargs = {
-        'action': 'store_true',
-        'default': None
-    }
+    default_action = 'store_true'
+    action_kwargs = {'default': None}
   else:
-    kwargs = {
-        'action': arg_parsers.StoreTrueFalseAction
-    }
+    default_action = arg_parsers.StoreTrueFalseAction
+    action_kwargs = {}
 
-  # --shielded-vm-secure-boot
+  # --shielded-secure-boot
   secure_boot_help = """\
       The instance will boot with secure boot enabled.
       """
@@ -1719,12 +1703,15 @@ def AddShieldedVMConfigArgs(parser, use_default_value=True, for_update=False):
       Changes to this setting (via the update command) will only take effect
       after stopping and starting the instance.
       """
-  parser.add_argument(
-      '--shielded-vm-secure-boot',
-      help=secure_boot_help,
-      **kwargs)
 
-  # --shielded-vm-vtpm
+  parser.add_argument(
+      '--shielded-secure-boot',
+      help=secure_boot_help,
+      dest='shielded_vm_secure_boot',
+      action=default_action,
+      **action_kwargs)
+
+  # --shielded-vtpm
   vtpm_help = """\
       The instance will boot with the TPM (Trusted Platform Module) enabled.
       A TPM is a hardware module that can be used for different security
@@ -1735,12 +1722,15 @@ def AddShieldedVMConfigArgs(parser, use_default_value=True, for_update=False):
       Changes to this setting (via the update command) will only take effect
       after stopping and starting the instance.
       """
-  parser.add_argument(
-      '--shielded-vm-vtpm',
-      help=vtpm_help,
-      **kwargs)
 
-  # --shielded-vm-integrity-monitoring
+  parser.add_argument(
+      '--shielded-vtpm',
+      dest='shielded_vm_vtpm',
+      help=vtpm_help,
+      action=default_action,
+      **action_kwargs)
+
+  # --shielded-integrity-monitoring
   integrity_monitoring_help = """\
       Enables monitoring and attestation of the boot integrity of the
       instance. The attestation is performed against the integrity policy
@@ -1753,24 +1743,30 @@ def AddShieldedVMConfigArgs(parser, use_default_value=True, for_update=False):
       Changes to this setting (via the update command) will only take effect
       after stopping and starting the instance.
       """
+
   parser.add_argument(
-      '--shielded-vm-integrity-monitoring',
+      '--shielded-integrity-monitoring',
       help=integrity_monitoring_help,
-      **kwargs)
+      dest='shielded_vm_integrity_monitoring',
+      action=default_action,
+      **action_kwargs)
 
 
-def AddShieldedVMIntegrityPolicyArgs(parser):
-  """Adds flags for shielded VM integrity policy settings."""
+def AddShieldedInstanceIntegrityPolicyArgs(parser):
+  """Adds flags for shielded instance integrity policy settings."""
+  help_text = """\
+  Causes the instance to re-learn the integrity policy baseline using
+  the current instance configuration. Use this flag after any planned
+  boot-specific changes in the instance configuration, like kernel
+  updates or kernel driver installation.
+  """
+  default_action = 'store_true'
   parser.add_argument(
-      '--shielded-vm-learn-integrity-policy',
-      action='store_true',
+      '--shielded-learn-integrity-policy',
+      dest='shielded_vm_learn_integrity_policy',
+      action=default_action,
       default=None,
-      help="""\
-      Causes the instance to re-learn the integrity policy baseline using
-      the current instance configuration. Use this flag after any planned
-      boot-specific changes in the instance configuration, like kernel
-      updates or kernel driver installation.
-      """)
+      help=help_text)
 
 
 def AddHostnameArg(parser):
@@ -1786,41 +1782,476 @@ def AddHostnameArg(parser):
       """)
 
 
-def AddAllocationAffinityGroup(parser):
-  """Adds the argument group to handle allocation affinity configurations."""
-  group = parser.add_group(help="""
-Manage the configuration of desired allocation which this instance could take
-capacity from.
-""")
+def AddReservationAffinityGroup(parser, group_text, affinity_text):
+  """Adds the argument group to handle reservation affinity configurations."""
+  group = parser.add_group(help=group_text)
   group.add_argument(
-      '--allocation-affinity',
+      '--reservation-affinity',
       choices=['any', 'none', 'specific'],
       default='any',
-      help="""
-Specifies the configuration of desired allocation which this instance could
-take capacity from. Choices are 'any', 'none' and 'specific', default is 'any'.
-""")
+      help=affinity_text)
   group.add_argument(
-      '--allocation-label',
-      type=arg_parsers.ArgDict(spec={
-          'key': str,
-          'value': str,
-      }),
+      '--reservation',
       help="""
-The key and values of the label of the allocation resource. Required if the
-value of `--allocation-affinity` is `specific`.
-
-*key*::: The label key of allocation resource.
-
-*value*::: The label value of allocation resource.
+The name of the reservation, required when `--reservation-affinity=specific`.
 """)
 
 
-def ValidateAllocationAffinityGroup(args):
-  """Validates flags specifying allocation affinity."""
-  affinity = getattr(args, 'allocation_affinity', None)
+def ValidateReservationAffinityGroup(args):
+  """Validates flags specifying reservation affinity."""
+  affinity = getattr(args, 'reservation_affinity', None)
   if affinity == 'specific':
-    if not args.IsSpecified('allocation_label'):
+    if not args.IsSpecified('reservation'):
       raise exceptions.InvalidArgumentException(
-          '--allocation-label',
-          'The label of the specific allocation must be specified.')
+          '--reservation',
+          'The name the specific reservation must be specified.')
+
+
+def _GetContainerMountDescriptionAndNameDescription(for_update=False):
+  """Get description text for --container-mount-disk flag."""
+  if for_update:
+    description = ("""\
+Mounts a disk to the container by using mount-path or updates how the volume is
+mounted if the same mount path has already been declared. The disk must already
+be attached to the instance with a device-name that matches the disk name.
+Multiple flags are allowed.
+""")
+    name_description = ("""\
+Name of the disk. Can be omitted if exactly one additional disk is attached to
+the instance. The name of the single additional disk will be used by default.
+""")
+    return description, name_description
+  else:
+    description = ("""\
+Mounts a disk to the specified mount path in the container. Multiple '
+flags are allowed. Must be used with `--disk` or `--create-disk`.
+""")
+    name_description = ("""\
+Name of the disk. If exactly one additional disk is attached
+to the instance using `--disk` or `--create-disk`, specifying disk
+name here is optional. The name of the single additional disk will be
+used by default.
+""")
+    return description, name_description
+
+
+def ParseMountVolumeMode(argument_name, mode):
+  """Parser for mode option in ArgDict specs."""
+  if not mode or mode == 'rw':
+    return containers_utils.MountVolumeMode.READ_WRITE
+  elif mode == 'ro':
+    return containers_utils.MountVolumeMode.READ_ONLY
+  else:
+    raise exceptions.InvalidArgumentException(
+        argument_name, 'Mode can only be [ro] or [rw].')
+
+
+def AddContainerMountDiskFlag(parser, for_update=False):
+  """Add --container-mount-disk flag."""
+  description, name_description = (
+      _GetContainerMountDescriptionAndNameDescription(for_update=for_update))
+  help_text = ("""\
+{}
+
+*name*::: {}
+
+*mount-path*::: Path on container to mount to. Mount paths with spaces
+      and commas (and other special characters) are not supported by this
+      command.
+
+*partition*::: Optional. The partition of the disk to mount. Multiple
+partitions of a disk may be mounted.{}
+
+*mode*::: Volume mount mode: `rw` (read/write) or `ro` (read-only).
+Defaults to `rw`. Fails if the disk mode is `ro` and volume mount mode
+is `rw`.
+""".format(description, name_description,
+           '' if for_update else ' May not be used with --create-disk.'))
+
+  spec = {
+      'name': str,
+      'mount-path': str,
+      'partition': int,
+      'mode': functools.partial(ParseMountVolumeMode,
+                                '--container-mount-disk')}
+  parser.add_argument(
+      '--container-mount-disk',
+      type=arg_parsers.ArgDict(spec=spec, required_keys=['mount-path']),
+      help=help_text,
+      action='append')
+
+
+def _GetMatchingDiskFromMessages(holder, mount_disk_name, disk, client=None):
+  """Helper to match a mount disk's name to a disk message."""
+  if client is None:
+    client = apis.GetClientClass('compute', 'alpha')
+  if mount_disk_name is None and len(disk) == 1:
+    return {
+        'name': holder.resources.Parse(disk[0].source).Name(),
+        'device_name': disk[0].deviceName,
+        'ro': (disk[0].mode ==
+               client.MESSAGES_MODULE.AttachedDisk.ModeValueValuesEnum
+               .READ_WRITE)}, False
+  for disk_spec in disk:
+    disk_name = holder.resources.Parse(disk_spec.source).Name()
+    if disk_name == mount_disk_name:
+      return {
+          'name': disk_name,
+          'device_name': disk_spec.deviceName,
+          'ro': (disk_spec.mode ==
+                 client.MESSAGES_MODULE.AttachedDisk.ModeValueValuesEnum
+                 .READ_WRITE)}, False
+  return None, None
+
+
+def _GetMatchingDiskFromFlags(mount_disk_name, disk, create_disk):
+  """Helper to match a mount disk's name to a disk spec from a flag."""
+
+  def _GetMatchingDiskFromSpec(spec):
+    return {'name': spec.get('name'),
+            'device_name': spec.get('device-name'),
+            'ro': spec.get('mode') == 'ro'}
+
+  if mount_disk_name is None and len(disk + create_disk) == 1:
+    disk_spec = (disk + create_disk)[0]
+    return _GetMatchingDiskFromSpec(disk_spec), bool(create_disk)
+  for disk_spec in disk:
+    if disk_spec.get('name') == mount_disk_name:
+      return _GetMatchingDiskFromSpec(disk_spec), False
+  for disk_spec in create_disk:
+    if disk_spec.get('name') == mount_disk_name:
+      return _GetMatchingDiskFromSpec(disk_spec), True
+  return None, None
+
+
+def _CheckMode(name, mode_value, mount_disk, matching_disk, create):
+  """Make sure the correct mode is specified for container mount disk."""
+  partition = mount_disk.get('partition')
+  if (mode_value == containers_utils.MountVolumeMode.READ_WRITE and
+      matching_disk.get('ro')):
+    raise exceptions.InvalidArgumentException(
+        '--container-mount-disk',
+        'Value for [mode] in [--container-mount-disk] cannot be [rw] if the '
+        'disk is attached in [ro] mode: disk name [{}], partition [{}]'
+        .format(name, partition))
+  if matching_disk.get('ro') and create:
+    raise exceptions.InvalidArgumentException(
+        '--container-mount-disk',
+        'Cannot mount disk named [{}] to container: disk is created in [ro] '
+        'mode and thus cannot be formatted.'.format(
+            name))
+
+
+def GetValidatedContainerMountDisk(holder, container_mount_disk, disk,
+                                   create_disk, for_update=False, client=None):
+  """Validate --container-mount-disk value."""
+  disk = disk or []
+  create_disk = create_disk or []
+  if not container_mount_disk:
+    return
+  if not (disk or create_disk or for_update):
+    raise exceptions.InvalidArgumentException(
+        '--container-mount-disk',
+        'Must be used with `--disk` or `--create-disk`')
+
+  message = '' if for_update else ' using `--disk` or `--create-disk`.'
+  validated_disks = []
+  for mount_disk in container_mount_disk:
+    if for_update:
+      matching_disk, create = _GetMatchingDiskFromMessages(
+          holder, mount_disk.get('name'), disk, client=client)
+    else:
+      matching_disk, create = _GetMatchingDiskFromFlags(
+          mount_disk.get('name'), disk, create_disk)
+    if not mount_disk.get('name'):
+      if len(disk + create_disk) != 1:
+        raise exceptions.InvalidArgumentException(
+            '--container-mount-disk',
+            'Must specify the name of the disk to be mounted unless exactly '
+            'one disk is attached to the instance{}.'.format(message))
+      name = matching_disk.get('name')
+      if not name:
+        raise exceptions.InvalidArgumentException(
+            '--container-mount-disk',
+            'When attaching or creating a disk that is also being mounted to '
+            'a container, must specify the disk name.')
+    else:
+      name = mount_disk.get('name')
+      if not matching_disk:
+        raise exceptions.InvalidArgumentException(
+            '--container-mount-disk',
+            'Attempting to mount a disk that is not attached to the instance: '
+            'must attach a disk named [{}]{}'.format(name, message))
+    if (matching_disk and
+        matching_disk.get('device_name') and
+        matching_disk.get('device_name') != matching_disk.get('name')):
+      raise exceptions.InvalidArgumentException(
+          '--container-mount-disk',
+          'Container mount disk cannot be used with a device whose device-name '
+          'is different from its name. The disk with name [{}] has the '
+          'device-name [{}].'.format(matching_disk.get('name'),
+                                     matching_disk.get('device_name')))
+
+    mode_value = mount_disk.get('mode')
+    if matching_disk:
+      _CheckMode(name, mode_value, mount_disk, matching_disk, create)
+    if matching_disk and create and mount_disk.get('partition'):
+      raise exceptions.InvalidArgumentException(
+          '--container-mount-disk',
+          'Container mount disk cannot specify a partition when the disk '
+          'is created with --create-disk: disk name [{}], partition [{}]'
+          .format(name, mount_disk.get('partition')))
+    mount_disk = copy.deepcopy(mount_disk)
+    mount_disk['name'] = mount_disk.get('name') or name
+    validated_disks.append(mount_disk)
+  return validated_disks
+
+
+def NonEmptyString(parameter_name):
+  def Factory(string):
+    if not string:
+      raise exceptions.InvalidArgumentException(
+          parameter_name, 'Empty string is not allowed.')
+    return string
+  return Factory
+
+
+def _AddContainerEnvGroup(parser):
+  """Add flags to update the container environment."""
+
+  env_group = parser.add_argument_group()
+
+  env_group.add_argument(
+      '--container-env',
+      type=arg_parsers.ArgDict(),
+      action='append',
+      metavar='KEY=VALUE, ...',
+      help="""\
+      Update environment variables `KEY` with value `VALUE` passed to
+      container.
+      - Sets `KEY` to the specified value.
+      - Adds `KEY` = `VALUE`, if `KEY` is not yet declared.
+      - Only the last value of `KEY` is taken when `KEY` is repeated more
+      than once.
+
+      Values, declared with `--container-env` flag override those with the
+      same `KEY` from file, provided in `--container-env-file`.
+      """)
+
+  env_group.add_argument(
+      '--container-env-file',
+      help="""\
+      Update environment variables from a file.
+      Same update rules as for `--container-env` apply.
+      Values, declared with `--container-env` flag override those with the
+      same `KEY` from file.
+
+      File with environment variables declarations in format used by docker
+      (almost). This means:
+      - Lines are in format KEY=VALUE
+      - Values must contain equality signs.
+      - Variables without values are not supported (this is different from
+      docker format).
+      - If # is first non-whitespace character in a line the line is ignored
+      as a comment.
+      """)
+
+  env_group.add_argument(
+      '--remove-container-env',
+      type=arg_parsers.ArgList(),
+      action='append',
+      metavar='KEY',
+      help="""\
+      Removes environment variables `KEY` from container declaration Does
+      nothing, if a variable is not present.
+      """)
+
+
+def _AddContainerArgGroup(parser):
+  """Add flags to update the container arg."""
+
+  arg_group = parser.add_mutually_exclusive_group()
+
+  arg_group.add_argument(
+      '--container-arg',
+      action='append',
+      help="""\
+      Completely replaces the list of arguments with the new list.
+      Each argument must have a separate --container-arg flag.
+      Arguments are appended the new list in the order of flags.
+
+      Cannot be used in the same command with `--clear-container-arg`.
+      """)
+
+  arg_group.add_argument(
+      '--clear-container-args',
+      action='store_true',
+      default=None,
+      help="""\
+      Removes the list of arguments from container declaration.
+
+      Cannot be used in the same command with `--container-arg`.
+      """
+  )
+
+
+def _AddContainerCommandGroup(parser):
+  """Add flags to update the command in the container declaration."""
+  command_group = parser.add_mutually_exclusive_group()
+
+  command_group.add_argument(
+      '--container-command',
+      type=NonEmptyString('--container-command'),
+      help="""\
+      Sets command in the declaration to the specified value.
+      Empty string is not allowed.
+
+      Cannot be used in the same command with `--clear-container-command`.
+      """)
+
+  command_group.add_argument(
+      '--clear-container-command',
+      action='store_true',
+      default=None,
+      help="""\
+      Removes command from container declaration.
+
+      Cannot be used in the same command with `--container-command`.
+      """)
+
+
+def _AddContainerMountHostPathFlag(parser, for_update=False):
+  """Helper to add --container-mount-host-path flag."""
+  if for_update:
+    additional = """\
+
+      - Adds a volume, if `mount-path` is not yet declared.
+      - Replaces a volume, if `mount-path` is declared.
+      All parameters (`host-path`, `mount-path`, `mode`) are completely
+      replaced."""
+  else:
+    additional = ''
+  parser.add_argument(
+      '--container-mount-host-path',
+      metavar='host-path=HOSTPATH,mount-path=MOUNTPATH[,mode=MODE]',
+      type=arg_parsers.ArgDict(spec={'host-path': str,
+                                     'mount-path': str,
+                                     'mode': functools.partial(
+                                         ParseMountVolumeMode,
+                                         '--container-mount-host-path')}),
+      action='append',
+      help="""\
+      Mounts a volume by using host-path.{}
+
+      *host-path*::: Path on host to mount from.
+
+      *mount-path*::: Path on container to mount to. Mount paths with spaces
+      and commas (and other special characters) are not supported by this
+      command.
+
+      *mode*::: Volume mount mode: rw (read/write) or ro (read-only).
+
+      Default: rw.
+      """.format(additional))
+
+
+def _AddContainerMountTmpfsFlag(parser):
+  """Helper to add --container-mount-tmpfs flag."""
+  parser.add_argument(
+      '--container-mount-tmpfs',
+      metavar='mount-path=MOUNTPATH',
+      type=arg_parsers.ArgDict(spec={'mount-path': str}),
+      action='append',
+      help="""\
+      Mounts empty tmpfs into container at MOUNTPATH.
+
+      *mount-path*::: Path on container to mount to. Mount paths with spaces
+      and commas (and other special characters) are not supported by this
+      command.
+      """)
+
+
+def _AddContainerMountGroup(parser, container_mount_disk_enabled=False):
+  """Add flags to update what is mounted to the container."""
+
+  mount_group = parser.add_argument_group()
+
+  _AddContainerMountHostPathFlag(mount_group, for_update=True)
+  _AddContainerMountTmpfsFlag(mount_group)
+
+  if container_mount_disk_enabled:
+    AddContainerMountDiskFlag(parser, for_update=True)
+
+  mount_types = ['`host-path`', '`tmpfs`']
+  if container_mount_disk_enabled:
+    mount_types.append('`disk`')
+  mount_group.add_argument(
+      '--remove-container-mounts',
+      type=arg_parsers.ArgList(),
+      metavar='MOUNTPATH[,MOUNTPATH,...]',
+      help="""\
+      Removes volume mounts ({}) with
+      `mountPath: MOUNTPATH` from container declaration.
+
+      Does nothing, if a volume mount is not declared.
+      """.format(', '.join(mount_types))
+  )
+
+
+def _AddContainerArgs(parser):
+  """Add basic args for update-container."""
+
+  parser.add_argument(
+      '--container-image',
+      type=NonEmptyString('--container-image'),
+      help="""\
+      Sets container image in the declaration to the specified value.
+
+      Empty string is not allowed.
+      """)
+
+  parser.add_argument(
+      '--container-privileged',
+      action='store_true',
+      default=None,
+      help="""\
+      Sets permission to run container to the specified value.
+      """)
+
+  parser.add_argument(
+      '--container-stdin',
+      action='store_true',
+      default=None,
+      help="""\
+      Sets configuration whether to keep container `STDIN` always open to the
+      specified value.
+      """)
+
+  parser.add_argument(
+      '--container-tty',
+      action='store_true',
+      default=None,
+      help="""\
+      Sets configuration whether to allocate a pseudo-TTY for the container
+      to the specified value.
+      """)
+
+  parser.add_argument(
+      '--container-restart-policy',
+      choices=['never', 'on-failure', 'always'],
+      metavar='POLICY',
+      type=lambda val: val.lower(),
+      help="""\
+      Sets container restart policy to the specified value.
+      """)
+
+
+def AddUpdateContainerArgs(parser, container_mount_disk_enabled=False):
+  INSTANCE_ARG.AddArgument(parser, operation_type='update')
+  _AddContainerCommandGroup(parser)
+  _AddContainerEnvGroup(parser)
+  _AddContainerArgGroup(parser)
+  _AddContainerMountGroup(
+      parser,
+      container_mount_disk_enabled=container_mount_disk_enabled)
+  _AddContainerArgs(parser)
