@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2017 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,11 +15,16 @@
 
 """Flags for the compute regions commands."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
+from googlecloudsdk.command_lib.compute import completers as compute_completers
 from googlecloudsdk.command_lib.compute import flags as compute_flags
 
 
 def MakeRegionArg():
   return compute_flags.ResourceArgument(
       resource_name='region',
-      completion_resource_id='compute.regions',
+      completer=compute_completers.RegionsCompleter,
       global_collection='compute.regions')
