@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2016 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,6 +14,11 @@
 # limitations under the License.
 """Flags and helpers for the compute target-pools commands."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
+from googlecloudsdk.command_lib.compute import completers as compute_completers
 from googlecloudsdk.command_lib.compute import flags as compute_flags
 
 DEFAULT_LIST_FORMAT = """\
@@ -25,10 +31,19 @@ DEFAULT_LIST_FORMAT = """\
     )"""
 
 
+class TargetPoolsCompleter(compute_completers.ListCommandCompleter):
+
+  def __init__(self, **kwargs):
+    super(TargetPoolsCompleter, self).__init__(
+        collection='compute.targetPools',
+        list_command='compute target-pools list --uri',
+        **kwargs)
+
+
 def TargetPoolArgument(required=True, help_suffix='.', plural=False):
   return compute_flags.ResourceArgument(
       resource_name='target pool',
-      completion_resource_id='compute.targetPools',
+      completer=TargetPoolsCompleter,
       plural=plural,
       required=required,
       regional_collection='compute.targetPools',
@@ -41,7 +56,7 @@ def BackupPoolArgument(required=True):
   return compute_flags.ResourceArgument(
       resource_name='backup pool',
       name='--backup-pool',
-      completion_resource_id='compute.targetPools',
+      completer=TargetPoolsCompleter,
       plural=False,
       required=required,
       regional_collection='compute.targetPools')
@@ -50,10 +65,10 @@ def BackupPoolArgument(required=True):
 def TargetPoolArgumentForAddRemoveInstances(required=True, help_suffix='.'):
   return compute_flags.ResourceArgument(
       resource_name='target pool',
-      completion_resource_id='compute.targetPools',
+      completer=TargetPoolsCompleter,
       plural=False,
       required=required,
       regional_collection='compute.targetPools',
       short_help='The name of the target pool{0}'.format(help_suffix),
-      region_explanation=('If not specified it will be set the'
+      region_explanation=('If not specified, it will be set to the'
                           ' region of the instances.'))
