@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2016 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,13 +14,27 @@
 # limitations under the License.
 """Flags and helpers for the compute target-ssl-proxies commands."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
+from googlecloudsdk.command_lib.compute import completers as compute_completers
 from googlecloudsdk.command_lib.compute import flags as compute_flags
 
 
+class TargetSslProxiesCompleter(compute_completers.ListCommandCompleter):
+
+  def __init__(self, **kwargs):
+    super(TargetSslProxiesCompleter, self).__init__(
+        collection='compute.targetSslProxies',
+        list_command='compute target-ssl-proxies list --uri',
+        **kwargs)
+
+
 def TargetSslProxyArgument(required=True, plural=False):
   return compute_flags.ResourceArgument(
       resource_name='target SSL proxy',
-      completion_resource_id='compute.targetSslProxies',
+      completer=TargetSslProxiesCompleter,
       plural=plural,
       custom_plural='target SSL proxies',
       required=required,
