@@ -181,6 +181,30 @@ The default Kubernetes version is available using the following command.
   return parser.add_argument('--cluster-version', help=help, hidden=suppressed)
 
 
+def AddReleaseChannelFlag(parser):
+  """Adds a --release-channel flag to the given parser."""
+  help_text = """\
+Release channel a cluster is subscribed to.
+
+When a cluster is subscribed to a release channel, Google maintains both the
+master version and the node version. Node auto-upgrade defaults to true and
+cannot be disabled. Updates to version related fields (e.g. --cluster-version)
+return an error.
+"""
+
+  return parser.add_argument(
+      '--release-channel',
+      metavar='CHANNEL',
+      choices={
+          'rapid': """\
+'rapid' channel clusters are the first to receive the latest releases of
+Kubernetes and other components.
+"""
+      },
+      help=help_text,
+      hidden=False)
+
+
 def AddClusterAutoscalingFlags(parser, update_group=None, hidden=False):
   """Adds autoscaling related flags to parser.
 
@@ -290,6 +314,27 @@ def AddAcceleratorArgs(parser):
       """)
 
 
+def AddAutoscalingProfilesFlag(parser, hidden=False):
+  """Adds autoscaling profiles flag to parser.
+
+  Autoscaling profiles flag is --autoscaling-profile.
+
+  Args:
+    parser: A given parser.
+    hidden: If true, suppress help text for added options.
+  """
+  parser.add_argument(
+      '--autoscaling-profile',
+      required=False,
+      default=None,
+      help="""\
+         Set autoscaling behaviour, choices are 'optimize_utilization' and 'balanced'.
+         Default is 'balanced'.
+      """,
+      hidden=hidden,
+      type=str)
+
+
 def AddAutoprovisioningFlags(parser, hidden=False):
   """Adds node autoprovisioning related flags to parser.
 
@@ -324,12 +369,13 @@ Path of the JSON/YAML file which contains information about the
 cluster's autoscaling configuration. Currently it only contains
 a list of resource limits of the cluster.
 
-Each resource limits contains three fields: resourceType, maximum and minimum.
-resourceType is the resource type to configure, e.g. "cpu", "memory" or
-a gpu-specific string (please run
-`gcloud compute accelerator-types list` to see the available ones).
-Maximum is the maximum amount with the unit of the resource.
-Minimum is the minimum amount with the unit of the resource.
+Each resource limits definition contains three fields:
+resourceType, maximum and minimum.
+Resource type can be "cpu", "memory" or an accelerator (e.g.
+"nvidia-tesla-k80" for nVidia Tesla K80). Use gcloud compute accelerator-types
+list to learn about available accelerator types.
+Maximum is the maximum allowed amount with the unit of the resource.
+Minimum is the minimum allowed amount with the unit of the resource.
 """)
 
   from_flags_group = limits_group.add_argument_group('Flags to configure '
@@ -380,8 +426,7 @@ Minimum number of gigabytes of memory to which the cluster can scale.""",
       metavar='type=TYPE,count=COUNT',
       hidden=hidden,
       help="""\
-Sets maximum limit for a single type of accelerators (e.g. GPUs) in cluster. Defaults
-to 0 for all accelerator types if it isn't set.
+Sets maximum limit for a single type of accelerators (e.g. GPUs) in cluster.
 
 *type*::: (Required) The specific type (e.g. nvidia-tesla-k80 for nVidia Tesla K80)
 of accelerator for which the limit is set. Use ```gcloud compute
@@ -411,7 +456,7 @@ to which the cluster can be scaled.
 """)
 
 
-def AddEnableBinAuthzFlag(parser, hidden=True):
+def AddEnableBinAuthzFlag(parser, hidden=False):
   """Adds a --enable-binauthz flag to parser."""
   help_text = """Enable Binary Authorization for this cluster."""
   parser.add_argument(
@@ -665,7 +710,10 @@ more info."""
       help=help_text)
 
 
-def AddEnableAutoUpgradeFlag(parser, for_node_pool=False, suppressed=False):
+def AddEnableAutoUpgradeFlag(parser,
+                             for_node_pool=False,
+                             suppressed=False,
+                             default=None):
   """Adds a --enable-autoupgrade flag to parser."""
   if for_node_pool:
     help_text = """\
@@ -680,17 +728,23 @@ Sets autoupgrade feature for a cluster's default node-pool(s).
   $ {command} example-cluster --enable-autoupgrade
 """
   help_text += """
-See https://cloud.google.com/kubernetes-engine/docs/node-management for more \
+See https://cloud.google.com/kubernetes-engine/docs/node-auto-upgrades for more \
 info."""
 
   parser.add_argument(
       '--enable-autoupgrade',
       action='store_true',
-      default=None,
+      default=default,
       help=help_text,
       hidden=suppressed)
 
 
+# Warn GA customers about the future node auto-upgrade default value change.
+# At the same time we will release the actual change to Alpha/Beta in parallel.
+def WarnGAForFutureAutoUpgradeChange():
+  log.warning(util.WARN_GA_FUTURE_AUTOUPGRADE_CHANGE)
+
+
 def AddTagsFlag(parser, help_text):
   """Adds a --tags to the given parser."""
   parser.add_argument(
@@ -710,24 +764,21 @@ def AddMasterAuthorizedNetworksFlags(parser, enable_group_for_update=None):
     parser: A given parser.
     enable_group_for_update: An optional group of mutually exclusive flag
         options to which an --enable-master-authorized-networks flag is added
-        in an update command. If given, the flag will default to None instead
-        of False.
+        in an update command.
   """
   if enable_group_for_update is None:
     # Flags are being added to the same group.
     master_flag_group = parser.add_argument_group('Master Authorized Networks')
     enable_flag_group = master_flag_group
-    enable_default = False
   else:
     # Flags are being added to different groups, so the new one should have no
     # help text (has only one arg).
     master_flag_group = parser.add_argument_group('')
     enable_flag_group = enable_group_for_update
-    enable_default = None
 
   enable_flag_group.add_argument(
       '--enable-master-authorized-networks',
-      default=enable_default,
+      default=None,
       help="""\
 Allow only specified set of CIDR blocks (specified by the
 `--master-authorized-networks` flag) to connect to Kubernetes master through
@@ -736,15 +787,13 @@ HTTPS. Besides these blocks, the following have access as well:\n
   `--enable-private-nodes` is specified.
   2) Google Compute Engine Public IPs if `--enable-private-nodes` is not
   specified.\n
-When disabled, public internet (0.0.0.0/0) is allowed to connect to Kubernetes
-master through HTTPS.
+Use `--no-enable-master-authorized-networks` to disable. When disabled, public
+internet (0.0.0.0/0) is allowed to connect to Kubernetes master through HTTPS.
 """,
       action='store_true')
   master_flag_group.add_argument(
       '--master-authorized-networks',
-      type=arg_parsers.ArgList(
-          min_length=1,
-          max_length=api_adapter.MAX_AUTHORIZED_NETWORKS_CIDRS),
+      type=arg_parsers.ArgList(min_length=1),
       metavar='NETWORK',
       help='The list of CIDR blocks (up to {max}) that are allowed to connect '
       'to Kubernetes master through HTTPS. Specified in CIDR notation (e.g. '
@@ -766,41 +815,51 @@ def AddNetworkPolicyFlags(parser, hidden=False):
       '--update-addons=NetworkPolicy=ENABLED flag.')
 
 
-def AddPrivateClusterFlags(parser, hidden=False):
+def AddPrivateClusterFlags(parser, with_deprecated=False, with_alpha=False):
   """Adds flags related to private clusters to parser."""
   group = parser.add_argument_group('Private Clusters')
-  group.add_argument(
-      '--private-cluster',
-      help=('Cluster is created with no public IP addresses on the cluster '
-            'nodes.'),
-      default=None,
-      action=actions.DeprecationAction(
-          'private-cluster',
-          warn='The --private-cluster flag is deprecated and will be removed '
-               'in a future release. Use --enable-private-nodes instead.',
-          action='store_true'),
-      hidden=hidden)
+  if with_deprecated:
+    group.add_argument(
+        '--private-cluster',
+        help=('Cluster is created with no public IP addresses on the cluster '
+              'nodes.'),
+        default=None,
+        action=actions.DeprecationAction(
+            'private-cluster',
+            warn='The --private-cluster flag is deprecated and will be removed '
+            'in a future release. Use --enable-private-nodes instead.',
+            action='store_true'))
   group.add_argument(
       '--enable-private-nodes',
       help=('Cluster is created with no public IP addresses on the cluster '
             'nodes.'),
       default=None,
-      action='store_true',
-      hidden=hidden)
+      action='store_true')
   group.add_argument(
       '--enable-private-endpoint',
       help=('Cluster is managed using the private IP address of the master '
             'API endpoint.'),
       default=None,
-      action='store_true',
-      hidden=hidden)
+      action='store_true')
+  if with_alpha:
+    AddPeeringRouteSharingFlag(group)
   group.add_argument(
       '--master-ipv4-cidr',
-      help=('IPv4 CIDR range to use for the master network.  This should be a '
-            '/28 and should be used in conjunction with the '
+      help=('IPv4 CIDR range to use for the master network.  This should have '
+            'a netmask of size /28 and should be used in conjunction with the '
             '--enable-private-nodes flag.'),
+      default=None)
+
+
+def AddPeeringRouteSharingFlag(group):
+  group.add_argument(
+      '--enable-peering-route-sharing',
+      help=(
+          'Enable custom route sharing between the master and node VPCs, which '
+          'ensures clients running in networks connected via a Cloud Router, '
+          'VPN, or Interconnect can reach the API server.'),
       default=None,
-      hidden=hidden)
+      action='store_true')
 
 
 def AddEnableLegacyAuthorizationFlag(parser, hidden=False):
@@ -821,7 +880,7 @@ instead, create or update your cluster with the option
       help=help_text)
 
 
-def AddAuthenticatorSecurityGroupFlags(parser, hidden=True):
+def AddAuthenticatorSecurityGroupFlags(parser, hidden=False):
   """Adds --security-group to parser."""
   help_text = """\
 The name of the RBAC security group for use with Google security groups
@@ -1121,12 +1180,10 @@ def AddMaxPodsPerNodeFlag(parser, for_node_pool=False, hidden=False):
                    False if it's applied to a cluster.
     hidden: Whether or not to hide the help text.
   """
-
-  if for_node_pool:
-    parser.add_argument(
-        '--max-pods-per-node',
-        default=None,
-        help="""\
+  parser.add_argument(
+      '--max-pods-per-node',
+      default=None,
+      help="""\
 The max number of pods per node for this node pool.
 
 This flag sets the maximum number of pods that can be run at the same time on a
@@ -1135,9 +1192,9 @@ set at the cluster level.
 
 Must be used in conjunction with '--enable-ip-alias'.
 """,
-        hidden=hidden,
-        type=int)
-  else:
+      hidden=hidden,
+      type=int)
+  if not for_node_pool:
     parser.add_argument(
         '--default-max-pods-per-node',
         default=None,
@@ -1168,6 +1225,7 @@ When specified, the nodes for the new node pool will be scheduled on host with
 specified CPU architecture or a newer one.
 
 Examples:
+
   $ {command} node-pool-1 --cluster=example-cluster --min-cpu-platform=PLATFORM
 
 """
@@ -1177,6 +1235,7 @@ When specified, the nodes for the new cluster's default node pool will be
 scheduled on host with specified CPU architecture or a newer one.
 
 Examples:
+
   $ {command} example-cluster --min-cpu-platform=PLATFORM
 
 """
@@ -1216,6 +1275,12 @@ is scheduled to be deprecated in the future and later removed.
                     'while the bootstrapping process for cluster nodes is '
                     'being redesigned with significant security improvements.',
           'EXPOSED': 'Exposes all VM metadata to workloads.',
+          'GKE_METADATA_SERVER':
+              'Run the Kubernetes Engine Metadata Server on this node. The Kubernetes '
+              'Engine Metadata Server exposes a metadata API to workloads that is '
+              'compatible with the V1 Compute Metadata APIs exposed by the Compute Engine '
+              'and App Engine Metadata Servers. This feature can only be enabled if '
+              'Workload Identity is enabled at the cluster level.',
           'UNSPECIFIED': 'Chooses the default.',
       },
       type=lambda x: x.upper(),
@@ -1229,6 +1294,7 @@ def AddTagOrDigestPositional(parser,
                              tags_only=False,
                              arg_name=None,
                              metavar=None):
+  """Adds a tag or digest positional arg."""
   digest_str = '*.gcr.io/PROJECT_ID/IMAGE_PATH@sha256:DIGEST or'
   if tags_only:
     digest_str = ''
@@ -1369,30 +1435,20 @@ behavior.
       help="""\
 Specifies scopes for the node instances. Examples:
 
-$ {{command}} {example_target} --scopes=https://www.googleapis.com/auth/devstorage.read_only
+  $ {{command}} {example_target} --scopes=https://www.googleapis.com/auth/devstorage.read_only
 
-$ {{command}} {example_target} --scopes=bigquery,storage-rw,compute-ro
+  $ {{command}} {example_target} --scopes=bigquery,storage-rw,compute-ro
 
 Multiple SCOPEs can be specified, separated by commas. `logging-write`
 and/or `monitoring` are added unless Cloud Logging and/or Cloud Monitoring
 are disabled (see `--enable-cloud-logging` and `--enable-cloud-monitoring`
 for more information).
 {track_help}
-SCOPE can be either the full URI of the scope or an alias. Available
-aliases are:
-
-[format="csv",options="header"]
-|========
-Alias,URI
-{aliases}
-|========
-
-{scope_deprecation_msg}
+{scopes_help}
 """.format(
-    aliases=compute_constants.ScopesForHelp(),
-    scope_deprecation_msg=compute_constants.DEPRECATED_SCOPES_MESSAGES,
     example_target=example_target,
-    track_help=track_help))
+    track_help=track_help,
+    scopes_help=compute_constants.ScopesHelp()))
 
   cloud_endpoints_help_text = """\
 Automatically enable Google Cloud Endpoints to take advantage of API management
@@ -1506,6 +1562,11 @@ def AddAlphaAddonsFlags(parser):
   AddAddonsFlagsWithOptions(parser, api_adapter.ALPHA_ADDONS_OPTIONS)
 
 
+def AddBetaAddonsFlags(parser):
+  """Adds the --addons flag to the parser for the beta track."""
+  AddAddonsFlagsWithOptions(parser, api_adapter.BETA_ADDONS_OPTIONS)
+
+
 def AddPodSecurityPolicyFlag(parser, hidden=False):
   """Adds a --enable-pod-security-policy flag to parser."""
   help_text = """\
@@ -1540,12 +1601,14 @@ When enabled, `--cluster-ipv4-cidr` must be fully specified (e.g. `10.96.0.0/14`
       help=help_text)
 
 
-def AddTpuFlags(parser, hidden=False):
+def AddTpuFlags(parser, hidden=False, enable_tpu_service_networking=False):
   """Adds flags related to TPUs to the parser.
 
   Args:
     parser: A given parser.
     hidden: Whether or not to hide the help text.
+    enable_tpu_service_networking: Whether to add the
+    enable_tpu_service_networking flag.
   """
 
   tpu_group = parser.add_group(help='Flags relating to Cloud TPUs:')
@@ -1557,11 +1620,26 @@ def AddTpuFlags(parser, hidden=False):
       help="""\
 Enable Cloud TPUs for this cluster.
 
-Can not be specified unless `--enable-kubernetes-alpha` and `--enable-ip-alias`
-are also specified.
+Can not be specified unless `--enable-ip-alias` is also specified.
 """)
 
-  tpu_group.add_argument(
+  group = tpu_group
+
+  if enable_tpu_service_networking:
+    group = tpu_group.add_mutually_exclusive_group()
+    group.add_argument(
+        '--enable-tpu-service-networking',
+        action='store_true',
+        hidden=hidden,
+        help="""\
+Enable Cloud TPU's Service Networking mode. In this mode, the CIDR blocks used
+by the Cloud TPUs will be allocated and managed by Service Networking, instead
+of Kubernetes Engine.
+
+This cannot be specified if `tpu-ipv4-cidr` is specified.
+""")
+
+  group.add_argument(
       '--tpu-ipv4-cidr',
       metavar='CIDR',
       hidden=hidden,
@@ -1608,14 +1686,14 @@ def AddIstioConfigFlag(parser, suppressed=False):
 Configurations for Istio addon, requires --addons contains Istio for create,
 or --update-addons Istio=ENABLED for update.
 
-*auth*:::Optional Type of auth NONE or MUTUAL_TLS
+*auth*:::Optional Type of auth MTLS_PERMISSIVE or MTLS_STRICT
 Example:
 
-  $ {command} example-cluster --istio-config=auth=NONE
+  $ {command} example-cluster --istio-config=auth=MTLS_PERMISSIVE
 """
   parser.add_argument(
       '--istio-config',
-      metavar='auth=NONE',
+      metavar='auth=MTLS_PERMISSIVE',
       type=arg_parsers.ArgDict(
           spec={
               'auth': (lambda x: x.upper()),
@@ -1631,15 +1709,15 @@ def ValidateIstioConfigCreateArgs(istio_config_args, addons_args):
     istio_config_args: parsed comandline arguments for --istio_config.
     addons_args: parsed comandline arguments for --addons.
   Raises:
-    InvalidArgumentException: when auth is not NONE nor MUTUAL_TLS, or
-    --addon=Istio is not specified
+    InvalidArgumentException: when auth is not MTLS_PERMISSIVE nor MTLS_STRICT,
+    or --addon=Istio is not specified
   """
   if istio_config_args:
     auth = istio_config_args.get('auth', '')
-    if auth not in ['NONE', 'MUTUAL_TLS']:
+    if auth not in ['MTLS_PERMISSIVE', 'MTLS_STRICT']:
       raise exceptions.InvalidArgumentException(
-          '--istio-config', 'auth is either NONE or MUTUAL_TLS '
-          'e.g. --istio-config auth=NONE')
+          '--istio-config', 'auth is either MTLS_PERMISSIVE or MTLS_STRICT'
+          'e.g. --istio-config auth=MTLS_PERMISSIVE')
     if 'Istio' not in addons_args:
       raise exceptions.InvalidArgumentException(
           '--istio-config', '--addon=Istio must be specified when '
@@ -1653,15 +1731,15 @@ def ValidateIstioConfigUpdateArgs(istio_config_args, disable_addons_args):
     istio_config_args: parsed comandline arguments for --istio_config.
     disable_addons_args: parsed comandline arguments for --update-addons.
   Raises:
-    InvalidArgumentException: when auth is not NONE nor MUTUAL_TLS, or
-    --update-addons=Istio=ENABLED is not specified
+    InvalidArgumentException: when auth is not MTLS_PERMISSIVE nor MTLS_STRICT,
+    or --update-addons=Istio=ENABLED is not specified
   """
   if istio_config_args:
     auth = istio_config_args.get('auth', '')
-    if auth not in ['NONE', 'MUTUAL_TLS']:
+    if auth not in ['MTLS_PERMISSIVE', 'MTLS_STRICT']:
       raise exceptions.InvalidArgumentException(
-          '--istio-config', 'auth must be one of NONE or MUTUAL_TLS '
-          'e.g. --istio-config auth=NONE')
+          '--istio-config', 'auth must be one of MTLS_PERMISSIVE or '
+          'MTLS_STRICT e.g. --istio-config auth=MTLS_PERMISSIVE')
     disable_istio = disable_addons_args.get('Istio')
     if disable_istio is None or disable_istio:
       raise exceptions.InvalidArgumentException(
@@ -1693,6 +1771,25 @@ def WarnForUnspecifiedIpAllocationPolicy(args):
         'to suppress this warning.')
 
 
+def WarnForNodeModification(args, enable_autorepair):
+  if (args.image_type or '').lower() != 'ubuntu':
+    return
+  if enable_autorepair or args.enable_autoupgrade:
+    log.warning('Modifications on the boot disks of node VMs do not persist '
+                'across node recreations. Nodes are recreated during '
+                'manual-upgrade, auto-upgrade, auto-repair, and auto-scaling. '
+                'To preserve modifications across node recreation, use a '
+                'DaemonSet.')
+
+
+def WarnForAutoUpgrade(args):
+  if not args.IsSpecified('enable_autoupgrade'):
+    log.warning(util.WARN_AUTOUPGRADE_ENABLED_BY_DEFAULT)
+
+  if args.IsSpecified('node_version') and args.enable_autoupgrade:
+    log.warning(util.WARN_NODE_VERSION_WITH_AUTOUPGRADE_ENABLED)
+
+
 def AddMachineTypeFlag(parser):
   """Adds --machine-type flag to the parser.
 
@@ -1707,8 +1804,8 @@ The list of predefined machine types is available using the following command:
   $ gcloud compute machine-types list
 
 You can also specify custom machine types with the string "custom-CPUS-RAM"
-where ``CPUS`` is the number of virtual CPUs and ``RAM`` is the amount of RAM in
-MiB.
+where ```CPUS``` is the number of virtual CPUs and ```RAM``` is the amount of
+RAM in MiB.
 
 For example, to create a node pool using custom machines with 2 vCPUs and 12 GB
 of RAM:
@@ -1721,9 +1818,9 @@ of RAM:
       help=help_text)
 
 
-def AddManagedPodIdentityFlag(parser):
-  """Adds --enable-managed-pod-identity flag to the parser."""
-  help_text = """\
+def AddManagedPodIdentityFlags(parser):
+  """Adds Managed Pod Identity flags to the parser."""
+  enable_help_text = """\
 Enable Managed Pod Identity on the cluster.
 
 When enabled, pods with cloud.google.com/service-account annotations will be
@@ -1732,18 +1829,81 @@ specified in the annotation.
 """
   parser.add_argument(
       '--enable-managed-pod-identity',
-      action='store_true',
+      action=actions.DeprecationAction(
+          '--[no-]enable-managed-pod-identity',
+          warn="""\
+Alpha flag `--[no-]enable-managed-pod-identity` is deprecated and will be removed
+in a future release.
+
+Instead, use the beta `--identity-namespace` flag:
+
+    $ gcloud beta container clusters create --identity-namespace=PROJECT_NAME.svc.id.goog
+""",
+          removed=False,
+          action='store_true'),
       default=False,
-      # TODO(b/109942548): unhide this flag for Beta
       hidden=True,
-      help=help_text)
+      help=enable_help_text)
+  sa_help_text = """\
+Federating Service Account to use with Managed Pod Identity.
+
+Sets the name (email) of the GCP Service Account used to connect
+Kubernetes Service Accounts to GCP Service Accounts.
+
+Must be set with `--enable-managed-pod-identity`.
+"""
+  parser.add_argument(
+      '--federating-service-account',
+      action=actions.DeprecationAction(
+          '--federating-service-account',
+          warn="""\
+Alpha flag `--federating-service-account` is deprecated and will be removed
+in a future release.
+
+Instead, use the beta `--identity-namespace` flag:
+
+    $ gcloud beta container clusters create --identity-namespace=PROJECT_NAME.svc.id.goog
+""",
+          removed=False),
+      default=None,
+      hidden=True,
+      help=sa_help_text)
 
 
-def AddResourceUsageBigqueryDatasetFlag(parser, add_clear_flag=False):
+def AddWorkloadIdentityFlags(parser):
+  """Adds Workload Identity flags to the parser."""
+  # TODO(b/126751755): Once we have a beta docs page, update the documentation
+  # link in the help text.
+  parser.add_argument(
+      '--identity-namespace',
+      default=None,
+      hidden=True,
+      help="""\
+Enable Workload Identity on the cluster.
+
+When enabled, Kubernetes service accounts will be able to act as Cloud IAM
+Service Accounts, through the provided identity namespace.
+
+Currently, the only accepted identity namespace is the identity namespace of
+the Cloud project containing the cluster, `PROJECT_NAME.svc.id.goog`.
+""")
+
+
+def AddResourceUsageExportFlags(parser, add_clear_flag=False, hidden=False):
   """Adds flags about exporting cluster resource usage to BigQuery."""
 
-  group = parser.add_mutually_exclusive_group(
-      "Exports cluster's usage of cloud resources")
+  group = parser.add_group(
+      "Exports cluster's usage of cloud resources",
+      hidden=hidden)
+  if add_clear_flag:
+    group.is_mutex = True
+    group.add_argument(
+        '--clear-resource-usage-bigquery-dataset',
+        action='store_true',
+        hidden=hidden,
+        default=None,
+        help='Disables exporting cluster resource usage to BigQuery.')
+    group = group.add_group()
 
   dataset_help_text = """\
 The name of the BigQuery dataset to which the cluster's usage of cloud
@@ -1759,14 +1919,80 @@ Example:
   group.add_argument(
       '--resource-usage-bigquery-dataset',
       default=None,
+      hidden=hidden,
       help=dataset_help_text)
 
-  if add_clear_flag:
-    group.add_argument(
-        '--clear-resource-usage-bigquery-dataset',
-        action='store_true',
-        default=None,
-        help='Disables exporting cluster resource usage to BigQuery.')
+  network_egress_help_text = """`
+Enable network egress metering on this cluster.
+
+When enabled, a DaemonSet is deployed into the cluster. Each DaemonSet pod
+meters network egress traffic by collecting data from the conntrack table, and
+exports the metered metrics to the specified destination.
+
+Network egress metering is disabled if this flag is omitted, or when
+`--no-enable-network-egress-metering` is set.
+"""
+  group.add_argument(
+      '--enable-network-egress-metering',
+      action='store_true',
+      default=None,
+      help=network_egress_help_text)
+
+
+def AddEnablePrivateIpv6AccessFlag(parser, hidden=False):
+  """Adds --enable-private-ipv6-access flag to the parser.
+
+  When enabled, this allows gRPC clients on this cluster's pods a fast
+  path to access Google hosted services (eg. Cloud Spanner,
+  Cloud Dataflow, Cloud Bigtable)
+  This is currently only available on Alpha clusters, and needs
+  '--enable-kubernetes-alpha' to be specified also.
+
+  Args:
+    parser: A given parser.
+    hidden: If true, suppress help text for added options.
+  """
+  parser.add_argument(
+      '--enable-private-ipv6-access',
+      default=None,
+      help="""\
+Enables private access to Google services over IPv6.
+
+When enabled, this allows gRPC clients on this cluster's pods a fast path to
+access Google hosted services (eg. Cloud Spanner, Cloud Dataflow, Cloud
+Bigtable).
+
+This is currently only available on Alpha clusters, specified by using
+--enable-kubernetes-alpha.
+      """,
+      hidden=hidden,
+      action='store_true')
+
+
+def AddEnableIntraNodeVisibilityFlag(parser, hidden=False):
+  """Adds --enable-intra-node-visibility flag to the parser.
+
+  When enabled, the intra-node traffic is visible to VPC network.
+
+  Args:
+    parser: A given parser.
+    hidden: If true, suppress help text for added options.
+  """
+  parser.add_argument(
+      '--enable-intra-node-visibility',
+      default=None,
+      hidden=hidden,
+      action='store_true',
+      help="""\
+Enable Intra-node visibility for this cluster.
+
+Enabling intra-node visibility makes your intra-node pod to pod traffic
+visible to the networking fabric. With this feature, you can use VPC flow
+logging or other VPC features for intra-node traffic.
+
+This is a beta feature, enabling it on existing cluster causes the cluster
+master and the cluster nodes to restart, which might cause disruption.
+""")
 
 
 def AddVerticalPodAutoscalingFlag(parser, hidden=False):
@@ -1915,8 +2141,8 @@ controller or webhook are installed.
 def AddNodeGroupFlag(parser):
   """Adds --node-group flag to the parser."""
   help_text = """\
-Assign instances of this pool to run on the specified GCE node group.
-This is useful for running workloads on sole tenant nodes.
+Assign instances of this pool to run on the specified Google Compute Engine
+node group. This is useful for running workloads on sole tenant nodes.
 
 To see available sole tenant node-groups, run:
 
@@ -1933,10 +2159,28 @@ information on sole tenancy and node groups.
 
   parser.add_argument(
       '--node-group',
-      hidden=True,
       help=help_text)
 
 
+def AddInitialNodePoolNameArg(parser, hidden=True):
+  """Adds --node-pool-name argument to the parser."""
+  help_text = """\
+Name of the initial node pool that will be created for the cluster.
+
+Specifies the name to use for the initial node pool that will be created
+with the cluster.  If the settings specified require multiple node pools
+to be created, the name for each pool will be prefixed by this name.  For
+example running the following will result in three node pools being
+created, example-node-pool-0, example-node-pool-1 and
+example-node-pool-2:
+
+  $ {command} example-cluster --num-nodes 9 --max-nodes-per-pool 3 \
+    --node-pool-name example-node-pool
+"""
+
+  parser.add_argument('--node-pool-name', hidden=hidden, help=help_text)
+
+
 def AddMetadataFlags(parser):
   """Adds --metadata and --metadata-from-file flags to the given parser."""
   metadata_help = """\
@@ -1947,23 +2191,24 @@ def AddMetadataFlags(parser):
       Metadata keys must be unique and less than 128 bytes in length. Values
       must be less than or equal to 32,768 bytes in length. The total size of
       all keys and values must be less than 512 KB. Multiple arguments can be
-      passed to this flag, e.g.,
-      ``--metadata key-1=value-1,key-2=value-2,key-3=value-3''.
+      passed to this flag. For example:
+
+      ``--metadata key-1=value-1,key-2=value-2,key-3=value-3''
 
       Additionally, the following keys are reserved for use by Kubernetes
       Engine:
 
-      "cluster-location"
-      "cluster-name"
-      "cluster-uid"
-      "configure-sh"
-      "enable-os-login"
-      "gci-update-strategy"
-      "gci-ensure-gke-docker"
-      "instance-template"
-      "kube-env"
-      "startup-script"
-      "user-data"
+      * ``cluster-location''
+      * ``cluster-name''
+      * ``cluster-uid''
+      * ``configure-sh''
+      * ``enable-os-login''
+      * ``gci-update-strategy''
+      * ``gci-ensure-gke-docker''
+      * ``instance-template''
+      * ``kube-env''
+      * ``startup-script''
+      * ``user-data''
 
       See also Compute Engine's
       link:https://cloud.google.com/compute/docs/storing-retrieving-metadata[documentation]
@@ -1989,3 +2234,41 @@ def AddMetadataFlags(parser):
       default={},
       help=metadata_from_file_help,
       metavar='KEY=LOCAL_FILE_PATH')
+
+
+def AddEnableShieldedContainersFlags(parser):
+  """Adds a --enable-shielded-containers flag to the given parser."""
+  help_text = """\
+Enable shielded containers for this cluster. Enabling shielded containers
+will enable a more secure Node credential bootstrapping implementation.
+"""
+  parser.add_argument(
+      '--enable-shielded-containers',
+      action='store_true',
+      help=help_text,
+      hidden=True)
+
+
+def AddSurgeUpgradeFlag(parser, hidden=False):
+  """Adds surge upgrade related flag to the parser.
+
+  Args:
+    parser: A given parser.
+    hidden: Whether or not to hide the help text.
+  """
+
+  max_surge_help = """\
+Number of extra (surge) nodes to be created on each upgrade of a node pool.
+
+Specifies the number of extra (surge) nodes to be created during this node
+pool's upgrades. For example, running the following command will result in
+creating an extra node each time a node is upgraded:
+
+  $ {command} example-cluster --max-surge-upgrade 1
+"""
+  parser.add_argument(
+      '--max-surge-upgrade',
+      type=int,
+      default=0,
+      help=max_surge_help,
+      hidden=hidden)
