@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2013 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,13 +15,17 @@
 
 """Error Reporting Handler."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 import sys
 import traceback
 
 from apitools.base.py import exceptions as apitools_exceptions
 from googlecloudsdk.api_lib.error_reporting import util
 from googlecloudsdk.api_lib.util import apis as core_apis
-from googlecloudsdk.calliope import backend
+from googlecloudsdk.calliope import command_loading
 from googlecloudsdk.command_lib import error_reporting_util
 from googlecloudsdk.core import config
 from googlecloudsdk.core import http
@@ -39,7 +44,7 @@ def _IsInstallationCorruption(err):
   Returns:
     bool, True if installation error, False otherwise
   """
-  return (isinstance(err, backend.CommandLoadFailure) and
+  return (isinstance(err, command_loading.CommandLoadFailure) and
           isinstance(err.root_exception, ImportError))
 
 
@@ -74,48 +79,57 @@ def _PrintInstallationAction(err, err_string):
       ).format(err.command, err_string, sys.executable))
 
 
-CRASH_SERVICE = 'gcloud'
-ERROR_SERVICE = 'gcloud-user-error'
-CRASH_PROJECT = 'cloud-sdk-errors'
-CRASH_API_KEY = 'AIzaSyA45D7bA0Y1vyLmQ_Gl10G149M8jiwwK-s'
+ERROR_PROJECT = 'cloud-sdk-user-errors'
+ERROR_API_KEY = 'AIzaSyCUuWyME_r4XylltWNeydEjKSkgXkvpVyU'
+SERVICE = 'gcloud'
+CRASH_PROJECT = 'cloud-sdk-crashes'
+CRASH_API_KEY = 'AIzaSyAp4DSI_Z3-mK-B8U0t7GE34n74OWDJmak'
 
 
-def _GetReportingClient():
+def _GetReportingClient(is_crash=True):
   """Returns a client that uses an API key for Cloud SDK crash reports.
 
+  Args:
+     is_crash: bool, True use CRASH_API_KEY, if False use ERROR_API_KEY.
+
   Returns:
     An error reporting client that uses an API key for Cloud SDK crash reports.
   """
   client_class = core_apis.GetClientClass(util.API_NAME, util.API_VERSION)
   client_instance = client_class(get_credentials=False, http=http.Http())
-  client_instance.AddGlobalParam('key', CRASH_API_KEY)
+  if is_crash:
+    client_instance.AddGlobalParam('key', CRASH_API_KEY)
+  else:
+    client_instance.AddGlobalParam('key', ERROR_API_KEY)
   return client_instance
 
 
-def ReportError(err, is_crash):
+def ReportError(is_crash):
   """Report the anonymous crash information to the Error Reporting service.
 
+  This will report the actively handled exception.
   Args:
-    err: Exception, the error that caused the crash.
     is_crash: bool, True if this is a crash, False if it is a user error.
   """
   if properties.VALUES.core.disable_usage_reporting.GetBool():
     return
 
-  stacktrace = traceback.format_exc(err)
+  # traceback prints the exception that is currently being handled
+  stacktrace = traceback.format_exc()
   stacktrace = error_reporting_util.RemovePrivateInformationFromTraceback(
       stacktrace)
   command = properties.VALUES.metrics.command_name.Get()
   cid = metrics.GetCIDIfMetricsEnabled()
 
-  client = _GetReportingClient()
+  client = _GetReportingClient(is_crash)
   reporter = util.ErrorReporting(client)
   try:
     method_config = client.projects_events.GetMethodConfig('Report')
     request = reporter.GenerateReportRequest(
         error_message=stacktrace,
-        service=CRASH_SERVICE if is_crash else ERROR_SERVICE,
-        version=config.CLOUD_SDK_VERSION, project=CRASH_PROJECT,
+        service=SERVICE,
+        version=config.CLOUD_SDK_VERSION,
+        project=CRASH_PROJECT if is_crash else ERROR_PROJECT,
         request_url=command, user=cid)
     http_request = client.projects_events.PrepareHttpRequest(
         method_config, request)
@@ -125,7 +139,7 @@ def ReportError(err, is_crash):
   except apitools_exceptions.Error as e:
     log.file_only_logger.error(
         'Unable to report crash stacktrace:\n{0}'.format(
-            console_attr.EncodeForConsole(e)))
+            console_attr.SafeText(e)))
 
 
 def HandleGcloudCrash(err):
@@ -134,14 +148,22 @@ def HandleGcloudCrash(err):
   Args:
     err: Exception err.
   """
-  err_string = console_attr.EncodeForConsole(err)
+  err_string = console_attr.SafeText(err)
   log.file_only_logger.exception('BEGIN CRASH STACKTRACE')
   if _IsInstallationCorruption(err):
     _PrintInstallationAction(err, err_string)
   else:
-    log.error(u'gcloud crashed ({0}): {1}'.format(
+    log.error('gcloud crashed ({0}): {1}'.format(
         getattr(err, 'error_name', type(err).__name__), err_string))
-    ReportError(err, is_crash=True)
+    if 'certificate verify failed' in err_string:
+      log.err.Print(
+          '\ngcloud\'s default CA certificates failed to verify your connection'
+          ', which can happen if you are behind a proxy or firewall.')
+      log.err.Print('To use a custom CA certificates file, please run the '
+                    'following command:')
+      log.err.Print(
+          '  gcloud config set core/custom_ca_certs_file /path/to/ca_certs')
+    ReportError(is_crash=True)
     log.err.Print('\nIf you would like to report this issue, please run the '
                   'following command:')
     log.err.Print('  gcloud feedback')
