@@ -12,16 +12,15 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-
 """Utilities for building the dataproc clusters CLI."""
 
 from __future__ import absolute_import
 from __future__ import division
 from __future__ import unicode_literals
 
+import collections
 from apitools.base.py import encoding
 
-from googlecloudsdk.api_lib.compute import constants as compute_constants
 from googlecloudsdk.api_lib.compute import utils as api_utils
 from googlecloudsdk.api_lib.dataproc import compute_helpers
 from googlecloudsdk.api_lib.dataproc import constants
@@ -32,15 +31,19 @@ from googlecloudsdk.calliope import arg_parsers
 from googlecloudsdk.calliope import base
 from googlecloudsdk.command_lib.compute.instances import flags as instances_flags
 from googlecloudsdk.command_lib.dataproc import flags
+from googlecloudsdk.command_lib.kms import resource_args as kms_resource_args
 from googlecloudsdk.command_lib.util.args import labels_util
 from googlecloudsdk.core import log
 from googlecloudsdk.core import properties
+from googlecloudsdk.core import yaml
+from googlecloudsdk.core.console import console_io
 from googlecloudsdk.core.util import times
 
-
 GENERATED_LABEL_PREFIX = 'goog-dataproc-'
 
 
+# beta is unused but still useful when we add new beta features
+# pylint: disable=unused-argument
 def ArgsForClusterRef(parser, beta=False, include_deprecated=True):
   """Register flags for creating a dataproc cluster.
 
@@ -54,6 +57,7 @@ def ArgsForClusterRef(parser, beta=False, include_deprecated=True):
   # 30m is backend timeout + 5m for safety buffer.
   flags.AddTimeoutFlag(parser, default='35m')
   flags.AddZoneFlag(parser, short_flags=include_deprecated)
+  flags.AddComponentFlag(parser)
 
   parser.add_argument(
       '--metadata',
@@ -138,6 +142,13 @@ def ArgsForClusterRef(parser, beta=False, include_deprecated=True):
       '--num-master-local-ssds',
       type=int,
       help='The number of local SSDs to attach to the master in a cluster.')
+  parser.add_argument(
+      '--num-preemptible-worker-local-ssds',
+      type=int,
+      help="""\
+      The number of local SSDs to attach to each preemptible worker in
+      a cluster.
+      """)
   parser.add_argument(
       '--initialization-actions',
       type=arg_parsers.ArgList(min_length=1),
@@ -160,12 +171,10 @@ def ArgsForClusterRef(parser, beta=False, include_deprecated=True):
       help="""\
       The number of master nodes in the cluster.
 
-      [format="csv",options="header"]
-      |========
-      Number of Masters,Cluster Mode
-      1,Standard
-      3,High Availability
-      |========
+      Number of Masters | Cluster Mode
+      --- | ---
+      1 | Standard
+      3 | High Availability
       """)
   parser.add_argument(
       '--properties',
@@ -180,23 +189,21 @@ Properties are mapped to configuration files by specifying a prefix, such as
 "core:io.serializations". The following are supported prefixes and their
 mappings:
 
-[format="csv",options="header"]
-|========
-Prefix,File,Purpose of file
-capacity-scheduler,capacity-scheduler.xml,Hadoop YARN Capacity Scheduler configuration
-core,core-site.xml,Hadoop general configuration
-distcp,distcp-default.xml,Hadoop Distributed Copy configuration
-hadoop-env,hadoop-env.sh,Hadoop specific environment variables
-hdfs,hdfs-site.xml,Hadoop HDFS configuration
-hive,hive-site.xml,Hive configuration
-mapred,mapred-site.xml,Hadoop MapReduce configuration
-mapred-env,mapred-env.sh,Hadoop MapReduce specific environment variables
-pig,pig.properties,Pig configuration
-spark,spark-defaults.conf,Spark configuration
-spark-env,spark-env.sh,Spark specific environment variables
-yarn,yarn-site.xml,Hadoop YARN configuration
-yarn-env,yarn-env.sh,Hadoop YARN specific environment variables
-|========
+Prefix | File | Purpose of file
+--- | --- | ---
+capacity-scheduler | capacity-scheduler.xml | Hadoop YARN Capacity Scheduler configuration
+core | core-site.xml | Hadoop general configuration
+distcp | distcp-default.xml | Hadoop Distributed Copy configuration
+hadoop-env | hadoop-env.sh | Hadoop specific environment variables
+hdfs | hdfs-site.xml | Hadoop HDFS configuration
+hive | hive-site.xml | Hive configuration
+mapred | mapred-site.xml | Hadoop MapReduce configuration
+mapred-env | mapred-env.sh | Hadoop MapReduce specific environment variables
+pig | pig.properties | Pig configuration
+spark | spark-defaults.conf | Spark configuration
+spark-env | spark-env.sh | Spark specific environment variables
+yarn | yarn-site.xml | Hadoop YARN configuration
+yarn-env | yarn-env.sh | Hadoop YARN specific environment variables
 
 See https://cloud.google.com/dataproc/docs/concepts/configuring-clusters/cluster-properties
 for more information.
@@ -221,44 +228,28 @@ Examples:
 The following *minimum scopes* are necessary for the cluster to function
 properly and are always added, even if not explicitly specified:
 
-[format="csv"]
-|========
-{minimum_scopes}
-|========
+  {minimum_scopes}
 
 If the `--scopes` flag is not specified, the following *default scopes*
 are also included:
 
-[format="csv"]
-|========
-{additional_scopes}
-|========
+  {additional_scopes}
 
 If you want to enable all scopes use the 'cloud-platform' scope.
 
-SCOPE can be either the full URI of the scope or an alias.
-Available aliases are:
-
-[format="csv",options="header"]
-|========
-Alias,URI
-{aliases}
-|========
-
-{scope_deprecation_msg}
+{scopes_help}
 """.format(
-    minimum_scopes='\n'.join(constants.MINIMUM_SCOPE_URIS),
-    additional_scopes='\n'.join(constants.ADDITIONAL_DEFAULT_SCOPE_URIS),
-    aliases=compute_helpers.SCOPE_ALIASES_FOR_HELP,
-    scope_deprecation_msg=compute_constants.DEPRECATED_SCOPES_MESSAGES))
+    minimum_scopes='\n  '.join(constants.MINIMUM_SCOPE_URIS),
+    additional_scopes='\n  '.join(constants.ADDITIONAL_DEFAULT_SCOPE_URIS),
+    scopes_help=compute_helpers.SCOPES_HELP))
 
   if include_deprecated:
     _AddDiskArgsDeprecated(parser)
   else:
     _AddDiskArgs(parser)
 
-  # The --no-address argument is present in both GA and Beta tracks,
-  # but it is hidden in the GA track.
+  # --no-address is an exception to the no negative-flag style guildline to be
+  # consistent with gcloud compute instances create --no-address
   parser.add_argument(
       '--no-address',
       action='store_true',
@@ -266,11 +257,13 @@ Alias,URI
       If provided, the instances in the cluster will not be assigned external
       IP addresses.
 
+      If omitted the instances in the cluster will each be assigned an
+      ephemeral external IP address.
+
       Note: Dataproc VMs need access to the Dataproc API. This can be achieved
       without external IP addresses using Private Google Access
       (https://cloud.google.com/compute/docs/private-google-access).
-      """,
-      hidden=not beta)
+      """)
 
   boot_disk_type_detailed_help = """\
       The type of the boot disk. The value must be ``pd-standard'' or
@@ -355,9 +348,22 @@ def _AddDiskArgsDeprecated(parser):
 
 def BetaArgsForClusterRef(parser):
   """Register beta-only flags for creating a Dataproc cluster."""
-  flags.AddComponentFlag(parser)
   flags.AddMinCpuPlatformArgs(parser, base.ReleaseTrack.BETA)
 
+  autoscaling_group = parser.add_argument_group()
+  flags.AddAutoscalingPolicyResourceArgForCluster(
+      autoscaling_group, api_version='v1beta2')
+
+  AddKerberosGroup(parser)
+
+  parser.add_argument(
+      '--enable-component-gateway',
+      action='store_true',
+      help="""\
+        Enable access to the web UIs of selected components on the cluster
+        through the component gateway.
+        """)
+
   parser.add_argument(
       '--max-idle',
       type=arg_parsers.Duration(),
@@ -415,6 +421,8 @@ def BetaArgsForClusterRef(parser):
         metavar='type=TYPE,[count=COUNT]',
         help=help_msg)
 
+  AddReservationAffinityGroup(parser)
+
 
 def GetClusterConfig(args,
                      dataproc,
@@ -502,15 +510,15 @@ def GetClusterConfig(args,
     args.properties[constants.ALLOW_ZERO_WORKERS_PROPERTY] = 'true'
 
   if args.properties:
-    software_config.properties = encoding.DictToMessage(
+    software_config.properties = encoding.DictToAdditionalPropertyMessage(
         args.properties, dataproc.messages.SoftwareConfig.PropertiesValue)
 
-  if beta:
-    if args.components:
-      software_config_cls = dataproc.messages.SoftwareConfig
-      software_config.optionalComponents.extend(list(map(
-          software_config_cls.OptionalComponentsValueListEntryValuesEnum,
-          args.components)))
+  if args.components:
+    software_config_cls = dataproc.messages.SoftwareConfig
+    software_config.optionalComponents.extend(
+        list(
+            map(software_config_cls.OptionalComponentsValueListEntryValuesEnum,
+                args.components)))
 
   gce_cluster_config = dataproc.messages.GceClusterConfig(
       networkUri=network_ref and network_ref.SelfLink(),
@@ -520,12 +528,17 @@ def GetClusterConfig(args,
       serviceAccountScopes=expanded_scopes,
       zoneUri=properties.VALUES.compute.zone.GetOrFail())
 
+  if beta:
+    reservation_affinity = GetReservationAffinity(args, dataproc)
+    gce_cluster_config.reservationAffinity = reservation_affinity
+
   if args.tags:
     gce_cluster_config.tags = args.tags
 
   if args.metadata:
-    flat_metadata = dict((k, v) for d in args.metadata for k, v in d.items())
-    gce_cluster_config.metadata = encoding.DictToMessage(
+    flat_metadata = collections.OrderedDict(
+        [(k, v) for d in args.metadata for k, v in d.items()])
+    gce_cluster_config.metadata = encoding.DictToAdditionalPropertyMessage(
         flat_metadata, dataproc.messages.GceClusterConfig.MetadataValue)
 
   master_accelerators = []
@@ -549,12 +562,9 @@ def GetClusterConfig(args,
           imageUri=image_ref and image_ref.SelfLink(),
           machineTypeUri=args.master_machine_type,
           accelerators=master_accelerators,
-          diskConfig=GetDiskConfig(
-              dataproc,
-              args.master_boot_disk_type,
-              master_boot_disk_size_gb,
-              args.num_master_local_ssds
-          )),
+          diskConfig=GetDiskConfig(dataproc, args.master_boot_disk_type,
+                                   master_boot_disk_size_gb,
+                                   args.num_master_local_ssds)),
       workerConfig=dataproc.messages.InstanceGroupConfig(
           numInstances=args.num_workers,
           imageUri=image_ref and image_ref.SelfLink(),
@@ -571,10 +581,32 @@ def GetClusterConfig(args,
   )
 
   if beta:
+    if args.kerberos_config_file or args.kerberos_root_principal_password_uri:
+      cluster_config.securityConfig = dataproc.messages.SecurityConfig()
+      if args.kerberos_config_file:
+        cluster_config.securityConfig.kerberosConfig = ParseKerberosConfigFile(
+            dataproc, args.kerberos_config_file)
+      else:
+        kerberos_config = dataproc.messages.KerberosConfig()
+        kerberos_config.enableKerberos = True
+        if args.kerberos_root_principal_password_uri:
+          kerberos_config.rootPrincipalPasswordUri = \
+            args.kerberos_root_principal_password_uri
+          kerberos_kms_ref = args.CONCEPTS.kerberos_kms_key.Parse()
+          kerberos_config.kmsKeyUri = kerberos_kms_ref.RelativeName()
+        cluster_config.securityConfig.kerberosConfig = kerberos_config
+
+  if beta:
+    if args.enable_component_gateway:
+      cluster_config.endpointConfig = dataproc.messages.EndpointConfig(
+          enableHttpPortAccess=args.enable_component_gateway)
+    if args.autoscaling_policy:
+      cluster_config.autoscalingConfig = dataproc.messages.AutoscalingConfig(
+          policyUri=args.CONCEPTS.autoscaling_policy.Parse().RelativeName())
+
     cluster_config.masterConfig.minCpuPlatform = args.master_min_cpu_platform
     cluster_config.workerConfig.minCpuPlatform = args.worker_min_cpu_platform
 
-  if beta:
     lifecycle_config = dataproc.messages.LifecycleConfig()
     changed_config = False
     if args.max_age is not None:
@@ -590,7 +622,7 @@ def GetClusterConfig(args,
     if changed_config:
       cluster_config.lifecycleConfig = lifecycle_config
 
-  if beta and hasattr(args.CONCEPTS, 'kms_key'):
+  if hasattr(args.CONCEPTS, 'kms_key'):
     kms_ref = args.CONCEPTS.kms_key.Parse()
     if kms_ref:
       encryption_config = dataproc.messages.EncryptionConfig()
@@ -611,6 +643,7 @@ def GetClusterConfig(args,
   if (args.num_preemptible_workers is not None or
       preemptible_worker_boot_disk_size_gb is not None or
       args.preemptible_worker_boot_disk_type is not None or
+      args.num_preemptible_worker_local_ssds is not None or
       (beta and args.worker_min_cpu_platform is not None)):
     cluster_config.secondaryWorkerConfig = (
         dataproc.messages.InstanceGroupConfig(
@@ -619,7 +652,7 @@ def GetClusterConfig(args,
                 dataproc,
                 args.preemptible_worker_boot_disk_type,
                 preemptible_worker_boot_disk_size_gb,
-                None,
+                args.num_preemptible_worker_local_ssds,
             )))
     if beta and args.worker_min_cpu_platform:
       cluster_config.secondaryWorkerConfig.minCpuPlatform = (
@@ -628,10 +661,7 @@ def GetClusterConfig(args,
   return cluster_config
 
 
-def GetDiskConfig(dataproc,
-                  boot_disk_type,
-                  boot_disk_size,
-                  num_local_ssds):
+def GetDiskConfig(dataproc, boot_disk_type, boot_disk_size, num_local_ssds):
   """Get dataproc cluster disk configuration.
 
   Args:
@@ -726,5 +756,220 @@ def DeleteGeneratedLabels(cluster, dataproc):
     if not labels:
       cluster.labels = None
     else:
-      cluster.labels = encoding.DictToMessage(
+      cluster.labels = encoding.DictToAdditionalPropertyMessage(
           labels, dataproc.messages.Cluster.LabelsValue)
+
+
+def AddReservationAffinityGroup(parser):
+  """Adds the argument group to handle reservation affinity configurations."""
+  group = parser.add_group(help='Manage the configuration of desired'
+                                'reservation which this instance could'
+                                'take capacity from.'
+                          )
+  group.add_argument(
+      '--reservation-affinity',
+      choices=['any', 'none', 'specific'],
+      default='any',
+      hidden=True,
+      help="""
+Specifies the configuration of desired reservation which this instance could
+take capacity from. Choices are 'any', 'none' and 'specific', default is 'any'.
+""")
+  group.add_argument(
+      '--reservation-label',
+      type=arg_parsers.ArgDict(spec={
+          'key': str,
+          'value': str,
+      }),
+      hidden=True,
+      help="""
+The key and values of the label of the reservation resource. Required if the
+value of `--reservation-affinity` is `specific`.
+
+*key*::: The label key of reservation resource.
+
+*value*::: The label value of reservation resource.
+""")
+
+
+def GetReservationAffinity(args, client):
+  """Returns the message of reservation affinity for the instance."""
+  if not args.IsSpecified('reservation_affinity'):
+    return None
+
+  type_msgs = (client.messages.
+               ReservationAffinity.ConsumeReservationTypeValueValuesEnum)
+
+  if args.reservation_affinity == 'none':
+    reservation_type = type_msgs.NO_RESERVATION
+    reservation_key = None
+    reservation_values = []
+  elif args.reservation_affinity == 'specific':
+    reservation_type = type_msgs.SPECIFIC_RESERVATION
+    # Currently, the key is fixed and the value is the name of the reservation.
+    # The value being a repeated field is reserved for future use when user
+    # can specify more than one reservation names from which the Vm can take
+    # capacity from.
+    reservation_key = args.reservation_label.get('key', None)
+    reservation_values = [args.reservation_label.get('value', None)]
+  else:
+    reservation_type = type_msgs.ANY_RESERVATION
+    reservation_key = None
+    reservation_values = []
+
+  return client.messages.ReservationAffinity(
+      consumeReservationType=reservation_type,
+      key=reservation_key,
+      values=reservation_values)
+
+
+def AddKerberosGroup(parser):
+  """Adds the argument group to handle Kerberos configurations."""
+  kerberos_group = parser.add_argument_group(
+      mutex=True,
+      help='Specifying these flags will enable Kerberos for the cluster.')
+  # Not mutually exclusive
+  kerberos_flag_group = kerberos_group.add_argument_group()
+  kerberos_flag_group.add_argument(
+      '--kerberos-root-principal-password-uri',
+      required=True,
+      help="""\
+        Google Cloud Storage URI of a KMS encrypted file containing
+        the root principal password. Must be a URL beginning with 'gs://'.
+        """)
+  # Add kerberos-kms-key args
+  kerberos_kms_flag_overrides = \
+      {'kms-key': '--kerberos-kms-key',
+       'kms-keyring': '--kerberos-kms-key-keyring',
+       'kms-location': '--kerberos-kms-key-location',
+       'kms-project': '--kerberos-kms-key-project'}
+  kms_resource_args.AddKmsKeyResourceArg(
+      kerberos_flag_group,
+      'password',
+      flag_overrides=kerberos_kms_flag_overrides,
+      required=True,
+      name='--kerberos-kms-key')
+
+  kerberos_group.add_argument(
+      '--kerberos-config-file',
+      help="""\
+Path to a YAML (or JSON) file containing the configuration for Kerberos on the
+cluster. If you pass `-` as the value of the flag the file content will be read
+from stdin.
+
+The YAML file is formatted as follows:
+
+```
+  # Optional. Flag to indicate whether to Kerberize the cluster.
+  # The default value is true.
+  enable_kerberos: true
+
+  # Required. The Google Cloud Storage URI of a KMS encrypted file
+  # containing the root principal password.
+  root_principal_password_uri: gs://bucket/password.encrypted
+
+  # Required. The URI of the KMS key used to encrypt various
+  # sensitive files.
+  kms_key_uri:
+    projects/myproject/locations/global/keyRings/mykeyring/cryptoKeys/my-key
+
+  # Configuration of SSL encryption. If specified, all sub-fields
+  # are required. Otherwise, Dataproc will provide a self-signed
+  # certificate and generate the passwords.
+  ssl:
+    # Optional. The Google Cloud Storage URI of the keystore file.
+    keystore_uri: gs://bucket/keystore.jks
+
+    # Optional. The Google Cloud Storage URI of a KMS encrypted
+    # file containing the password to the keystore.
+    keystore_password_uri: gs://bucket/keystore_password.encrypted
+
+    # Optional. The Google Cloud Storage URI of a KMS encrypted
+    # file containing the password to the user provided key.
+    key_password_uri: gs://bucket/key_password.encrypted
+
+    # Optional. The Google Cloud Storage URI of the truststore
+    # file.
+    truststore_uri: gs://bucket/truststore.jks
+
+    # Optional. The Google Cloud Storage URI of a KMS encrypted
+    # file containing the password to the user provided
+    # truststore.
+    truststore_password_uri:
+      gs://bucket/truststore_password.encrypted
+
+  # Configuration of cross realm trust.
+  cross_realm_trust:
+    # Optional. The remote realm the Dataproc on-cluster KDC will
+    # trust, should the user enable cross realm trust.
+    realm: REMOTE.REALM
+
+    # Optional. The KDC (IP or hostname) for the remote trusted
+    # realm in a cross realm trust relationship.
+    kdc: kdc.remote.realm
+
+    # Optional. The admin server (IP or hostname) for the remote
+    # trusted realm in a cross realm trust relationship.
+    admin_server: admin-server.remote.realm
+
+    # Optional. The Google Cloud Storage URI of a KMS encrypted
+    # file containing the shared password between the on-cluster
+    # Kerberos realm and the remote trusted realm, in a cross
+    # realm trust relationship.
+    shared_password_uri:
+      gs://bucket/cross-realm.password.encrypted
+
+  # Optional. The Google Cloud Storage URI of a KMS encrypted file
+  # containing the master key of the KDC database.
+  kdc_db_key_uri: gs://bucket/kdc_db_key.encrypted
+
+  # Optional. The lifetime of the ticket granting ticket, in
+  # hours. If not specified, or user specifies 0, then default
+  # value 10 will be used.
+  tgt_lifetime_hours: 1
+```
+        """)
+
+
+def ParseKerberosConfigFile(dataproc, kerberos_config_file):
+  """Parse a kerberos-config-file into the KerberosConfig message."""
+  data = console_io.ReadFromFileOrStdin(kerberos_config_file, binary=False)
+  try:
+    kerberos_config_data = yaml.load(data)
+  except Exception as e:
+    raise exceptions.ParseError('Cannot parse YAML:[{0}]'.format(e))
+
+  ssl_config = kerberos_config_data.get('ssl', {})
+  keystore_uri = ssl_config.get('keystore_uri')
+  truststore_uri = ssl_config.get('truststore_uri')
+  keystore_password_uri = ssl_config.get('keystore_password_uri')
+  key_password_uri = ssl_config.get('key_password_uri')
+  truststore_password_uri = ssl_config.get('truststore_password_uri')
+
+  cross_realm_trust_config = kerberos_config_data.get('cross_realm_trust', {})
+  cross_realm_trust_realm = cross_realm_trust_config.get('realm')
+  cross_realm_trust_kdc = cross_realm_trust_config.get('kdc')
+  cross_realm_trust_admin_server = cross_realm_trust_config.get('admin_server')
+  cross_realm_trust_shared_password_uri = cross_realm_trust_config.get(
+      'shared_password_uri')
+  kerberos_config_msg = dataproc.messages.KerberosConfig(
+      # Unless user explicitly disable kerberos in kerberos config,
+      # consider the existence of the kerberos config is enabling
+      # kerberos, explicitly or implicitly.
+      enableKerberos=kerberos_config_data.get('enable_kerberos', True),
+      rootPrincipalPasswordUri=kerberos_config_data.get(
+          'root_principal_password_uri'),
+      kmsKeyUri=kerberos_config_data.get('kms_key_uri'),
+      kdcDbKeyUri=kerberos_config_data.get('kdc_db_key_uri'),
+      tgtLifetimeHours=kerberos_config_data.get('tgt_lifetime_hours'),
+      keystoreUri=keystore_uri,
+      keystorePasswordUri=keystore_password_uri,
+      keyPasswordUri=key_password_uri,
+      truststoreUri=truststore_uri,
+      truststorePasswordUri=truststore_password_uri,
+      crossRealmTrustRealm=cross_realm_trust_realm,
+      crossRealmTrustKdc=cross_realm_trust_kdc,
+      crossRealmTrustAdminServer=cross_realm_trust_admin_server,
+      crossRealmTrustSharedPasswordUri=cross_realm_trust_shared_password_uri)
+
+  return kerberos_config_msg
