@@ -22,7 +22,7 @@ from googlecloudsdk.calliope import actions
 from googlecloudsdk.calliope import arg_parsers
 from googlecloudsdk.calliope.concepts import concepts
 from googlecloudsdk.calliope.concepts import deps
-from googlecloudsdk.command_lib.projects import resource_args as project_resource_args
+from googlecloudsdk.command_lib.export import util as export_util
 from googlecloudsdk.command_lib.util import completers
 from googlecloudsdk.command_lib.util.concepts import concept_parsers
 from googlecloudsdk.core import properties
@@ -32,7 +32,7 @@ def AddZoneFlag(parser, short_flags=True):
   """Add zone flag."""
   parser.add_argument(
       '--zone',
-      *['-z'] if short_flags else [],
+      *(['-z'] if short_flags else []),
       help="""
             The compute zone (e.g. us-central1-a) for the cluster. If empty
             and --region is set to a value other than `global`, the server will
@@ -54,60 +54,6 @@ def AddFileFlag(parser, input_type, action):
       required=True)
 
 
-def AddTemplateSourceFlag(parser, schema_path):
-  parser.add_argument(
-      '--source',
-      help="""Path to a YAML file containing a Dataproc WorkflowTemplate
-      resource. The provided YAML file must not contain id, version, or any
-      output-only fields.
-      Alternatively, you may omit this flag to read from standard input.
-      A schema describing the import format can be found in:
-      $CLOUDSDKROOT/lib/googlecloudsdk/api_lib/dataproc/schemas/{}.
-      """.format(schema_path),
-      # Allow reading from stdin.
-      required=False)
-
-
-def AddClusterSourceFlag(parser):
-  parser.add_argument(
-      '--source',
-      help="""Path to a YAML file containing the configuration of a cluster.
-      The provided configuration should be an instance of ClusterConfig.
-      Alternatively, you may omit this flag to read from standard input.
-      A schema describing the import format can be found in:
-      $CLOUDSDKROOT/lib/googlecloudsdk/api_lib/dataproc/schemas/v1beta2/Cluster.yaml
-      """,
-      # Allow reading from stdin.
-      required=False)
-
-
-def AddTemplateDestinationFlag(parser, schema_path):
-  parser.add_argument(
-      '--destination',
-      help="""Path to a YAML file to which the Dataproc WorkflowTemplate
-      resource will be exported. The exported template will not contain id,
-      version, or any output-only fields.
-      Alternatively, you may omit this flag to write to standard output.
-      A schema describing the export format can be found in:
-      $CLOUDSDKROOT/lib/googlecloudsdk/api_lib/dataproc/schemas/{}.
-      """.format(schema_path),
-      # Allow writing to stdout.
-      required=False)
-
-
-def AddClusterDestinationFlag(parser):
-  parser.add_argument(
-      '--destination',
-      help="""Path to a YAML file to which the cluster's configuration
-      will be exported.
-      Alternatively, you may omit this flag to write to standard output.
-      A schema describing the export format can be found in:
-      $CLOUDSDKROOT/lib/googlecloudsdk/api_lib/dataproc/schemas/v1beta2/Cluster.yaml
-      """,
-      # Allow writing to stdout.
-      required=False)
-
-
 def AddJobFlag(parser, action):
   parser.add_argument(
       'job', help='The ID of the job to {0}.'.format(action))
@@ -124,7 +70,7 @@ def AddTimeoutFlag(parser, default='10m'):
       '--timeout',
       type=arg_parsers.Duration(),
       default=default,
-      help=('Client side timeout on how long to wait for Datproc operations. '
+      help=('Client side timeout on how long to wait for Dataproc operations. '
             'See $ gcloud topic datetimes for information on duration '
             'formats.'),
       hidden=True)
@@ -178,7 +124,7 @@ def AddComponentFlag(parser):
       List of optional components to be installed on cluster machines.
 
       The following page documents the optional components that can be
-      installed.
+      installed:
       https://cloud.google.com/dataproc/docs/concepts/configuring-clusters/optional-components.
       """
   parser.add_argument(
@@ -209,7 +155,7 @@ def RegionAttributeConfig():
   return concepts.ResourceParameterAttributeConfig(
       name='region',
       help_text=(
-          'The Cloud DataProc region for the {resource}. Each Cloud Dataproc '
+          'Cloud Dataproc region for the {resource}. Each Cloud Dataproc '
           'region constitutes an independent resource namespace constrained to '
           'deploying instances into Google Compute Engine zones inside the '
           'region. The default value of `global` is a special multi-region '
@@ -230,7 +176,7 @@ def GetTemplateResourceSpec(api_version):
       api_version=api_version,
       resource_name='template',
       disable_auto_completers=False,
-      projectsId=project_resource_args.PROJECT_ATTRIBUTE_CONFIG,
+      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
       regionsId=RegionAttributeConfig(),
       workflowTemplatesId=TemplateAttributeConfig(),
   )
@@ -252,3 +198,119 @@ def AddTemplateResourceArg(parser, verb, api_version, positional=True):
       GetTemplateResourceSpec(api_version=api_version),
       'The name of the workflow template to {}.'.format(verb),
       required=True).AddToParser(parser)
+
+
+def _AutoscalingPolicyResourceSpec(api_version):
+  return concepts.ResourceSpec(
+      'dataproc.projects.regions.autoscalingPolicies',
+      api_version=api_version,
+      resource_name='autoscaling policy',
+      disable_auto_completers=False,
+      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
+      regionsId=RegionAttributeConfig(),
+      autoscalingPoliciesId=concepts.ResourceParameterAttributeConfig(
+          name='autoscaling_policy',
+          help_text='The autoscaling policy id.',
+      ),
+  )
+
+
+def AddAutoscalingPolicyResourceArg(parser, verb, api_version):
+  """Adds a workflow template resource argument.
+
+  Args:
+    parser: the argparse parser for the command.
+    verb: str, the verb to apply to the resource, such as 'to update'.
+    api_version: api version, for example v1 or v1beta2
+  """
+  concept_parsers.ConceptParser.ForResource(
+      'autoscaling_policy',
+      _AutoscalingPolicyResourceSpec(api_version),
+      'The autoscaling policy to {}.'.format(verb),
+      required=True).AddToParser(parser)
+
+
+def AddAutoscalingPolicyResourceArgForCluster(parser, api_version):
+  """Adds a workflow template resource argument.
+
+  Args:
+    parser: the argparse parser for the command.
+    api_version: api version, for example v1 or v1beta2
+  """
+  concept_parsers.ConceptParser.ForResource(
+      '--autoscaling-policy',
+      _AutoscalingPolicyResourceSpec(api_version),
+      'The autoscaling policy to use.',
+      command_level_fallthroughs={
+          'region': ['--region'],
+      },
+      flag_name_overrides={
+          'region': ''
+      },
+      required=False).AddToParser(parser)
+
+
+def AddImportArgs(parser, verb, api_version, resource_message_name):
+  AddAutoscalingPolicyResourceArg(parser, verb, api_version)
+
+  schema_path = export_util.GetSchemaPath(
+      'dataproc', api_version, resource_message_name, for_help=True)
+  export_util.AddImportFlags(parser, schema_path)
+
+
+def AddExportArgs(parser, verb, api_version, resource_message_name):
+  AddAutoscalingPolicyResourceArg(parser, verb, api_version)
+
+  schema_path = export_util.GetSchemaPath(
+      'dataproc', api_version, resource_message_name, for_help=True)
+  export_util.AddExportFlags(parser, schema_path)
+
+
+def AddListOperationsFormat(parser):
+  parser.display_info.AddTransforms(
+      {'operationState': _TransformOperationState,
+       'operationTimestamp': _TransformOperationTimestamp,
+       'operationType': _TransformOperationType,
+       'operationWarnings': _TransformOperationWarnings,
+      })
+  parser.display_info.AddFormat(
+      'table(name.segment():label=NAME, '
+      'metadata.operationTimestamp():label=TIMESTAMP,'
+      'metadata.operationType():label=TYPE, '
+      'metadata.operationState():label=STATE, '
+      'status.code.yesno(no=\'\'):label=ERROR, '
+      'metadata.operationWarnings():label=WARNINGS)')
+
+
+def _TransformOperationType(metadata):
+  """Extract operation type from metadata."""
+  if 'operationType' in metadata:
+    return metadata['operationType']
+  elif 'graph' in metadata:
+    return 'WORKFLOW'
+  return ''
+
+
+def _TransformOperationState(metadata):
+  """Extract operation state from metadata."""
+  if 'status' in metadata:
+    return metadata['status']['state']
+  elif 'state' in metadata:
+    return metadata['state']
+  return ''
+
+
+def _TransformOperationTimestamp(metadata):
+  """Extract operation start timestamp from metadata."""
+  if 'statusHistory' in metadata:
+    return metadata['statusHistory'][0]['stateStartTime']
+  elif 'startTime' in metadata:
+    return metadata['startTime']
+  return ''
+
+
+def _TransformOperationWarnings(metadata):
+  """Returns a count of operations if any are present."""
+  if 'warnings' in metadata:
+    return len(metadata['warnings'])
+  return ''
