@@ -20,6 +20,7 @@ from __future__ import division
 from __future__ import unicode_literals
 
 import abc
+import collections
 import os
 
 from apitools.base.py import encoding
@@ -40,7 +41,7 @@ class JobBase(six.with_metaclass(abc.ABCMeta, object)):
     super(JobBase, self).__init__(*args, **kwargs)
     self.files_by_type = {}
     self.files_to_stage = []
-    self._staging_dir = None  # type: str
+    self._staging_dir = None
 
   def _GetStagedFile(self, file_str):
     """Validate file URI and register it for uploading if it is local."""
@@ -98,10 +99,14 @@ class JobBase(six.with_metaclass(abc.ABCMeta, object)):
     if not driver_logging:
       return None
 
+    value_enum = (messages.LoggingConfig.DriverLogLevelsValue.
+                  AdditionalProperty.ValueValueValuesEnum)
+    config = collections.OrderedDict(
+        [(key, value_enum(value)) for key, value in driver_logging.items()])
     return messages.LoggingConfig(
-        driverLogLevels=encoding.DictToMessage(
-            driver_logging,
+        driverLogLevels=encoding.DictToAdditionalPropertyMessage(
+            config,
             messages.LoggingConfig.DriverLogLevelsValue))
 
   def PopulateFilesByType(self, args):
-    self.files_by_type.update(self.GetFilesByType(args))  # pytype: disable=attribute-error
+    self.files_by_type.update(self.GetFilesByType(args))
