@@ -19,21 +19,33 @@ from __future__ import absolute_import
 from __future__ import division
 from __future__ import unicode_literals
 
+from googlecloudsdk.calliope import arg_parsers
 from googlecloudsdk.calliope import base
 from googlecloudsdk.calliope.concepts import concepts
-from googlecloudsdk.command_lib.projects import resource_args as project_resource_args
 from googlecloudsdk.command_lib.util import completers
 from googlecloudsdk.command_lib.util.apis import arg_utils
 from googlecloudsdk.command_lib.util.concepts import concept_parsers
 
 
+class BetaKeyCompleter(completers.ListCommandCompleter):
+
+  def __init__(self, **kwargs):
+    super(BetaKeyCompleter, self).__init__(
+        collection='dns.dnsKeys',
+        api_version='v1beta2',
+        list_command=('beta dns dns-keys list --format=value(keyTag)'),
+        parse_output=True,
+        flags=['zone'],
+        **kwargs)
+
+
 class KeyCompleter(completers.ListCommandCompleter):
 
   def __init__(self, **kwargs):
     super(KeyCompleter, self).__init__(
         collection='dns.dnsKeys',
-        api_version='v2beta1',
-        list_command=('beta dns dns-keys list --format=value(keyTag)'),
+        api_version='v1',
+        list_command=('dns dns-keys list --format=value(keyTag)'),
         parse_output=True,
         flags=['zone'],
         **kwargs)
@@ -48,11 +60,11 @@ class ManagedZoneCompleter(completers.ListCommandCompleter):
         **kwargs)
 
 
-def GetKeyArg(help_text='The DNS key identifier.'):
+def GetKeyArg(help_text='The DNS key identifier.', is_beta=False):
   return base.Argument(
       'key_id',
       metavar='KEY-ID',
-      completer=KeyCompleter,
+      completer=BetaKeyCompleter if is_beta else KeyCompleter,
       help=help_text)
 
 
@@ -74,7 +86,7 @@ def GetZoneResourceSpec():
       'dns.managedZones',
       resource_name='zone',
       managedZone=ZoneAttributeConfig(),
-      project=project_resource_args.PROJECT_ATTRIBUTE_CONFIG,
+      project=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
       disable_auto_completers=False)
 
 
@@ -149,20 +161,20 @@ def GetDoeFlagMapper(messages):
       help_str='Requires DNSSEC enabled.')
 
 
+def GetKeyAlgorithmFlag(key_type, messages):
+  return arg_utils.ChoiceEnumMapper(
+      '--{}-algorithm'.format(key_type),
+      messages.DnsKeySpec.AlgorithmValueValuesEnum,
+      help_str='String mnemonic specifying the DNSSEC algorithm of the '
+               'key-signing key. Requires DNSSEC enabled')
+
+
 def AddCommonManagedZonesDnssecArgs(parser, messages):
   """Add Common DNSSEC flags for the managed-zones group."""
   GetDnsSecStateFlagMapper(messages).choice_arg.AddToParser(parser)
   GetDoeFlagMapper(messages).choice_arg.AddToParser(parser)
-  parser.add_argument(
-      '--ksk-algorithm',
-      help='String mnemonic specifying the DNSSEC algorithm of the '
-           'key-signing key. Requires DNSSEC enabled. Example algorithms: '
-           'RSASHA1, RSASHA256, RSASHA512, ECDSAP256SHA256, ECDSAP384SHA384')
-  parser.add_argument(
-      '--zsk-algorithm',
-      help='String mnemonic specifying the DNSSEC algorithm of the '
-           'zone-signing key. Requires DNSSEC enabled. Example algorithms: '
-           'RSASHA1, RSASHA256, RSASHA512, ECDSAP256SHA256, ECDSAP384SHA384')
+  GetKeyAlgorithmFlag('ksk', messages).choice_arg.AddToParser(parser)
+  GetKeyAlgorithmFlag('zsk', messages).choice_arg.AddToParser(parser)
   parser.add_argument(
       '--ksk-key-length',
       type=int,
@@ -173,4 +185,94 @@ def AddCommonManagedZonesDnssecArgs(parser, messages):
       help='Length of the zone-signing key in bits. Requires DNSSEC enabled.')
 
 
+def GetManagedZoneVisibilityArg():
+  return base.Argument(
+      '--visibility',
+      choices=['public', 'private'],
+      default='public',
+      help='Visibility of the zone. Public zones are visible to the public '
+      'internet. Private zones are only visible in your internal '
+      'networks denoted by the `--networks` flag.')
+
+
+def GetManagedZoneNetworksArg():
+  return base.Argument(
+      '--networks',
+      metavar='NETWORK',
+      type=arg_parsers.ArgList(),
+      help='List of networks that the zone should be visible in if the zone '
+      'visibility is [private].')
+
+
+def GetDnsPeeringArgs():
+  """Return arg group for DNS Peering flags."""
+  peering_group = base.ArgumentGroup(required=False)
+  target_network_help_text = (
+      'The network id of the Google Compute Engine private network to forward'
+      ' queries to.')
+  target_project_help_text = (
+      'The project id of the Google Compute Engine private network to forward'
+      ' queries to.')
+  peering_group.AddArgument(
+      base.Argument(
+          '--target-network',
+          required=True,
+          help=target_network_help_text))
+  peering_group.AddArgument(
+      base.Argument(
+          '--target-project',
+          required=True,
+          help=target_project_help_text))
+  return peering_group
+
+
+def GetForwardingTargetsArg():
+  return base.Argument(
+      '--forwarding-targets',
+      type=arg_parsers.ArgList(),
+      metavar='IP_ADDRESSES',
+      help=('List of IPv4 addresses of target name servers that the zone '
+            'will forward queries to. Ignored for `public` visibility.'))
+
+
+# Policy Flags
+def GetPolicyDescriptionArg():
+  return base.Argument(
+      '--description',
+      help='A description of the policy.')
+
+
+def GetPolicyNetworksArg():
+  return base.Argument(
+      '--networks',
+      type=arg_parsers.ArgList(),
+      metavar='NETWORKS',
+      help=('The comma separated list of network names to associate with '
+            'the policy.'))
+
+
+def GetPolicyInboundForwardingArg():
+  return base.Argument(
+      '--enable-inbound-forwarding',
+      action='store_true',
+      help=('Specifies whether to allow networks bound to this policy to '
+            'receive DNS queries sent by VMs or applications over VPN '
+            'connections. Defaults to False.'))
+
+
+def GetPolicyLoggingArg():
+  return base.Argument(
+      '--enable-logging',
+      action='store_true',
+      help='Specifies whether to enable query logging. Defaults to False.')
+
+
+def GetPolicyAltNameServersnArg():
+  return base.Argument(
+      '--alternative-name-servers',
+      type=arg_parsers.ArgList(),
+      metavar='NAME_SERVERS',
+      help=('List of alternative name servers to forward to. Must be a '
+            'comma separated list of IPv4 addresses.'))
+
 CHANGES_FORMAT = 'table(id, startTime, status)'
