@@ -31,7 +31,7 @@ Usage:
     command = <the next command line to run>
     try:
       command_exit_status = cosh.Run(command)
-    except coshell.CoshellExitException:
+    except coshell.CoshellExitError:
       break
   coshell_exit_status = cosh.Close()
 
@@ -56,6 +56,7 @@ import subprocess
 import six
 
 
+COSHELL_ENV = 'COSHELL'
 COSHELL_VERSION = '1.1'
 
 
@@ -302,11 +303,11 @@ __coshell_init_completions__
 """
 
 
-class CoshellExitException(Exception):
+class CoshellExitError(Exception):
   """The coshell exited."""
 
   def __init__(self, message, status=None):
-    super(CoshellExitException, self).__init__(message)
+    super(CoshellExitError, self).__init__(message)
     self.status = status
 
 
@@ -351,6 +352,20 @@ class _CoshellBase(six.with_metaclass(abc.ABCMeta, object)):
       status = 256 - status
     return status
 
+  def _Decode(self, data):
+    """Decodes external data if needed and returns internal string."""
+    try:
+      return data.decode(self._encoding)
+    except (AttributeError, UnicodeError):
+      return data
+
+  def _Encode(self, string):
+    """Encodes internal string if needed and returns external data."""
+    try:
+      return string.encode(self._encoding)
+    except UnicodeError:
+      return string
+
   def Close(self):
     """Closes the coshell connection and release any resources."""
     pass
@@ -422,9 +437,9 @@ class _UnixCoshellBase(six.with_metaclass(abc.ABCMeta, _CoshellBase)):
 
   def __init__(self):
     super(_UnixCoshellBase, self).__init__()
-    self.status = None  # type: int
-    self._status_fd = None  # type: int
-    self._shell = None  # type: subprocess.Popen
+    self.status = None
+    self._status_fd = None
+    self._shell = None
 
   @staticmethod
   def _Quote(command):
@@ -434,19 +449,31 @@ class _UnixCoshellBase(six.with_metaclass(abc.ABCMeta, _CoshellBase)):
   def _Exited(self):
     """Raises the coshell exit exception."""
     try:
-      self._shell.communicate(b':')
+      self._WriteLine(':')
     except (IOError, OSError, ValueError):
       # Yeah, ValueError for IO on a closed file.
       pass
     status = self._ShellStatus(self._shell.returncode)
-    raise CoshellExitException(
+    raise CoshellExitError(
         'The coshell exited [status={}].'.format(status),
         status=status)
 
+  def _ReadLine(self):
+    """Reads and returns a decoded stripped line from the coshell."""
+    return self._Decode(self._shell.stdout.readline()).strip()
+
+  def _ReadStatusChar(self):
+    """Reads and returns one encoded character from the coshell status fd."""
+    return os.read(self._status_fd, 1)
+
+  def _WriteLine(self, line):
+    """Writes an encoded line to the coshell."""
+    self._shell.communicate(self._Encode(line + '\n'))
+
   def _SendCommand(self, command):
     """Sends command to the coshell for execution."""
     try:
-      self._shell.stdin.write((command + '\n').encode(self._encoding))
+      self._shell.stdin.write(self._Encode(command + '\n'))
       self._shell.stdin.flush()
     except (IOError, OSError, ValueError):
       # Yeah, ValueError for IO on a closed file.
@@ -455,13 +482,14 @@ class _UnixCoshellBase(six.with_metaclass(abc.ABCMeta, _CoshellBase)):
   def _GetStatus(self):
     """Gets the status of the last command sent to the coshell."""
     line = []
+    shell_status_exit = self.SHELL_STATUS_EXIT.encode('ascii')
     while True:
-      c = os.read(self._status_fd, 1)
-      if c in (None, b'\n', self.SHELL_STATUS_EXIT.encode('ascii')):
+      c = self._ReadStatusChar()
+      if c in (None, b'\n', shell_status_exit):
         break
       line.append(c)
-    status_string = b''.join(line).decode(self._encoding)
-    if not status_string.isdigit() or c == self.SHELL_STATUS_EXIT:
+    status_string = self._Decode(b''.join(line))
+    if not status_string.isdigit() or c == shell_status_exit:
       self._Exited()
     return int(status_string)
 
@@ -610,8 +638,16 @@ class _UnixCoshell(_UnixCoshellBase):
     os.dup2(w, self.SHELL_STATUS_FD)
     os.close(w)
 
+    # Check for an alternate coshell command.
+
+    coshell_command_line = os.environ.get(COSHELL_ENV)
+    if coshell_command_line:
+      shell_command = coshell_command_line.split(' ')
+    else:
+      shell_command = [self.SHELL_PATH]
+
     self._shell = subprocess.Popen(
-        [self.SHELL_PATH],
+        shell_command,
         env=os.environ,  # NOTE: Needed to pass mocked environ to children.
         stdin=subprocess.PIPE,
         stdout=stdout,
@@ -638,7 +674,7 @@ class _UnixCoshell(_UnixCoshellBase):
       os.close(self._status_fd)
       self._status_fd = -1
     try:
-      self._shell.communicate(b'exit')  # This closes internal fds.
+      self._WriteLine('exit')  # This closes internal fds.
     except (IOError, ValueError):
       # Yeah, ValueError for IO on a closed file.
       pass
@@ -683,14 +719,14 @@ class _UnixCoshell(_UnixCoshellBase):
     line = []
     while True:
       try:
-        c = os.read(self._status_fd, 1)
+        c = self._ReadStatusChar()
       except (IOError, OSError, ValueError):
         # Yeah, ValueError for IO on a closed file.
         self._Exited()
       if c in (None, b'\n'):
         if not line:
           break
-        lines.append(b''.join(line).decode(self._encoding).rstrip())
+        lines.append(self._Decode(b''.join(line).rstrip()))
         line = []
       else:
         line.append(c)
@@ -732,7 +768,7 @@ class _MinGWCoshell(_UnixCoshellBase):
   def Close(self):
     """Closes the coshell connection and release any resources."""
     try:
-      self._shell.communicate(b'exit')  # This closes internal fds.
+      self._WriteLine('exit')  # This closes internal fds.
     except (IOError, ValueError):
       # Yeah, ValueError for IO on a closed file.
       pass
@@ -740,7 +776,7 @@ class _MinGWCoshell(_UnixCoshellBase):
 
   def _GetStatus(self):
     """Gets the status of the last command sent to the coshell."""
-    status_string = self._shell.stdout.readline().strip()
+    status_string = self._ReadLine()
     if status_string.endswith(self.SHELL_STATUS_EXIT):
       c = self.SHELL_STATUS_EXIT
       status_string = status_string[:-1]
@@ -788,18 +824,18 @@ class _MinGWCoshell(_UnixCoshellBase):
     lines = []
     while True:
       try:
-        line = self._shell.stdout.readline().rstrip('\n')
+        line = self._ReadLine()
       except (IOError, OSError, ValueError):
         # Yeah, ValueError for IO on a closed file.
         self._Exited()
       if not line:
         break
-      lines.append(line.decode(self._encoding))
+      lines.append(line)
     return lines
 
   def Interrupt(self):
     """Sends the interrupt signal to the coshell."""
-    self._shell.send_signal(signal.CTRL_C_EVENT)  # pytype: disable=module-attr
+    self._shell.send_signal(signal.CTRL_C_EVENT)
 
 
 class _WindowsCoshell(_CoshellBase):
@@ -824,7 +860,7 @@ class _WindowsCoshell(_CoshellBase):
 def _RunningOnWindows():
   """Lightweight mockable Windows check."""
   try:
-    return bool(WindowsError)  # pytype: disable=name-error
+    return bool(WindowsError)
   except NameError:
     return False
 
