@@ -19,9 +19,11 @@ from __future__ import absolute_import
 from __future__ import division
 from __future__ import unicode_literals
 
+from apitools.base.py import encoding
 from googlecloudsdk.api_lib.cloudiot import devices
 from googlecloudsdk.api_lib.cloudiot import registries
 from googlecloudsdk.command_lib.iot import flags
+from googlecloudsdk.command_lib.util.apis import arg_utils
 from googlecloudsdk.core import exceptions
 from googlecloudsdk.core import properties
 from googlecloudsdk.core import resources
@@ -38,7 +40,6 @@ DEVICES_COLLECTION = 'cloudiot.projects.locations.registries.devices'
 DEVICE_CONFIGS_COLLECTION = 'cloudiot.projects.locations.registries.devices.configVersions'
 _PROJECT = lambda: properties.VALUES.core.project.Get(required=True)
 
-
 # Maximum number of public key credentials for a device.
 MAX_PUBLIC_KEY_NUM = 3
 
@@ -54,6 +55,16 @@ MAX_METADATA_VALUE_SIZE = 1024 * 32
 # Maximum size of metadata keys and values (256 KB).
 MAX_METADATA_SIZE = 1024 * 256
 
+# Mapping of apitools request message fields to  their json parameters
+# TODO (b/124063772): Remove this mapping fix once apitools base fix is applied
+# pylint: disable=line-too-long, for readability.
+_CUSTOM_JSON_FIELD_MAPPINGS = {
+    'gatewayListOptions_gatewayType': 'gatewayListOptions.gatewayType',
+    'gatewayListOptions_associationsGatewayId': 'gatewayListOptions.associationsGatewayId',
+    'gatewayListOptions_associationsDeviceId': 'gatewayListOptions.associationsDeviceId',
+}
+# pylint: enable=line-too-long
+
 
 class InvalidPublicKeySpecificationError(exceptions.Error):
   """Indicates an issue with supplied public key credential(s)."""
@@ -74,6 +85,10 @@ class BadCredentialIndexError(exceptions.Error):
             resource=resource))
 
 
+class InvalidAuthMethodError(exceptions.Error):
+  """Indicates that auth method was provided for non-gateway device."""
+
+
 class BadDeviceError(exceptions.Error):
   """Indicates that a given device is malformed."""
 
@@ -112,6 +127,12 @@ def ParseEnableHttpConfig(enable_http_config, client=None):
     return http_config_enum.HTTP_DISABLED
 
 
+def ParseLogLevel(log_level, enum_class):
+  if log_level is None:
+    return None
+  return arg_utils.ChoiceToEnum(log_level, enum_class)
+
+
 def AddBlockedToRequest(ref, args, req):
   """Python hook for yaml commands to process the blocked flag."""
   del ref
@@ -137,8 +158,7 @@ def _ValidatePublicKeyDict(public_key):
 
 
 def _ConvertStringToFormatEnum(type_, messages):
-  # TODO(b/71388306): Enums aren't handled well by pytype.
-  # pytype: disable=attribute-error
+  """Convert string values to Enum object type."""
   if (type_ == flags.KeyTypes.RS256.choice_name or
       type_ == flags.KeyTypes.RSA_X509_PEM.choice_name):
     return messages.PublicKeyCredential.FormatValueValuesEnum.RSA_X509_PEM
@@ -149,7 +169,6 @@ def _ConvertStringToFormatEnum(type_, messages):
   elif (type_ == flags.KeyTypes.ES256.choice_name or
         type_ == flags.KeyTypes.ES256_PEM.choice_name):
     return messages.PublicKeyCredential.FormatValueValuesEnum.ES256_PEM
-  # pytype: enable=attribute-error
   else:
     # Should have been caught by argument parsing
     raise ValueError('Invalid key type [{}]'.format(type_))
@@ -398,17 +417,9 @@ def AddMetadataToRequest(ref, args, req):
   return req
 
 
-def ParseEventNotificationConfig(event_notification_configs, event_pubsub_topic,
-                                 messages=None):
+def ParseEventNotificationConfig(event_notification_configs, messages=None):
   """Creates a list of EventNotificationConfigs from args."""
   messages = messages or registries.GetMessagesModule()
-
-  # These flags are in a mutex group.
-  if event_pubsub_topic:
-    topic_ref = ParsePubsubTopic(event_pubsub_topic)
-    return [messages.EventNotificationConfig(
-        pubsubTopicName=topic_ref.RelativeName())]
-
   if event_notification_configs:
     configs = []
     for config in event_notification_configs:
@@ -423,7 +434,102 @@ def ParseEventNotificationConfig(event_notification_configs, event_pubsub_topic,
 def AddEventNotificationConfigsToRequest(ref, args, req):
   """Python hook for yaml commands to process event config flags."""
   del ref
-  configs = ParseEventNotificationConfig(args.event_notification_configs,
-                                         args.event_pubsub_topic)
+  configs = ParseEventNotificationConfig(args.event_notification_configs)
   req.deviceRegistry.eventNotificationConfigs = configs or []
   return req
+
+
+def AddCreateGatewayArgsToRequest(ref, args, req):
+  """Python hook for yaml create command to process gateway flags."""
+  del ref
+  gateway = args.device_type
+  auth_method = args.auth_method
+
+  # Don't set gateway config if no flags provided
+  if not (gateway or auth_method):
+    return req
+
+  messages = devices.GetMessagesModule()
+  req.device.gatewayConfig = messages.GatewayConfig()
+  if auth_method:
+    if not gateway or gateway == 'non-gateway':
+      raise InvalidAuthMethodError(
+          'auth_method can only be set on gateway devices.')
+    auth_enum = flags.GATEWAY_AUTH_METHOD_ENUM_MAPPER.GetEnumForChoice(
+        auth_method)
+    req.device.gatewayConfig.gatewayAuthMethod = auth_enum
+
+  if gateway:
+    gateway_enum = flags.CREATE_GATEWAY_ENUM_MAPPER.GetEnumForChoice(gateway)
+    req.device.gatewayConfig.gatewayType = gateway_enum
+
+  return req
+
+
+def AddBindArgsToRequest(ref, args, req):
+  """Python hook for yaml gateways bind command to process resource_args."""
+  del ref
+  messages = devices.GetMessagesModule()
+  gateway_ref = args.CONCEPTS.gateway.Parse()
+  device_ref = args.CONCEPTS.device.Parse()
+  registry_ref = gateway_ref.Parent()
+
+  bind_request = messages.BindDeviceToGatewayRequest(
+      deviceId=device_ref.Name(), gatewayId=gateway_ref.Name())
+  req.bindDeviceToGatewayRequest = bind_request
+  req.parent = registry_ref.RelativeName()
+
+  return req
+
+
+def AddUnBindArgsToRequest(ref, args, req):
+  """Python hook for yaml gateways unbind command to process resource_args."""
+  del ref
+  messages = devices.GetMessagesModule()
+  gateway_ref = args.CONCEPTS.gateway.Parse()
+  device_ref = args.CONCEPTS.device.Parse()
+  registry_ref = gateway_ref.Parent()
+
+  unbind_request = messages.UnbindDeviceFromGatewayRequest(
+      deviceId=device_ref.Name(), gatewayId=gateway_ref.Name())
+  req.unbindDeviceFromGatewayRequest = unbind_request
+  req.parent = registry_ref.RelativeName()
+
+  return req
+
+
+# TODO(b/124063772): Workaround for apitools issues with nested GET request
+# message fields.
+def RegistriesDevicesListRequestHook(ref, args, req):
+  """Add Api field query string mappings to list requests."""
+  del ref
+  del args
+  msg = devices.GetMessagesModule()
+  updated_requests_type = (
+      msg.CloudiotProjectsLocationsRegistriesDevicesListRequest)
+  for req_field, mapped_param in _CUSTOM_JSON_FIELD_MAPPINGS.items():
+    encoding.AddCustomJsonFieldMapping(updated_requests_type,
+                                       req_field,
+                                       mapped_param)
+  return req
+
+
+# Argument Processors
+def GetCommandFromFileProcessor(path):
+  """Builds a binary data for a SendCommandToDeviceRequest message from a path.
+
+  Args:
+    path: the path arg given to the command.
+
+  Raises:
+    ValueError: if the path does not exist or can not be read.
+
+  Returns:
+    binary data to be set on a message.
+  """
+  try:
+    return files.ReadBinaryFileContents(path)
+
+  except Exception as e:
+    raise ValueError('Command File [{}] can not be opened: {}'.format(path, e))
+
