@@ -20,7 +20,6 @@ from __future__ import unicode_literals
 
 from googlecloudsdk.core import properties
 from googlecloudsdk.core import resources
-from googlecloudsdk.core.console import console_io
 
 
 def Cancel(operations_client, operation):
@@ -31,17 +30,6 @@ def Cancel(operations_client, operation):
   return operations_client.Cancel(operation_ref)
 
 
-def Delete(operations_client, operation):
-  operation_ref = resources.REGISTRY.Parse(
-      operation,
-      params={'projectsId': properties.VALUES.core.project.GetOrFail},
-      collection='ml.projects.operations')
-  console_io.PromptContinue(
-      'This will delete operation [{}]...'.format(operation_ref.operationsId),
-      cancel_on_no=True)
-  return operations_client.Delete(operation_ref)
-
-
 def Describe(operations_client, operation):
   operation_ref = resources.REGISTRY.Parse(
       operation,
