@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2015 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,6 +15,10 @@
 
 """Common flags for projects commands."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.calliope import base
 from googlecloudsdk.command_lib.resource_manager import completers
 
@@ -24,3 +29,8 @@ def GetProjectFlag(verb):
       metavar='PROJECT_ID',
       completer=completers.ProjectCompleter,
       help='ID for the project you want to {0}.'.format(verb))
+
+
+CREATE_DELETE_IN_CONSOLE_SEE_ALSO = (
+    'See https://support.google.com/cloud/answer/6251787 for information on '
+    'creating or deleting projects from the Google Cloud Platform Console.')
