@@ -18,26 +18,12 @@ from __future__ import absolute_import
 from __future__ import division
 from __future__ import unicode_literals
 
-from googlecloudsdk.api_lib.app import region_util
-
 
 PROJECTS_COLLECTION = 'cloudtasks.projects'
 LOCATIONS_COLLECTION = 'cloudtasks.projects.locations'
 QUEUES_COLLECTION = 'cloudtasks.projects.locations.queues'
 TASKS_COLLECTION = 'cloudtasks.projects.locations.queues.tasks'
 
-# TODO(b/33038795): App Engine apps are currently being created with
-# multi-regional locations. Remove this map when this is no longer the case.
-CLOUD_MULTIREGION_TO_REGION_MAP = {
-    'us-central': 'us-central1',
-    'europe-west': 'europe-west1',
-}
-VALID_REGIONS = [
-    region_util.Region('us-central', True, True),
-    region_util.Region('europe-west', True, True),
-    region_util.Region('asia-northeast1', True, True)
-]
-
 PULL_QUEUE = 'pull'
 APP_ENGINE_QUEUE = 'app-engine'
 VALID_QUEUE_TYPES = [PULL_QUEUE, APP_ENGINE_QUEUE]
