@@ -26,10 +26,13 @@ from __future__ import division
 from __future__ import unicode_literals
 
 from apitools.base.protorpclite import messages as apitools_messages
+from apitools.base.py import encoding
+from apitools.base.py.exceptions import HttpBadRequestError
 from googlecloudsdk.api_lib.util import waiter
 from googlecloudsdk.calliope import base
 from googlecloudsdk.calliope import command_loading
 from googlecloudsdk.command_lib.iam import iam_util
+from googlecloudsdk.command_lib.util import completers
 from googlecloudsdk.command_lib.util.apis import arg_marshalling
 from googlecloudsdk.command_lib.util.apis import arg_utils
 from googlecloudsdk.command_lib.util.apis import registry
@@ -39,6 +42,7 @@ from googlecloudsdk.core import exceptions
 from googlecloudsdk.core import log
 from googlecloudsdk.core import resources
 from googlecloudsdk.core.console import console_io
+from googlecloudsdk.core.resource import resource_transform
 
 
 class Translator(command_loading.YamlCommandTranslator):
@@ -50,6 +54,38 @@ class Translator(command_loading.YamlCommandTranslator):
     return c.Generate()
 
 
+class DeclarativeIamRolesCompleter(completers.ListCommandCompleter):
+  """An IAM role completer for a resource argument.
+
+  The Complete() method override bypasses the completion cache.
+
+  Attributes:
+    _get_resource_ref: DeclarativeArgumentGenerator.GetRequestResourceRef method
+      to parse the resource ref.
+  """
+
+  def __init__(self, get_resource_ref, **kwargs):
+    super(DeclarativeIamRolesCompleter, self).__init__(**kwargs)
+    self._get_resource_ref = get_resource_ref
+
+  def GetListCommand(self, parameter_info):
+    resource_ref = self._get_resource_ref(parameter_info.parsed_args)
+    resource_uri = resource_ref.SelfLink()
+    return [
+        'iam', 'list-grantable-roles', '--quiet', '--flatten=name',
+        '--format=disable', resource_uri
+    ]
+
+  def Complete(self, prefix, parameter_info):
+    """Bypasses the cache and returns completions matching prefix."""
+    command = self.GetListCommand(parameter_info)
+    items = self.GetAllItems(command, parameter_info)
+    return [
+        item for item in items or []
+        if item is not None and item.startswith(prefix)
+    ]
+
+
 class CommandBuilder(object):
   """Generates calliope commands based on the yaml spec."""
 
@@ -66,7 +102,10 @@ class CommandBuilder(object):
         self.method,
         self.spec.arguments.params,
         resource_arg)
-    self.resource_type = resource_arg.name if resource_arg else None
+    self.display_resource_type = self.spec.request.display_resource_type
+    if (not self.display_resource_type
+        and resource_arg and not resource_arg.is_parent_resource):
+      self.display_resource_type = resource_arg.name if resource_arg else None
 
   def Generate(self):
     """Generates a calliope command from the yaml spec.
@@ -212,7 +251,7 @@ class CommandBuilder(object):
 
         response = self._HandleResponse(response, args)
         log.DeletedResource(self._GetDisplayName(ref, args),
-                            kind=self.resource_type)
+                            kind=self.display_resource_type)
         return response
 
     return Command
@@ -245,8 +284,10 @@ class CommandBuilder(object):
 
       def Run(self_, args):
         ref, response = self._CommonRun(args)
+        is_parent_resource = (self.spec.arguments.resource and
+                              self.spec.arguments.resource.is_parent_resource)
         if self.spec.async:
-          if ref is not None:
+          if ref is not None and not is_parent_resource:
             request_string = 'Create request issued for: [{{{}}}]'.format(
                 yaml_command_schema.NAME_FORMAT_KEY)
           else:
@@ -257,9 +298,16 @@ class CommandBuilder(object):
           if args.async:
             return self._HandleResponse(response, args)
 
+        if is_parent_resource:
+          # Data on responses from operation polling is stored in
+          # additionalProperties, so convert to dict for consistent behavior.
+          response_obj = encoding.MessageToDict(response)
+          full_name = response_obj.get('response', {}).get('name')
+          resource_name = resource_transform.TransformBaseName(full_name)
+        else:
+          resource_name = self._GetDisplayName(ref, args)
+        log.CreatedResource(resource_name, kind=self.display_resource_type)
         response = self._HandleResponse(response, args)
-        log.CreatedResource(self._GetDisplayName(ref, args),
-                            kind=self.resource_type)
         return response
 
     return Command
@@ -298,6 +346,10 @@ class CommandBuilder(object):
 
     return Command
 
+  @property
+  def _add_condition(self):
+    return self.spec.iam and self.spec.iam.enable_condition
+
   def _GenerateGetIamPolicyCommand(self):
     """Generates a get-iam-policy command.
 
@@ -375,18 +427,41 @@ class CommandBuilder(object):
 
         self.spec.request.static_fields[policy_field_path] = policy
         self._SetPolicyUpdateMask(update_mask)
-        ref, response = self._CommonRun(args)
-        iam_util.LogSetIamPolicy(ref.Name(), self.resource_type)
+        try:
+          ref, response = self._CommonRun(args)
+        except HttpBadRequestError as ex:
+          log.err.Print(
+              'ERROR: Policy modification failed. For bindings with conditions'
+              ', run "gcloud alpha iam policies lint-condition" to identify '
+              'issues in conditions.'
+          )
+          raise ex
+
+        iam_util.LogSetIamPolicy(ref.Name(), self.display_resource_type)
         return self._HandleResponse(response, args)
 
     return Command
 
+  def _GenerateDeclarativeIamRolesCompleter(self):
+    """Generate a IAM role completer."""
+
+    get_resource_ref = self.arg_generator.GetRequestResourceRef
+
+    class Completer(DeclarativeIamRolesCompleter):
+
+      def __init__(self, **kwargs):
+        super(Completer, self).__init__(
+            get_resource_ref=get_resource_ref, **kwargs)
+
+    return Completer
+
   def _GenerateAddIamPolicyBindingCommand(self):
     """Generates an add-iam-policy-binding command.
 
-    An add-iam-policy-binding command takes a resource argument, a member to add
-    the binding for, a role to define the role of the member, and two API
-    methods to get and set the policy on the resource.
+    An add-iam-policy-binding command adds a binding to a IAM policy. A
+    binding consists of a member, a role to define the role of the member, and
+    an optional condition to define in what condition the binding is valid.
+    Two API methods are called to get and set the policy on the resource.
 
     Returns:
       calliope.base.Command, The command that implements the spec.
@@ -401,8 +476,11 @@ class CommandBuilder(object):
 
       @staticmethod
       def Args(parser):
+        iam_util.AddArgsForAddIamPolicyBinding(
+            parser,
+            role_completer=self._GenerateDeclarativeIamRolesCompleter(),
+            add_condition=self._add_condition)
         self._CommonArgs(parser)
-        iam_util.AddArgsForAddIamPolicyBinding(parser)
         base.URI_FLAG.RemoveFromParser(parser)
 
       def Run(self_, args):
@@ -415,11 +493,21 @@ class CommandBuilder(object):
               self.spec.iam.set_iam_policy_request_path or policy_request_path)
         policy_field_path = policy_request_path + '.policy'
 
-        policy = self._GetModifiedIamPolicy(args, 'add')
+        policy = self._GetModifiedIamPolicyAddIamBinding(
+            args, add_condition=self._add_condition)
         self.spec.request.static_fields[policy_field_path] = policy
 
-        ref, response = self._CommonRun(args)
-        iam_util.LogSetIamPolicy(ref.Name(), self.resource_type)
+        try:
+          ref, response = self._CommonRun(args)
+        except HttpBadRequestError as ex:
+          log.err.Print(
+              'ERROR: Policy modification failed. For a binding with condition'
+              ', run "gcloud alpha iam policies lint-condition" to identify '
+              'issues in condition.'
+          )
+          raise ex
+
+        iam_util.LogSetIamPolicy(ref.Name(), self.display_resource_type)
         return self._HandleResponse(response, args)
 
     return Command
@@ -427,9 +515,10 @@ class CommandBuilder(object):
   def _GenerateRemoveIamPolicyBindingCommand(self):
     """Generates a remove-iam-policy-binding command.
 
-    A remove-iam-policy-binding command takes a resource argument, a member,
-    a role to remove the member from, and two API methods to get and set the
-    policy on the resource.
+    A remove-iam-policy-binding command removes a binding from a IAM policy. A
+    binding consists of a member, a role to define the role of the member, and
+    an optional condition to define in what condition the binding is valid.
+    Two API methods are called to get and set the policy on the resource.
 
     Returns:
       calliope.base.Command, The command that implements the spec.
@@ -444,8 +533,11 @@ class CommandBuilder(object):
 
       @staticmethod
       def Args(parser):
+        iam_util.AddArgsForRemoveIamPolicyBinding(
+            parser,
+            role_completer=self._GenerateDeclarativeIamRolesCompleter(),
+            add_condition=self._add_condition)
         self._CommonArgs(parser)
-        iam_util.AddArgsForRemoveIamPolicyBinding(parser)
         base.URI_FLAG.RemoveFromParser(parser)
 
       def Run(self_, args):
@@ -458,11 +550,12 @@ class CommandBuilder(object):
               self.spec.iam.set_iam_policy_request_path or policy_request_path)
         policy_field_path = policy_request_path + '.policy'
 
-        policy = self._GetModifiedIamPolicy(args, 'remove')
+        policy = self._GetModifiedIamPolicyRemoveIamBinding(
+            args, add_condition=self._add_condition)
         self.spec.request.static_fields[policy_field_path] = policy
 
         ref, response = self._CommonRun(args)
-        iam_util.LogSetIamPolicy(ref.Name(), self.resource_type)
+        iam_util.LogSetIamPolicy(ref.Name(), self.display_resource_type)
         return self._HandleResponse(response, args)
 
     return Command
@@ -533,10 +626,16 @@ class CommandBuilder(object):
           base.ASYNC_FLAG.AddToParser(parser)
 
       def Run(self_, args):
-        # Check if mask is required for an update request.
+        # Check if mask is required for an update request, if required, return
+        # the dotted path, e.g updateRequest.fieldMask.
         mask_path = update.GetMaskFieldPath(self.method)
         if mask_path:
-          mask_string = update.GetMaskString(args, self.spec, mask_path)
+          # If user sets to disable the auto-generated field mask, set the value
+          # to the empty string instead so that custom hooks can be used.
+          if self.spec.update and self.spec.update.disable_auto_field_mask:
+            mask_string = ''
+          else:
+            mask_string = update.GetMaskString(args, self.spec, mask_path)
           self.spec.request.static_fields[mask_path] = mask_string
 
         # Check if the update is full-update, which requires a get request.
@@ -555,7 +654,7 @@ class CommandBuilder(object):
               args, ref, response, request_string=request_string)
 
         log.UpdatedResource(
-            self._GetDisplayName(ref, args), kind=self.resource_type)
+            self._GetDisplayName(ref, args), kind=self.display_resource_type)
         return self._HandleResponse(response, args)
 
     return Command
@@ -600,6 +699,12 @@ class CommandBuilder(object):
                        self._GetDisplayName(ref, args)),
           throw_if_unattended=True, cancel_on_no=True)
 
+    if self.spec.request.modify_method_hook:
+      self.spec.request.method = self.spec.request.modify_method_hook(ref, args)
+      self.method = registry.GetMethod(
+          self.spec.request.collection, self.spec.request.method,
+          self.spec.request.api_version)
+
     if self.spec.request.issue_request_hook:
       # Making the request is overridden, just call into the custom code.
       return ref, self.spec.request.issue_request_hook(ref, args)
@@ -615,7 +720,8 @@ class CommandBuilder(object):
           self.spec.request.resource_method_params,
           use_relative_name=self.spec.request.use_relative_name,
           parse_resource_into_request=parse_resource,
-          existing_message=existing_message)
+          existing_message=existing_message,
+          override_method=self.method)
       for hook in self.spec.request.modify_request_hooks:
         request = hook(ref, args, request)
 
@@ -653,20 +759,7 @@ class CommandBuilder(object):
     if hasattr(update_request, 'updateMask'):
       self.spec.request.static_fields[mask_field_path] = update_mask
 
-  def _GetModifiedIamPolicy(self, args, policy_binding_type):
-    """Get the current IAM policy and then add/remove bindings as specified.
-
-    An IAM binding is a pair of role and member. If policy_binding_type is add,
-    the member and role specified in args would be added; if policy_binding_type
-    is remove, the member and role specified in args would be removed.
-
-    Args:
-      args: The argparse parser.
-      policy_binding_type: string, add or remove.
-
-    Returns:
-      IAM policy.
-    """
+  def _GetIamPolicy(self, args):
     get_iam_method = registry.GetMethod(self.spec.request.collection,
                                         'getIamPolicy',
                                         self.spec.request.api_version)
@@ -675,15 +768,50 @@ class CommandBuilder(object):
         use_relative_name=self.spec.request.use_relative_name,
         override_method=get_iam_method)
     policy = get_iam_method.Call(get_iam_request)
+    return policy
 
-    if policy_binding_type == 'add':
-      binding = self.method.GetMessageByName('Binding')
-      iam_util.AddBindingToIamPolicy(binding, policy, args.member, args.role)
-    elif policy_binding_type == 'remove':
-      iam_util.RemoveBindingFromIamPolicy(policy, args.member, args.role)
+  def _GetModifiedIamPolicyAddIamBinding(self, args, add_condition=False):
+    """Get the IAM policy and add the specified binding to it.
+
+    Args:
+      args: an argparse namespace.
+      add_condition: True if support condition.
+
+    Returns:
+      IAM policy.
+    """
+    binding_message_type = self.method.GetMessageByName('Binding')
+    if add_condition:
+      condition = iam_util.ValidateAndExtractConditionMutexRole(args)
+      policy = self._GetIamPolicy(args)
+      condition_message_type = self.method.GetMessageByName('Expr')
+      iam_util.AddBindingToIamPolicyWithCondition(
+          binding_message_type, condition_message_type, policy, args.member,
+          args.role, condition)
     else:
-      pass
+      policy = self._GetIamPolicy(args)
+      iam_util.AddBindingToIamPolicy(binding_message_type, policy, args.member,
+                                     args.role)
+    return policy
 
+  def _GetModifiedIamPolicyRemoveIamBinding(self, args, add_condition=False):
+    """Get the IAM policy and remove the specified binding to it.
+
+    Args:
+      args: an argparse namespace.
+      add_condition: True if support condition.
+
+    Returns:
+      IAM policy.
+    """
+    if add_condition:
+      condition = iam_util.ValidateAndExtractCondition(args)
+      policy = self._GetIamPolicy(args)
+      iam_util.RemoveBindingFromIamPolicyWithCondition(
+          policy, args.member, args.role, condition, all_conditions=args.all)
+    else:
+      policy = self._GetIamPolicy(args)
+      iam_util.RemoveBindingFromIamPolicy(policy, args.member, args.role)
     return policy
 
   def _GetExistingResource(self, args):
@@ -825,7 +953,7 @@ class CommandBuilder(object):
       d[yaml_command_schema.REL_NAME_FORMAT_KEY] = resource_ref.RelativeName()
     else:
       d = {yaml_command_schema.NAME_FORMAT_KEY: display_name}
-    d[yaml_command_schema.RESOURCE_TYPE_FORMAT_KEY] = self.resource_type
+    d[yaml_command_schema.RESOURCE_TYPE_FORMAT_KEY] = self.display_resource_type
     return format_string.format(**d)
 
   def _RegisterURIFunc(self, args):
@@ -856,7 +984,12 @@ class CommandBuilder(object):
     if self.spec.release_tracks:
       command = base.ReleaseTracks(*self.spec.release_tracks)(command)
     if not hasattr(command, 'detailed_help'):
-      command.detailed_help = self.spec.help_text
+      key_map = {
+          'description': 'DESCRIPTION',
+          'examples': 'EXAMPLES',
+      }
+      command.detailed_help = {
+          key_map.get(k, k): v for k, v in self.spec.help_text.items()}
     command.detailed_help['API REFERENCE'] = (
         'This command uses the *{}/{}* API. The full documentation for this '
         'API can be found at: {}'.format(
