@@ -23,9 +23,11 @@ import errno
 import getpass
 import os
 import re
+import string
 import enum
 
 from googlecloudsdk.api_lib.oslogin import client as oslogin_client
+from googlecloudsdk.command_lib.oslogin import oslogin_utils
 from googlecloudsdk.command_lib.util import gaia
 from googlecloudsdk.core import config
 from googlecloudsdk.core import exceptions as core_exceptions
@@ -81,6 +83,10 @@ class InvalidConfigurationError(core_exceptions.Error):
         .format(sources, destination))
 
 
+class BadCharacterError(core_exceptions.Error):
+  """Indicates a character was found that couldn't be escaped."""
+
+
 class Suite(enum.Enum):
   """Represents an SSH implementation suite."""
   OPENSSH = 'OpenSSH'
@@ -142,7 +148,6 @@ class Environment(object):
     """
     self.suite = suite
     self.bin_path = bin_path
-    # So pytype is aware of attributes.
     self.ssh = None
     self.ssh_term = None
     self.scp = None
@@ -304,14 +309,13 @@ class Keys(object):
           expansion.
       env: Environment, Current environment or None to infer from current.
     """
-    private_key_file = os.path.realpath(os.path.expanduser(key_file))
+    private_key_file = os.path.realpath(files.ExpandHomeDir(key_file))
     self.dir = os.path.dirname(private_key_file)
     self.env = env or Environment.Current()
-    # TODO(b/71388306): Enums aren't handled well by pytype.
     self.keys = {
         _KeyFileKind.PRIVATE: self.KeyFileData(private_key_file),
         _KeyFileKind.PUBLIC: self.KeyFileData(private_key_file + '.pub')
-    }  # type: dict[enum.Enum, Keys.KeyFileData]
+    }
     if self.env.suite is Suite.PUTTY:
       self.keys[_KeyFileKind.PPK] = self.KeyFileData(private_key_file + '.ppk')
 
@@ -340,13 +344,13 @@ class Keys(object):
     status_padding = 0
     for kind in self.keys:
       data = self.keys[kind]
-      key_padding = max(key_padding, len(kind.value))  # pytype: disable=attribute-error
-      status_padding = max(status_padding, len(data.status.value))  # pytype: disable=attribute-error
+      key_padding = max(key_padding, len(kind.value))
+      status_padding = max(status_padding, len(data.status.value))
     for kind in self.keys:
       data = self.keys[kind]
       messages.append('{} {} [{}]\n'.format(
-          (kind.value + ' key').ljust(key_padding + 4),  # pytype: disable=attribute-error
-          ('(' + data.status.value + ')') .ljust(status_padding + 2),  # pytype: disable=attribute-error
+          (kind.value + ' key').ljust(key_padding + 4),
+          ('(' + data.status.value + ')') .ljust(status_padding + 2),
           data.filename))
     messages.sort()
     return ''.join(messages)
@@ -365,7 +369,7 @@ class Keys(object):
     """
     def ValidateFile(kind):
       status_or_line = self._WarnOrReadFirstKeyLine(self.keys[kind].filename,
-                                                    kind.value)  # pytype: disable=attribute-error
+                                                    kind.value)
       if isinstance(status_or_line, KeyFileStatus):
         return status_or_line
       else:  # returned line - present
@@ -466,7 +470,6 @@ class Keys(object):
     return status
 
   def GetPublicKey(self):
-    # type: () -> Keys.PublicKey
     """Returns the public key verbatim from file as a string.
 
     Precondition: The public key must exist. Run Keys.EnsureKeysExist() prior.
@@ -530,7 +533,7 @@ class KnownHosts(object):
   """
 
   # TODO(b/33467618): Rename the file itself
-  DEFAULT_PATH = os.path.realpath(os.path.expanduser(
+  DEFAULT_PATH = os.path.realpath(files.ExpandHomeDir(
       os.path.join('~', '.ssh', 'google_compute_known_hosts')))
 
   def __init__(self, known_hosts, file_path):
@@ -670,7 +673,7 @@ def CheckForOsloginAndGetUser(instance, project, requested_user, public_key,
 
   Args:
     instance: instance, The object representing the instance we are
-      connecting to.
+      connecting to. If None, instance metadata will be ignored.
     project: project, The object representing the current project.
     requested_user: str, The default or requested username to connect as.
     public_key: str, The public key of the user connecting.
@@ -682,7 +685,9 @@ def CheckForOsloginAndGetUser(instance, project, requested_user, public_key,
   """
   # Instance metadata has priority
   use_oslogin = False
-  oslogin_enabled = _MetadataHasOsloginEnable(instance.metadata)
+  oslogin_enabled = None
+  if instance is not None:
+    oslogin_enabled = _MetadataHasOsloginEnable(instance.metadata)
   if oslogin_enabled is None:
     project_metadata = project.commonInstanceMetadata
     oslogin_enabled = _MetadataHasOsloginEnable(project_metadata)
@@ -698,14 +703,22 @@ def CheckForOsloginAndGetUser(instance, project, requested_user, public_key,
         'in the {0} version of gcloud.'.format(release_track.id))
     return requested_user, use_oslogin
   user_email = properties.VALUES.core.account.Get()
-  login_profile = oslogin.ImportSshPublicKey(user_email, public_key)
+
+  # Check to see if public key is already in profile, and import if not.
+  login_profile = oslogin.GetLoginProfile(user_email, project.name)
+  keys = oslogin_utils.GetKeyDictionaryFromProfile(
+      user_email, oslogin, profile=login_profile)
+  fingerprint = oslogin_utils.FindKeyInKeyList(public_key, keys)
+  if not fingerprint:
+    import_response = oslogin.ImportSshPublicKey(user_email, public_key)
+    login_profile = import_response.loginProfile
   use_oslogin = True
 
   # Get the username for the oslogin user. If the username is the same as the
   # default user, return that one. Otherwise, return the 'primary' username.
   # If no 'primary' exists, return the first username.
   oslogin_user = None
-  for pa in login_profile.loginProfile.posixAccounts:
+  for pa in login_profile.posixAccounts:
     oslogin_user = oslogin_user or pa.username
     if pa.username == requested_user:
       return requested_user, use_oslogin
@@ -806,6 +819,148 @@ class Remote(object):
     return self.ToArg()
 
 
+def _EscapeProxyCommandArg(s, env):
+  """Returns s escaped such that it can be a ProxyCommand arg.
+
+  Args:
+    s: str, Argument to escape. Must be non-empty.
+    env: Environment, data about the ssh client.
+  Raises:
+    BadCharacterError: If s contains a bad character.
+  """
+  for c in s:
+    if not 0x20 <= ord(c) < 0x7f:
+      # For ease of implementation we ban control characters and non-ASCII.
+      raise BadCharacterError(
+          ('Special character %r (part of %r) couldn\'t be escaped for '
+           'ProxyCommand') % (c, s))
+  if env.suite is Suite.PUTTY:
+    # When using proxycmd with putty or plink, 3 unescapes happen:
+    # 1 putty/plink does command line -> argv unescape.
+    # 2 putty/plink does backslash and percent unescape.
+    # 3 Inner gcloud python binary does command line -> argv unescape.
+    #
+    # We reverse this, doing escapes in reverse order, doing 3, 2.
+    # We don't do the 1 escape here because that's done later inside the
+    # subprocess.Popen() function.
+    s = _EscapeWindowsArgvElement(s)
+    s = _EscapePuttyBackslashPercent(s)
+    return s
+  # When using ProxyCommand with OpenSSH, 2 unescapes happen:
+  # 1 OpenSSH does percent unescape.
+  # 2 bash does unescape.
+  # We do the corresponding escapes in reverse.
+  return _EscapeForBash(s).replace('%', '%%')
+
+
+def _EscapeWindowsArgvElement(s):
+  """Returns s escaped such that it can be passed to a windows executable.
+
+  Args:
+    s: str, What to escape. Must be ASCII and non-control.
+  """
+  # Each Windows binary can unescape its commandline arguments to argv how it
+  # wants, but they tend to behave like this:
+  # https://docs.microsoft.com/en-us/cpp/cpp/parsing-cpp-command-line-arguments?view=vs-2017
+  # We escape in that format because that format is similar to how python (of
+  # the inner gcloud) does it. The primary difference is what happens when
+  # inside a doublequoted string there is an even number of backslashes
+  # (possibly 0) then at least 2 doublequotes. This function never returns a
+  # string like that, so it avoids the ambiguity.
+  #
+  # We escape in reverse, because that's easiest.
+  result = []
+  # Whether (in the reversed input) we are following a non-broken chain of
+  # backslashes (possibly 0-length) after a doublequote.
+  # The final output will have a doublequote appended to the end, so the first
+  # character of the reversed input is considered to follow a doublequote.
+  following_quote = True
+  for c in s[::-1]:
+    if c == '"':
+      result.append('"\\')
+      following_quote = True
+    elif c == '\\':
+      if following_quote:
+        result.append('\\\\')
+      else:
+        result.append('\\')
+    else:
+      result.append(c)
+      following_quote = False
+  return '"' + ''.join(result)[::-1] + '"'
+
+
+def _EscapePuttyBackslashPercent(s):
+  # s must be ASCII and non-control.
+  # The putty unescaping is documented at
+  # https://the.earth.li/~sgtatham/putty/0.70/htmldoc/Chapter4.html#config-proxy-command
+  return s.replace('\\', '\\\\').replace('%', '%%')
+
+
+def _EscapeForBash(s):
+  """Returns s escaped so it can be used as a single bash argument.
+
+  Args:
+    s: str, What to escape. Must be ASCII, non-control, and non-empty.
+  """
+  # From https://stackoverflow.com/q/15783701
+  good_chars = set(string.ascii_letters + string.digits + '%+-./:=@_')
+  result = []
+  for c in s:
+    if c in good_chars:
+      result.append(c)
+    else:
+      result.append('\\' + c)
+  return ''.join(result)
+
+
+def _BuildIapTunnelProxyCommandArgs(iap_tunnel_args, env):
+  """Calculate the ProxyCommand flags for IAP Tunnel if necessary.
+
+  IAP Tunnel with ssh runs an second inner version of gcloud by passing a
+  command to do so as a ProxyCommand argument to OpenSSH/Putty.
+
+  Args:
+    iap_tunnel_args: iap_tunnel.SshTunnelArgs or None, options about IAP Tunnel.
+    env: Environment, data about the ssh client.
+  Returns:
+    [str], the additional arguments for OpenSSH or Putty.
+  """
+  if not iap_tunnel_args:
+    return []
+
+  gcloud_command = execution_utils.ArgsForGcloud()
+  # Applying _EscapeProxyCommandArg to the first item (the python executable
+  # path) doesn't make 100% sense on Windows, because the full unescaping only
+  # happens to arguments, not to the executable path. But this escaping will be
+  # correct as long as the python executable path doesn't contain a doublequote
+  # or end with a backslash, which should never happen.
+  gcloud_command = [_EscapeProxyCommandArg(x, env) for x in gcloud_command]
+  # track, project, zone, instance, interface, verbosity should only contain
+  # characters that don't need escaping, so don't bother escaping them.
+  if iap_tunnel_args.track:
+    gcloud_command.append(iap_tunnel_args.track)
+  port_token = '%port' if env.suite is Suite.PUTTY else '%p'
+  gcloud_command.extend([
+      'compute', 'start-iap-tunnel', iap_tunnel_args.instance, port_token,
+      '--listen-on-stdin',
+      '--project=' + iap_tunnel_args.project,
+      '--zone=' + iap_tunnel_args.zone,
+      '--network-interface=' + iap_tunnel_args.interface])
+  for arg in iap_tunnel_args.pass_through_args:
+    gcloud_command.append(_EscapeProxyCommandArg(arg, env))
+
+  verbosity = log.GetVerbosityName()
+  if verbosity:
+    gcloud_command.append('--verbosity=' + verbosity)
+
+  if env.suite is Suite.PUTTY:
+    return ['-proxycmd', ' '.join(gcloud_command)]
+  else:
+    return ['-o', ' '.join(['ProxyCommand'] + gcloud_command),
+            '-o', 'ProxyUseFdpass=no']
+
+
 class KeygenCommand(object):
   """Platform independent SSH client key generation command.
 
@@ -900,7 +1055,7 @@ class SSHCommand(object):
 
   def __init__(self, remote, port=None, identity_file=None,
                options=None, extra_flags=None, remote_command=None, tty=None,
-               remainder=None):
+               iap_tunnel_args=None, remainder=None):
     """Construct a suite independent SSH command.
 
     Note that `extra_flags` and `remote_command` arguments are lists of strings:
@@ -910,7 +1065,7 @@ class SSHCommand(object):
 
     Args:
       remote: Remote, the remote to connect to.
-      port: int, port.
+      port: str, port.
       identity_file: str, path to private key file.
       options: {str: str}, options (`-o`) for OpenSSH, see `ssh_config(5)`.
       extra_flags: [str], extra flags to append to ssh invocation. Both binary
@@ -918,6 +1073,8 @@ class SSHCommand(object):
       remote_command: [str], command to run remotely.
       tty: bool, launch a terminal. If None, determine automatically based on
         presence of remote command.
+      iap_tunnel_args: iap_tunnel.SshTunnelArgs or None, options about IAP
+        Tunnel.
       remainder: [str], NOT RECOMMENDED. Arguments to be appended directly to
         the native tool invocation, after the `[user@]host` part but prior to
         the remote command. On PuTTY, this can only be a remote command. On
@@ -932,6 +1089,7 @@ class SSHCommand(object):
     self.extra_flags = extra_flags or []
     self.remote_command = remote_command or []
     self.tty = tty
+    self.iap_tunnel_args = iap_tunnel_args
     self.remainder = remainder
 
   def Build(self, env=None):
@@ -967,6 +1125,8 @@ class SSHCommand(object):
       # Always, always deterministic order
       for key, value in sorted(six.iteritems(self.options)):
         args.extend(['-o', '{k}={v}'.format(k=key, v=value)])
+
+    args.extend(_BuildIapTunnelProxyCommandArgs(self.iap_tunnel_args, env))
     args.extend(self.extra_flags)
     args.append(self.remote.ToArg())
 
@@ -1046,7 +1206,8 @@ class SCPCommand(object):
   """
 
   def __init__(self, sources, destination, recursive=False, compress=False,
-               port=None, identity_file=None, options=None, extra_flags=None):
+               port=None, identity_file=None, options=None, extra_flags=None,
+               iap_tunnel_args=None):
     """Construct a suite independent SCP command.
 
     Args:
@@ -1057,11 +1218,13 @@ class SCPCommand(object):
         source, this must be local, and vice versa.
       recursive: bool, recursive directory copy.
       compress: bool, enable compression.
-      port: int, port.
+      port: str, port.
       identity_file: str, path to private key file.
       options: {str: str}, options (`-o`) for OpenSSH, see `ssh_config(5)`.
       extra_flags: [str], extra flags to append to scp invocation. Both binary
         style flags `['-b']` and flags with values `['-k', 'v']` are accepted.
+      iap_tunnel_args: iap_tunnel.SshTunnelArgs or None, options about IAP
+        Tunnel.
     """
     self.sources = [sources] if isinstance(sources, FileReference) else sources
     self.destination = destination
@@ -1071,6 +1234,7 @@ class SCPCommand(object):
     self.identity_file = identity_file
     self.options = options or {}
     self.extra_flags = extra_flags or []
+    self.iap_tunnel_args = iap_tunnel_args
 
   @classmethod
   def Verify(cls, sources, destination, single_remote=False, env=None):
@@ -1159,6 +1323,7 @@ class SCPCommand(object):
       for key, value in sorted(six.iteritems(self.options)):
         args.extend(['-o', '{k}={v}'.format(k=key, v=value)])
 
+    args.extend(_BuildIapTunnelProxyCommandArgs(self.iap_tunnel_args, env))
     args.extend(self.extra_flags)
 
     # Positionals
@@ -1205,22 +1370,25 @@ class SSHPoller(object):
 
   def __init__(self, remote, port=None, identity_file=None,
                options=None, extra_flags=None, max_wait_ms=60*1000,
-               sleep_ms=5*1000):
+               sleep_ms=5*1000, iap_tunnel_args=None):
     """Construct a poller for an SSH connection.
 
     Args:
       remote: Remote, the remote to poll.
-      port: int, port to poll.
+      port: str, port to poll.
       identity_file: str, path to private key file.
       options: {str: str}, options (`-o`) for OpenSSH, see `ssh_config(5)`.
       extra_flags: [str], extra flags to append to ssh invocation. Both binary
         style flags `['-b']` and flags with values `['-k', 'v']` are accepted.
       max_wait_ms: int, number of ms to wait before raising.
       sleep_ms: int, time between trials.
+      iap_tunnel_args: iap_tunnel.SshTunnelArgs or None, information about IAP
+        Tunnel.
     """
     self.ssh_command = SSHCommand(
         remote, port=port, identity_file=identity_file, options=options,
-        extra_flags=extra_flags, remote_command=['true'], tty=False)
+        extra_flags=extra_flags, remote_command=['true'], tty=False,
+        iap_tunnel_args=iap_tunnel_args)
     self._sleep_ms = sleep_ms
     self._retryer = retry.Retryer(max_wait_ms=max_wait_ms, jitter_ms=0)
 
